@@ -180,17 +180,17 @@ If PATTERN is omitted, it defaults to \"[ \\f\\t\\n\\r\\v]+\"."
 
             An Interactive Plotting Program
              Thomas Williams & Colin Kelley
-                Version 4.6 organized by:
-    Hans-Bernhard Broeker, Ethan A Merritt, and others
+                Version 5.0 organized by
+       Ethan A Merritt with help from many others
 
    Copyright (C) 1986 - 1993, 1998, 2004   Thomas Williams, Colin Kelley
-           Copyright (C) 2004 - 2011  various authors
+           Copyright (C) 2004 - 2012  various authors
 
        Mailing list for comments: gnuplot-info@@lists.sourceforge.net
      Mailing list for bug reports: gnuplot-bugs@@lists.sourceforge.net
 
          This manual was originally prepared by Dick Crawford
-                   Version 4.6 - December 2011
+                   Version 5.0 - March 2014
 
 
 Major contributors (alphabetic order):
@@ -260,7 +260,6 @@ Major contributors (alphabetic order):
 	"be"
 	"cairo"
 	"canvas"
-	"cgi"
 	"cgm"
 	"context"
 	"corel"
@@ -321,7 +320,6 @@ Major contributors (alphabetic order):
 	"tikz"
 	"tkcanvas"
 	"tpic"
-	"unixpc"
 	"v384"
 	"vgagl"
 	"vws"
@@ -461,7 +459,6 @@ particular conversion chore."
     ("windows" "win")
     ("next" "next")
     ("os2" "pm" "emxvga")
-    ("sco" "cgi")
     ("sun" "sun")
     ("vms" "vws"))
 
