@@ -1,5 +1,5 @@
 /*
- * $Id: command.h,v 1.70 2015/08/08 18:32:17 sfeam Exp $
+ * $Id: command.h,v 1.66 2014/03/30 18:33:21 markisch Exp $
  */
 
 /* GNUPLOT - command.h */
@@ -101,6 +101,11 @@ extern char HelpFile[];         /* patch for do_help  - AP */
 # define SET_CURSOR_ARROW       /* nought, zilch */
 #endif
 
+/* wrapper for calling kill_pending_Pause_dialog() from win/winmain.c */
+#ifdef _Windows
+void call_kill_pending_Pause_dialog(void);
+#endif
+
 /* Include code to support deprecated "call" syntax. */
 #ifdef BACKWARDS_COMPATIBLE
 #define OLD_STYLE_CALL_ARGS
@@ -142,12 +147,11 @@ void restore_prompt __PROTO((void));
 #else
 #define bind_command()
 #endif
-void array_command __PROTO((void));
-void break_command __PROTO((void));
+void refresh_request __PROTO((void));
+void refresh_command __PROTO((void));
 void call_command __PROTO((void));
 void changedir_command __PROTO((void));
 void clear_command __PROTO((void));
-void continue_command __PROTO((void));
 void eval_command __PROTO((void));
 void exit_command __PROTO((void));
 void help_command __PROTO((void));
@@ -166,10 +170,7 @@ void null_command __PROTO((void));
 void pause_command __PROTO((void));
 void plot_command __PROTO((void));
 void print_command __PROTO((void));
-void printerr_command __PROTO((void));
 void pwd_command __PROTO((void));
-void refresh_request __PROTO((void));
-void refresh_command __PROTO((void));
 void replot_command __PROTO((void));
 void reread_command __PROTO((void));
 void save_command __PROTO((void));
@@ -178,7 +179,6 @@ void splot_command __PROTO((void));
 void stats_command __PROTO((void));
 void system_command __PROTO((void));
 void test_command __PROTO((void));
-void toggle_command __PROTO((void));
 void update_command __PROTO((void));
 void do_shell __PROTO((void));
 void undefine_command __PROTO((void));
@@ -191,7 +191,6 @@ int com_line __PROTO((void));
 int do_line __PROTO((void));
 void do_string __PROTO((const char* s));
 void do_string_and_free __PROTO((char* s));
-TBOOLEAN iteration_early_exit __PROTO((void));
 #ifdef USE_MOUSE
 void toggle_display_of_ipc_commands __PROTO((void));
 int display_ipc_commands __PROTO((void));
