@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: gadgets.c,v 1.115 2014/08/16 04:47:04 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - gadgets.c */
 
 /*[
@@ -36,11 +40,11 @@
 #include "graph3d.h" /* for map3d_position_r() */
 #include "graphics.h"
 #include "plot3d.h" /* For is_plot_with_palette() */
-#include "axis.h" /* For CB_AXIS */
 
 #include "pm3d.h"
 
 /* This file contains mainly a collection of global variables that
+ * used to be in 'set.c', where they didn't really belong. They
  * describe the status of several parts of the gnuplot plotting engine
  * that are used by both 2D and 3D plots, and thus belong neither to
  * graphics.c nor graph3d.c, alone. This is not a very clean solution,
@@ -53,22 +57,20 @@ legend_key keyT = DEFAULT_KEY_PROPS;
 color_box_struct color_box; /* initialized in init_color() */
 color_box_struct default_color_box = {SMCOLOR_BOX_DEFAULT, 'v', 1, LT_BLACK, LAYER_FRONT, 0,
 					{screen, screen, screen, 0.90, 0.2, 0.0},
-					{screen, screen, screen, 0.05, 0.6, 0.0}, FALSE,
+					{screen, screen, screen, 0.05, 0.6, 0.0},
 					{0,0,0,0} };
 
-/* The graph box (terminal coordinates) calculated by boundary() or boundary3d() */
+/* The graph box, in terminal coordinates, as calculated by boundary()
+ * or boundary3d(): */
 BoundingBox plot_bounds;
 
-/* The bounding box for 3D plots prior to applying view transformations */
-BoundingBox page_bounds;
-
 /* The bounding box for the entire drawable area  of current terminal */
 BoundingBox canvas;
 
 /* The bounding box against which clipping is to be done */
 BoundingBox *clip_area = &plot_bounds;
 
-/* 'set size', 'set origin' settings */
+/* 'set size', 'set origin' setttings */
 float xsize = 1.0;		/* scale factor for size */
 float ysize = 1.0;		/* scale factor for size */
 float zsize = 1.0;		/* scale factor for size */
@@ -83,9 +85,9 @@ int aspect_ratio_3D = 0;	/* 2 will put x and y on same scale, 3 for z also */
 t_position lmargin = DEFAULT_MARGIN_POSITION;
 /* space between bottom and plot_bounds.ybot in chars (-1: computed) */
 t_position bmargin = DEFAULT_MARGIN_POSITION;
-/* space between right edge and plot_bounds.xright in chars (-1: computed) */
+/* space between right egde and plot_bounds.xright in chars (-1: computed) */
 t_position rmargin = DEFAULT_MARGIN_POSITION;
-/* space between top edge and plot_bounds.ytop in chars (-1: computed) */
+/* space between top egde and plot_bounds.ytop in chars (-1: computed) */
 t_position tmargin = DEFAULT_MARGIN_POSITION;
 
 /* Pointer to first 'set dashtype' definition in linked list */
@@ -97,41 +99,31 @@ struct text_label *first_label = NULL;
 /* Pointer to first 'set linestyle' definition in linked list */
 struct linestyle_def *first_linestyle = NULL;
 struct linestyle_def *first_perm_linestyle = NULL;
-struct linestyle_def *first_mono_linestyle = NULL;
 
 /* Pointer to first 'set style arrow' definition in linked list */
 struct arrowstyle_def *first_arrowstyle = NULL;
 
-/* Listhead for pixmaps */
-t_pixmap *pixmap_listhead = NULL;
-
 /* Holds the properties from 'set style parallelaxis' */
 struct pa_style parallel_axis_style = DEFAULT_PARALLEL_AXIS_STYLE;
 
-/* Holds properties for 'set style spiderplot' */
-struct spider_web spiderplot_style = DEFAULT_SPIDERPLOT_STYLE;
-
 /* set arrow */
 struct arrow_def *first_arrow = NULL;
 
+#ifdef EAM_OBJECTS
 /* Pointer to first object instance in linked list */
 struct object *first_object = NULL;
-/* Pointer to array of grid walls */
-struct object grid_wall[5] = {WALL_Y0, WALL_X0, WALL_Y1, WALL_X1, WALL_Z0};
+#endif
 
 /* 'set title' status */
 text_label title = EMPTY_LABELSTRUCT;
 
 /* 'set timelabel' status */
 text_label timelabel = EMPTY_LABELSTRUCT;
+int timelabel_rotate = FALSE;
 int timelabel_bottom = TRUE;
 
 /* flag for polar mode */
 TBOOLEAN polar = FALSE;
-TBOOLEAN inverted_raxis = FALSE;
-
-/* toggle spiderplot mode on/off */
-TBOOLEAN spiderplot = FALSE;
 
 /* zero threshold, may _not_ be 0! */
 double zero = ZERO;
@@ -140,30 +132,21 @@ double zero = ZERO;
 double pointsize = 1.0;
 double pointintervalbox = 1.0;
 
-/* used for filled points */
-t_colorspec background_fill = BACKGROUND_COLORSPEC;
-
-/* box width (automatic) for plot style "with boxes" */
-double   boxwidth              = -1.0;
-/* whether box width is absolute (default) or relative */
-TBOOLEAN boxwidth_is_absolute  = TRUE;
-
 /* set border */
 int draw_border = 31;	/* The current settings */
 int user_border = 31;	/* What the user last set explicitly */
 int border_layer = LAYER_FRONT;
-# define DEFAULT_BORDER_LP { 0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 0, 1.0, 1.0, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN }
+# define DEFAULT_BORDER_LP { 0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0, 1.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN }
 struct lp_style_type border_lp = DEFAULT_BORDER_LP;
 const struct lp_style_type default_border_lp = DEFAULT_BORDER_LP;
-const struct lp_style_type background_lp = {0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN};
+const struct lp_style_type background_lp = {0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN};
 
 /* set clip */
 TBOOLEAN clip_lines1 = TRUE;
 TBOOLEAN clip_lines2 = FALSE;
 TBOOLEAN clip_points = FALSE;
-TBOOLEAN clip_radial = FALSE;
 
-static int clip_line(int *, int *, int *, int *);
+static int clip_line __PROTO((int *, int *, int *, int *));
 
 /* set samples */
 int samples_1 = SAMPLES;
@@ -176,7 +159,6 @@ enum PLOT_STYLE data_style = POINTSTYLE;
 enum PLOT_STYLE func_style = LINES;
 
 TBOOLEAN parametric = FALSE;
-TBOOLEAN in_parametric = FALSE;
 
 /* If last plot was a 3d one. */
 TBOOLEAN is_3d_plot = FALSE;
@@ -190,17 +172,21 @@ TBOOLEAN volatile_data = FALSE;
 
 fill_style_type default_fillstyle = { FS_EMPTY, 100, 0, DEFAULT_COLORSPEC } ;
 
+#ifdef EAM_OBJECTS
 /* Default rectangle style - background fill, black border */
 struct object default_rectangle = DEFAULT_RECTANGLE_STYLE;
 struct object default_circle = DEFAULT_CIRCLE_STYLE;
 struct object default_ellipse = DEFAULT_ELLIPSE_STYLE;
+#endif
 
 /* filledcurves style options */
 filledcurves_opts filledcurves_opts_data = EMPTY_FILLEDCURVES_OPTS;
 filledcurves_opts filledcurves_opts_func = EMPTY_FILLEDCURVES_OPTS;
 
+#if TRUE || defined(BACKWARDS_COMPATIBLE)
 /* Prefer line styles over plain line types */
 TBOOLEAN prefer_line_styles = FALSE;
+#endif
 
 /* If current terminal claims to be monochrome, don't try to send it colors */
 #define monochrome_terminal ((t->flags & TERM_MONOCHROME) != 0)
@@ -212,7 +198,9 @@ boxplot_style boxplot_opts = DEFAULT_BOXPLOT_STYLE;
 /* WINDOWID to be filled by terminals running on X11 (x11, wxt, qt, ...) */
 int current_x11_windowid = 0;
 
-textbox_style textbox_opts[NUM_TEXTBOX_STYLES];
+#ifdef EAM_BOXED_TEXT
+textbox_style textbox_opts = DEFAULT_TEXTBOX_STYLE;
+#endif
 
 /*****************************************************************/
 /* Routines that deal with global objects defined in this module */
@@ -229,7 +217,7 @@ textbox_style textbox_opts[NUM_TEXTBOX_STYLES];
  * 0 is returned if inside.
  */
 int
-clip_point(int x, int y)
+clip_point(unsigned int x, unsigned int y)
 {
     int ret_val = 0;
 
@@ -251,19 +239,18 @@ clip_point(int x, int y)
  *   This routine uses the cohen & sutherland bit mapping for fast clipping -
  * see "Principles of Interactive Computer Graphics" Newman & Sproull page 65.
  */
-int
+void
 draw_clip_line(int x1, int y1, int x2, int y2)
 {
     struct termentry *t = term;
-    int state;
 
-    state = clip_line(&x1, &y1, &x2, &y2);
-    if (state != 0) {
-	(*t->move) (x1, y1);
-	(*t->vector) (x2, y2);
-    }
+    if (!clip_line(&x1, &y1, &x2, &y2))
+	/* clip_line() returns zero --> segment completely outside
+	 * bounding box */
+	return;
 
-    return state;
+    (*t->move) (x1, y1);
+    (*t->vector) (x2, y2);
 }
 
 /* Draw a contiguous line path which may be clipped. Compared to
@@ -277,22 +264,16 @@ draw_clip_polygon(int points, gpiPoint *p)
     int x1, y1, x2, y2;
     int pos1, pos2, clip_ret;
     struct termentry *t = term;
-    TBOOLEAN continuous = TRUE;
 
     if (points <= 1) 
 	return;
 
-    if (p[0].x != p[points-1].x || p[0].y != p[points-1].y)
-	continuous = FALSE;
-
     x1 = p[0].x;
     y1 = p[0].y;
     pos1 = clip_point(x1, y1);
     if (!pos1) /* move to first point if it is inside */
 	(*t->move)(x1, y1);
 
-    newpath();
-
     for (i = 1; i < points; i++) {
 	x2 = p[i].x;
 	y2 = p[i].y;
@@ -304,9 +285,6 @@ draw_clip_polygon(int points, gpiPoint *p)
 	    if (pos1) /* first vertex was recalculated, move to new start point */
 		(*t->move)(x1, y1);
 	    (*t->vector)(x2, y2);
-	} else {
-	    /* Path is not continuous; make sure closepath is not called */
-	    continuous = FALSE;
 	}
 
 	x1 = p[i].x;
@@ -314,88 +292,29 @@ draw_clip_polygon(int points, gpiPoint *p)
 	/* The end point and the line do not necessarily have the same
 	 * status. The end point can be 'inside', but the whole line is
 	 * 'outside'. Do not update pos1 in this case.  Bug #1268.
+	 * FIXME: This is papering over an inconsistency in coordinate
+	 * calculation somewhere else!
 	 */
 	if (!(clip_ret == 0 && pos2 == 0))
 	    pos1 = pos2;
     }
-
-    /* Only call closepath if the polygon is truly closed; otherwise */
-    /* a spurious line connecting the start and end is generated.    */
-    if (continuous)
-	closepath();
 }
 
-/*
- * arrow is specified in terminal coordinates
- * but we use double rather than int so that the precision is sufficient
- * to orient and draw the arrow head correctly even for very short vectors.
- */
 void
-draw_clip_arrow( double dsx, double dsy, double dex, double dey, t_arrow_head head)
+draw_clip_arrow( int sx, int sy, int ex, int ey, int head)
 {
     struct termentry *t = term;
 
-    int sx = axis_map_toint(dsx);
-    int sy = axis_map_toint(dsy);
-    int ex = axis_map_toint(dex);
-    int ey = axis_map_toint(dey);
-    int dx, dy;
-
     /* Don't draw head if the arrow itself is clipped */
     if (clip_point(sx,sy))
 	head &= ~BACKHEAD;
     if (clip_point(ex,ey))
 	head &= ~END_HEAD;
+    clip_line(&sx, &sy, &ex, &ey);
 
-    /* clip_line returns 0 if the whole thing is out of range */
-    if (!clip_line(&sx, &sy, &ex, &ey))
-	return;
-
-    /* Special case code for short vectors */
-    /* Most terminals are OK with using this code for long vectors also.	*/
-    /* However some terminals (e.g. tikz) look terrible because the shaft of a	*/
-    /* long vector overruns the head when the head is drawn with HEADS_ONLY.	*/
-    /* FIXME:  this is a very ad hoc definition of "short".			*/
-    dx = abs(ex-sx);
-    dy = abs(ey-sy);
-    if (dx < 25 && dy < 25) {
-
-	/* draw the body of the vector (rounding errors are a problem) */
-	if (dx > 1 || dy > 1)
-	    if (!((t->flags & TERM_IS_LATEX)))
-		(*t->arrow)(sx, sy, ex, ey, SHAFT_ONLY | head);
-
-	/* if we're not supposed to be drawing any heads, we're done */
-	if ((head & BOTH_HEADS) == NOHEAD)
-	    return;
-
-	/* If this is truly a 0-vector, then we CAN'T draw the head */
-	/* because the orientation of the head is indeterminate     */
-	if (dsx == dex && dsy == dey)
-	    return;
-
-	/* If the head size is fixed we are free to change the length of */
-	/* very short vectors so that the orientation is accurate.	 */
-	if (curr_arrow_headfixedsize) {
-	    /* Direction vector in (dex,dey). I need to convert this to integers
-	     * with a scale that's large-enough to give me good angular resolution,
-	     * but small-enough to not overflow the data type.
-	     */
-	    double rescale = 1000. / GPMAX( fabs(dex-dsx), fabs(dey-dsy) );
-	    int newlenx = (dex - dsx) * rescale;
-	    int newleny = (dey - dsy) * rescale;
-
-	    if (head & END_HEAD)
-		(*t->arrow)(ex - newlenx, ey - newleny, ex, ey, END_HEAD|HEADS_ONLY);
-	    if (head & BACKHEAD)
-		(*t->arrow)(sx, sy, sx + newlenx, sy + newleny, BACKHEAD|HEADS_ONLY);
-	} else {
-	    (*t->arrow)(sx, sy, ex, ey, head|HEADS_ONLY);
-	}
-    } else {
-	/* The normal case, draw the whole thing at once */
-	(*t->arrow)(sx, sy, ex, ey, head);
-    }
+    /* Call terminal routine to draw the clipped arrow */
+    (*t->arrow)((unsigned int)sx, (unsigned int)sy,
+		(unsigned int)ex, (unsigned int)ey, head);
 }
 
 /* Clip the given line to drawing coords defined by BoundingBox.
@@ -417,9 +336,9 @@ clip_line(int *x1, int *y1, int *x2, int *y2)
      * This is now addressed by making dx and dy (double) rather than (int)
      * but it might be better to hard-code the sign tests.
      */
-    double dx, dy, x, y;
+    double dx, dy;
 
-    int x_intr[4], y_intr[4], count, pos1, pos2;
+    int x, y, x_intr[4], y_intr[4], count, pos1, pos2;
     int x_max, x_min, y_max, y_min;
     pos1 = clip_point(*x1, *y1);
     pos2 = clip_point(*x2, *y2);
@@ -501,6 +420,11 @@ clip_line(int *x1, int *y1, int *x2, int *y2)
 	    *y2 = y_intr[1];
 	}
     } else if (pos1) {		/* Only x1/y1 was out - update only it */
+	/* Nov 2010: When clip_line() and draw_clip_line() were consolidated in */
+	/* 2000, the test below was the only point of difference between them.  */
+	/* Unfortunately, the wrong version was kept. Now I change it back.     */
+	/* The effect of the wrong version (>= rather than >) was that a line   */
+	/* from ymin to ymax+eps was clipped to ymin,ymin rather than ymin,ymax */
 	if (dx * (*x2 - x_intr[0]) + dy * (*y2 - y_intr[0]) > 0) {
 	    *x1 = x_intr[0];
 	    *y1 = y_intr[0];
@@ -545,22 +469,16 @@ vertex_is_inside(gpiPoint test_vertex, gpiPoint *clip_boundary)
 void
 intersect_polyedge_with_boundary(gpiPoint first, gpiPoint second, gpiPoint *intersect, gpiPoint *clip_boundary)
 {
-    /* This routine is called only if one point is outside and the other
-     * is inside, which implies that clipping is needed at a horizontal
-     * boundary, that second.y is different from first.y and no division
-     * by zero occurs. Same for vertical boundary and x coordinates.
-     * Conversion to double is needed to prevent integer overflow.
-     */
-    double dx = (second.x - first.x);
-    double dy = (second.y - first.y);
+    /* this routine is called only if one point is outside and the other
+       is inside, which implies that clipping is needed at a horizontal
+       boundary, that second.y is different from first.y and no division
+       by zero occurs. Same for vertical boundary and x coordinates. */
     if (clip_boundary[0].y == clip_boundary[1].y) { /* horizontal */
 	(*intersect).y = clip_boundary[0].y;
-	(*intersect).x = first.x
-			+ (double)(clip_boundary[0].y - first.y) * dx/dy;
+	(*intersect).x = first.x + (clip_boundary[0].y - first.y) * (second.x - first.x)/(second.y - first.y);
     } else { /* vertical */
 	(*intersect).x = clip_boundary[0].x;
-	(*intersect).y = first.y
-			+ (double)(clip_boundary[0].x - first.x) * dy/dx;
+	(*intersect).y = first.y + (clip_boundary[0].x - first.x) * (second.y - first.y)/(second.x - first.x);
     }
 }
 
@@ -614,12 +532,12 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
     gpiPoint clip_boundary[5];
     static gpiPoint *tmp_corners = NULL;
 
-    if (!clip_area || in_length < 3) {
+    if (!clip_area) {
 	memcpy(out, in, in_length * sizeof(gpiPoint));
 	*out_length = in_length;
 	return;
     }
-    tmp_corners = gp_realloc(tmp_corners, 4 * in_length * sizeof(gpiPoint), "clip_polygon");
+    tmp_corners = gp_realloc(tmp_corners, 2 * in_length * sizeof(gpiPoint), "clip_polygon");
 
     /* vertices of the rectangular clipping window starting from
        top-left in counterclockwise direction */
@@ -634,7 +552,7 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
     clip_boundary[4] = clip_boundary[0];
 
     memcpy(tmp_corners, in, in_length * sizeof(gpiPoint));
-    for (i = 0; i < 4; i++) {
+    for(i = 0; i < 4; i++) {
 	clip_polygon_to_boundary(tmp_corners, out, in_length, out_length, clip_boundary+i);
 	memcpy(tmp_corners, out, *out_length * sizeof(gpiPoint));
 	in_length = *out_length;
@@ -642,122 +560,28 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
 }
 
 /* Two routines to emulate move/vector sequence using line drawing routine. */
-static int move_pos_x, move_pos_y;
+static unsigned int move_pos_x, move_pos_y;
 
 void
-clip_move(int x, int y)
+clip_move(unsigned int x, unsigned int y)
 {
     move_pos_x = x;
     move_pos_y = y;
 }
 
 void
-clip_vector(int x, int y)
+clip_vector(unsigned int x, unsigned int y)
 {
     draw_clip_line(move_pos_x, move_pos_y, x, y);
     move_pos_x = x;
     move_pos_y = y;
 }
 
-/*
- * draw_polar_clip_line() assumes that the endpoints have already
- * been categorized as INRANGE/OUTRANGE, and that "set clip radial"
- * is in effect.
- */
-void
-draw_polar_clip_line( double xbeg, double ybeg, double xend, double yend)
-{
-    double R;			/* radius of limiting circle */
-    double a, b;		/* line expressed as y = a*x + b */
-    double x1, y1, x2, y2;	/* Intersections of line and circle */
-    double Q, Q2;		/* sqrt term of quadratic equation */
-    TBOOLEAN vertical = FALSE;	/* flag for degenerate case */
-    TBOOLEAN beg_inrange, end_inrange;
-
-    if (R_AXIS.set_max == -VERYLARGE)
-	goto outside;
-    R = R_AXIS.set_max - R_AXIS.set_min;
-
-    /* If both endpoints are inside the limiting circle, draw_clip_line suffices */
-    beg_inrange = (xbeg*xbeg + ybeg*ybeg) <= R*R;
-    end_inrange = (xend*xend + yend*yend) <= R*R;
-    if (beg_inrange && end_inrange) {
-	draw_clip_line(map_x(xbeg), map_y(ybeg), map_x(xend), map_y(yend));
-	return;
-    }
-
-    /* FIXME:  logscale and other odd cases are not covered by this equation */
-    if (fabs(xbeg - xend) > ZERO) {
-	/* Recast line in the form y = a*x + b */
-	a = (yend - ybeg) / (xend - xbeg);
-	b = ybeg - xbeg * a;
-	/* the line may intersect a circle of radius R in two places */
-	Q2 = 4*a*a*b*b - 4 * (1 + a*a) * (b*b - R*R);
-	if (Q2 < 0)
-	    goto outside;
-	Q = sqrt(Q2);
-	x1 = (-2*a*b + Q) / ( 2*(1+a*a));
-	x2 = (-2*a*b - Q) / ( 2*(1+a*a));
-	y1 = a * x1 + b;
-	y2 = a * x2 + b;
-    } else {
-	/* degenerate case (vertical line) */
-	x1 = x2 = xbeg;
-	if (fabs(x1) > R)
-	    goto outside;
-	vertical = TRUE;
-	y1 = sqrt(R*R - x1*x1);
-	y2 = -y1;
-	if (!inrange(y1,ybeg,yend) && !inrange(y2,ybeg,yend))
-	    goto outside;
-    }
-
-    /* If one of the original endpoints was INRANGE then use it */
-    /* rather than the second intersection point.               */
-    if (vertical) {
-	y1 = GPMIN(y1, GPMAX(ybeg, yend));
-	y2 = GPMAX(y2, GPMIN(ybeg, yend));
-    } else if (beg_inrange) {
-	if (!inrange(x1, xbeg, xend)) {
-	    x1 = xbeg;
-	    y1 = ybeg;
-	} else {
-	    x2 = xbeg;
-	    y2 = ybeg;
-	}
-    } else if (end_inrange) {
-	if (!inrange(x1, xbeg, xend)) {
-	    x1 = xend;
-	    y1 = yend;
-	} else {
-	    x2 = xend;
-	    y2 = yend;
-	}
-    } else {
-	/* Both OUTRANGE. Are they on the same side of the circle? */
-	if (!inrange(x1, xbeg, xend))
-	    goto outside;
-    }
-
-    /* Draw the part of the line inside the bounding circle */
-    (term->move)(map_x(x1), map_y(y1));
-    (term->vector)(map_x(x2), map_y(y2));
-    /* fall through */
-
-outside:
-    /* Leave current position at unclipped endpoint */
-    (term->move)(map_x(xend), map_y(yend));
-    return;
-}
-
 /* Common routines for setting text or line color from t_colorspec */
 
 void
-apply_pm3dcolor(struct t_colorspec *tc)
+apply_pm3dcolor(struct t_colorspec *tc, const struct termentry *t)
 {
-    struct termentry *t = term;
-    double cbval;
-
     /* V5 - term->linetype(LT_BLACK) would clobber the current	*/
     /* dashtype so instead we use term->set_color(black).	*/
     static t_colorspec black = BLACK_COLORSPEC; 
@@ -774,7 +598,8 @@ apply_pm3dcolor(struct t_colorspec *tc)
 	return;
     }
     if (tc->type == TC_LT) {
-	t->set_color(tc);
+	if (!monochrome_terminal)
+	    t->set_color(tc);
 	return;
     }
     if (tc->type == TC_RGB) {
@@ -790,43 +615,31 @@ apply_pm3dcolor(struct t_colorspec *tc)
 	    t->set_color(tc);
 	return;
     }
-    /* Leave unchanged. (used only by "set errorbars"??) */
-    if (tc->type == TC_VARIABLE)
+    if (!is_plot_with_palette()) {
+	t->set_color(&black);
 	return;
-
+    }
     switch (tc->type) {
-	case TC_Z:
-		set_color(cb2gray(tc->value));
-		break;
-	case TC_CB:
-		if (CB_AXIS.log)
-		    cbval = (tc->value <= 0) ? CB_AXIS.min : tc->value;
-		else
-		    cbval = tc->value;
-		set_color(cb2gray(cbval));
-		break;
-	case TC_FRAC:
-		set_color(sm_palette.positive == SMPAL_POSITIVE ?  tc->value : 1-tc->value);
-		break;
-	default:
-		break; /* cannot happen */
+	case TC_Z:    set_color(cb2gray(z2cb(tc->value))); break;
+	case TC_CB:   set_color(cb2gray(tc->value));       break;
+	case TC_FRAC: set_color(sm_palette.positive == SMPAL_POSITIVE ?
+				tc->value : 1-tc->value);
+		      break;
     }
 }
 
 void
-reset_textcolor(const struct t_colorspec *tc)
+reset_textcolor(const struct t_colorspec *tc, const struct termentry *t)
 {
     if (tc->type != TC_DEFAULT)
-	term->linetype(LT_BLACK);
+	(*t->linetype)(LT_BLACK);
 }
 
 
 void
 default_arrow_style(struct arrow_style_type *arrow)
 {
-    static const struct lp_style_type tmp_lp_style = 
-	{0, LT_SOLID, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR,
-	DEFAULT_COLORSPEC, DEFAULT_DASHPATTERN};
+    static const struct lp_style_type tmp_lp_style = DEFAULT_LP_STYLE_TYPE;
 
     arrow->tag = -1;
     arrow->layer = LAYER_BACK;
@@ -862,23 +675,6 @@ apply_head_properties(struct arrow_style_type *arrow_properties)
     }
 }
 
-/* Used by arrow-drawing code in graphics.c graph3d.c */
-double effective_aspect_ratio(void)
-{
-#ifdef _WIN32
-    if (strcmp(term->name, "windows") == 0) {
-	double xscale = (axis_array[FIRST_X_AXIS].term_upper - axis_array[FIRST_X_AXIS].term_lower)
-			/ (axis_array[FIRST_X_AXIS].max - axis_array[FIRST_X_AXIS].min);
-	double yscale = (axis_array[FIRST_Y_AXIS].term_upper - axis_array[FIRST_Y_AXIS].term_lower)
-			/ (axis_array[FIRST_Y_AXIS].max - axis_array[FIRST_Y_AXIS].min);
-	double aspect_ratio = fabs(yscale/xscale);
-	return aspect_ratio;
-    }
-#endif
-
-    return (double)term->v_tic / (double)term->h_tic;
-}
-
 void
 free_labels(struct text_label *label)
 {
@@ -904,11 +700,12 @@ char *master_font = label->font;
 void
 get_offsets(
     struct text_label *this_label,
+    struct termentry *t,
     int *htic, int *vtic)
 {
-    if ((this_label->lp_properties.flags & LP_SHOW_POINTS)) {
-	*htic = (pointsize * term->h_tic * 0.5);
-	*vtic = (pointsize * term->v_tic * 0.5);
+    if (this_label->lp_properties.pointflag) {
+	*htic = (pointsize * t->h_tic * 0.5);
+	*vtic = (pointsize * t->v_tic * 0.5);
     } else {
 	*htic = 0;
 	*vtic = 0;
@@ -932,37 +729,32 @@ get_offsets(
  * This routine is used for both 2D and 3D plots.
  */
 void
-write_label(int x, int y, struct text_label *this_label)
+write_label(unsigned int x, unsigned int y, struct text_label *this_label)
 {
 	int htic, vtic;
 	int justify = JUST_TOP;	/* This was the 2D default; 3D had CENTRE */
-	textbox_style *textbox = NULL;
 
-	apply_pm3dcolor(&(this_label->textcolor));
+	apply_pm3dcolor(&(this_label->textcolor),term);
 	ignore_enhanced(this_label->noenhanced);
 
 	/* The text itself */
 	if (this_label->hypertext) {
-	    if (this_label->text && *(this_label->text)) {
-		/* Treat text as hypertext */
-		char *font = this_label->font;
-		if (font)
-		    term->set_font(font);
-		if (term->hypertext)
-		    term->hypertext(TERM_HYPERTEXT_TOOLTIP, this_label->text);
-		if (font)
-		    term->set_font("");
-	    }
+	    /* Treat text as hypertext */
+	    char *font = this_label->font;
+	    if (font)
+		term->set_font(font);
+	    if (term->hypertext)
+	        term->hypertext(TERM_HYPERTEXT_TOOLTIP, this_label->text);
+	    if (font)
+		term->set_font("");
 	} else {
 	    /* A normal label (always print text) */
-	    get_offsets(this_label, &htic, &vtic);
-	    if (this_label->boxed < 0)
-		textbox = &textbox_opts[0];
-	    else if (this_label->boxed > 0)
-		textbox = &textbox_opts[this_label->boxed];
+	    get_offsets(this_label, term, &htic, &vtic);
+#ifdef EAM_BOXED_TEXT
 	    /* Initialize the bounding box accounting */
-	    if (textbox && term->boxed_text && (textbox->opaque || !textbox->noborder))
+	    if (this_label->boxed && term->boxed_text)
 		(*term->boxed_text)(x + htic, y + vtic, TEXTBOX_INIT);
+#endif
 	    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
 		write_multiline(x + htic, y + vtic, this_label->text,
 				this_label->pos, justify, this_label->rotate,
@@ -973,50 +765,38 @@ write_label(int x, int y, struct text_label *this_label)
 				this_label->pos, justify, 0, this_label->font);
 	    }
 	}
-	if (textbox && term->boxed_text && (textbox->opaque || !textbox->noborder))
-	{
-
-	    /* Adjust the bounding box margins */
-	    (*term->boxed_text)((int)(textbox->xmargin * 100.),
-		(int)(textbox->ymargin * 100.), TEXTBOX_MARGINS);
+#ifdef EAM_BOXED_TEXT
+	/* Adjust the bounding box margins */
+	if (this_label->boxed && term->boxed_text)
+	    (*term->boxed_text)((int)(textbox_opts.xmargin * 100.),
+		(int)(textbox_opts.ymargin * 100.), TEXTBOX_MARGINS);
 
+	if (this_label->boxed && term->boxed_text && textbox_opts.opaque) {
 	    /* Blank out the box and reprint the label */
-	    if (textbox->opaque) {
-		apply_pm3dcolor(&textbox->fillcolor);
-		(*term->boxed_text)(0,0, TEXTBOX_BACKGROUNDFILL);
-		/* The epslatex/cairolatex terminals reuse a previously saved textbox
-		 * so explicitly re-writing it here is at best redundant.
-		 */
-		if ((term->flags & TERM_REUSES_BOXTEXT) == 0) {
-		    apply_pm3dcolor(&(this_label->textcolor));
-		    /* Init for each of fill and border */
-		    if (!textbox->noborder)
-			(*term->boxed_text)(x + htic, y + vtic, TEXTBOX_INIT);
-		    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
-			write_multiline(x + htic, y + vtic, this_label->text,
-				    this_label->pos, justify, this_label->rotate,
-				    this_label->font);
-			(*term->text_angle) (0);
-		    } else {
-			write_multiline(x + htic, y + vtic, this_label->text,
-				    this_label->pos, justify, 0, this_label->font);
-		    }
-		}
+	    (*term->boxed_text)(0,0, TEXTBOX_BACKGROUNDFILL);
+	    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
+		write_multiline(x + htic, y + vtic, this_label->text,
+			    this_label->pos, justify, this_label->rotate,
+			    this_label->font);
+		(*term->text_angle) (0);
+	    } else {
+		write_multiline(x + htic, y + vtic, this_label->text,
+			    this_label->pos, justify, 0, this_label->font);
 	    }
-
-	    /* Draw the bounding box */
-	    if (!textbox->noborder) {
-		(*term->linewidth)(textbox->linewidth);
-		apply_pm3dcolor(&textbox->border_color);
-		(*term->boxed_text)(0,0, TEXTBOX_OUTLINE);
-	    }
-
-	    (*term->boxed_text)(0,0, TEXTBOX_FINISH);
 	}
 
+	/* Draw the bounding box - FIXME should set line properties first */
+	if (this_label->boxed && term->boxed_text) {
+	    if (!textbox_opts.noborder)
+		(*term->boxed_text)(0,0, TEXTBOX_OUTLINE);
+	    else
+		(*term->boxed_text)(0,0, TEXTBOX_FINISH);
+	}
+#endif
+
 	/* The associated point, if any */
 	/* write_multiline() clips text to on_page; do the same for any point */
-	if ((this_label->lp_properties.flags & LP_SHOW_POINTS) && on_page(x,y)) {
+	if (this_label->lp_properties.pointflag && on_page(x,y)) {
 	    term_apply_lp_properties(&this_label->lp_properties);
 	    (*term->point) (x, y, this_label->lp_properties.p_type);
 	    /* the default label color is that of border */
@@ -1050,7 +830,7 @@ label_width(const char *str, int *lines)
     s = lab;
     while ((e = (char *) strchr(s, '\n')) != NULL) {
 	*e = '\0';
-	len = estimate_strlen(s, NULL);	/* = e-s ? */
+	len = estimate_strlen(s);	/* = e-s ? */
 	if (len > mlen)
 	    mlen = len;
 	if (len || l || *str == '\n')
@@ -1065,79 +845,3 @@ label_width(const char *str, int *lines)
     return (mlen);
 }
 
-/*
- * Here so that it can be shared by the 2D and 3D code
- */
-void
-do_timelabel(int x, int y)
-{
-    struct text_label temp = timelabel;
-    char str[MAX_LINE_LEN+1];
-    time_t now;
-
-    if (timelabel.rotate == 0 && !timelabel_bottom)
-	y -= term->v_char;
-
-    time(&now);
-    strftime(str, MAX_LINE_LEN, timelabel.text, localtime(&now));
-    temp.text = str;
-
-    write_label(x, y, &temp);
-}
-
-void
-init_gadgets()
-{
-   int i;
-   static t_position y0_wall_corners[5] = WALL_Y0_CORNERS;
-   static t_position x0_wall_corners[5] = WALL_X0_CORNERS;
-   static t_position y1_wall_corners[5] = WALL_Y1_CORNERS;
-   static t_position x1_wall_corners[5] = WALL_X1_CORNERS;
-   static t_position z0_wall_corners[5] = WALL_Z0_CORNERS;
-   for (i=0; i<4; i++)
-       grid_wall[i].next = &grid_wall[i+1];
-
-   grid_wall[WALL_Y0_TAG].o.polygon.vertex = y0_wall_corners;
-   grid_wall[WALL_X0_TAG].o.polygon.vertex = x0_wall_corners;
-   grid_wall[WALL_Y1_TAG].o.polygon.vertex = y1_wall_corners;
-   grid_wall[WALL_X1_TAG].o.polygon.vertex = x1_wall_corners;
-   grid_wall[WALL_Z0_TAG].o.polygon.vertex = z0_wall_corners;
-   grid_wall[WALL_Y0_TAG].lp_properties.pm3d_color.type = TC_RGB;
-   grid_wall[WALL_X0_TAG].lp_properties.pm3d_color.type = TC_RGB;
-   grid_wall[WALL_Y1_TAG].lp_properties.pm3d_color.type = TC_RGB;
-   grid_wall[WALL_X1_TAG].lp_properties.pm3d_color.type = TC_RGB;
-   grid_wall[WALL_Z0_TAG].lp_properties.pm3d_color.type = TC_RGB;
-   grid_wall[WALL_Y0_TAG].lp_properties.pm3d_color.lt = WALL_Y_COLOR;
-   grid_wall[WALL_X0_TAG].lp_properties.pm3d_color.lt = WALL_X_COLOR;
-   grid_wall[WALL_Y1_TAG].lp_properties.pm3d_color.lt = WALL_Y_COLOR;
-   grid_wall[WALL_X1_TAG].lp_properties.pm3d_color.lt = WALL_X_COLOR;
-   grid_wall[WALL_Z0_TAG].lp_properties.pm3d_color.lt = WALL_Z_COLOR;
-}
-
-/*
- * walk through the list of objects to see if any require pm3d processing
- */
-TBOOLEAN
-pm3d_objects(void)
-{
-    struct object *obj = first_object;
-    while (obj) {
-	if (obj->layer == LAYER_DEPTHORDER)
-	    return TRUE;
-	obj = obj->next;
-    }
-    return FALSE;
-}
-
-/*
- * Place overall title on the canvas (shared by plot and splot).
- */
-void
-place_title(int title_x, int title_y)
-{
-    if (title.text) {
-	/* NB: write_label applies text color but does not reset it */
-	write_label(title_x, title_y, &title);
-	reset_textcolor(&(title.textcolor));
-    }
-}
