@@ -1,5 +1,5 @@
 /*
- * $Id: gp_types.h,v 1.73 2016/11/05 21:21:07 sfeam Exp $
+ * $Id: gp_types.h,v 1.63 2014/07/22 23:11:16 sfeam Exp $
  */
 
 /* GNUPLOT - gp_types.h */
@@ -50,10 +50,7 @@ enum DATA_TYPES {
 	CMPLX,
 	STRING,
 	DATABLOCK,
-	ARRAY,
-	NOTDEFINED,	/* exists, but value is currently undefined */
-	INVALID_VALUE,	/* used only for error return by external functions */
-	INVALID_NAME	/* used only to trap errors in linked axis function definition */
+	INVALID_VALUE	/* used only for error return by external functions */
 };
 
 enum MODE_PLOT_TYPE {
@@ -115,12 +112,11 @@ typedef enum PLOT_STYLE {
     SURFACEGRID  = 31*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     PARALLELPLOT = 32*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     TABLESTYLE   = 33*PLOT_STYLE_BITS,
-    ZERRORFILL   = 34*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
     PLOT_STYLE_NONE = -1
 } PLOT_STYLE;
 
 typedef enum PLOT_SMOOTH {
-    SMOOTH_NONE = 0,
+    SMOOTH_NONE,
     SMOOTH_ACSPLINES,
     SMOOTH_BEZIER,
     SMOOTH_CSPLINES,
@@ -131,9 +127,7 @@ typedef enum PLOT_SMOOTH {
     SMOOTH_CUMULATIVE,
     SMOOTH_KDENSITY,
     SMOOTH_CUMULATIVE_NORMALISED,
-    SMOOTH_MONOTONE_CSPLINE,
-    SMOOTH_BINS,
-    SMOOTH_FREQUENCY_NORMALISED
+    SMOOTH_MONOTONE_CSPLINE
 } PLOT_SMOOTH;
 
 /* FIXME HBB 20000521: 'struct value' and its part, 'cmplx', should go
@@ -151,7 +145,6 @@ typedef struct value {
 	struct cmplx cmplx_val;
 	char *string_val;
 	char **data_array;
-	struct value *value_array;
     } v;
 } t_value;
 
@@ -166,21 +159,18 @@ typedef enum coord_type {
 } coord_type;
 
 
-/* These fields of 'struct coordinate' hold extra properties of 3D data points */
-/* Used by splot styles RGBIMAGE and RGBA_IMAGE */
+/* These fields of 'struct coordinate' used for storing the color of 3D data
+ * points (if requested by NEED_PALETTE(this_plot), for instance).
+ */
+#define CRD_COLOR yhigh
 #define CRD_R yhigh
 #define CRD_G xlow
 #define CRD_B xhigh
 #define CRD_A ylow
-/* Used by all splot style with variable line/point color */
-#define CRD_COLOR yhigh
-/* Used by splot styles POINTSTYLE and LINESPOINTS with variable point size */
+/* The field of 'struct coordinate' used for storing the point size in plot
+ * style POINTSTYLE with variable point size
+ */
 #define CRD_PTSIZE xlow
-/* Used by splot styles POINTSTYLE and LINESPOINTS with variable point type */
-#define CRD_PTTYPE xhigh
-/* Used by splot style ZERRORFILL */
-#define CRD_ZLOW xlow
-#define CRD_ZHIGH xhigh
 
 
 typedef struct coordinate {
@@ -197,12 +187,4 @@ typedef enum lp_class {
 	LP_NOFILL = 3	/* special treatment of fillcolor */
 } lp_class;
 
-/*
- * Introduction of nonlinear axes makes it possible for an axis-mapping function
- * to return "undefined" or NaN. These cannot be encoded as an integer coordinate.
- * So we introduce an integer equivalent to NaN and provide a macro to test for
- * whether a coordinate mapping returned it.
- */
-#define intNaN (~((unsigned int)(~0)>>1))
-
 #endif /* GNUPLOT_GPTYPES_H */
