@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.99 2014/06/04 03:21:21 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - hidden3d.c */
 
 /*[
@@ -130,7 +134,7 @@
 
 /* The actual configuration is stored in these variables, modifiable
  * at runtime through 'set hidden3d' options */
-       int hiddenBacksideLinetypeOffset = BACKSIDE_LINETYPE_OFFSET;
+static int hiddenBacksideLinetypeOffset = BACKSIDE_LINETYPE_OFFSET;
 static long hiddenTriangleLinesdrawnPattern = TRIANGLE_LINESDRAWN_PATTERN;
 static int hiddenHandleUndefinedPoints = HANDLE_UNDEFINED_POINTS;
 static int hiddenShowAlternativeDiagonal = SHOW_ALTERNATIVE_DIAGONAL;
@@ -299,8 +303,8 @@ static void color_edges __PROTO((long int new_edge, long int old_edge,
 				 int style_above, int style_below));
 static void build_networks __PROTO((struct surface_points * plots,
 				    int pcount));
-static int compare_edges_by_zmin __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
-static int compare_polys_by_zmax __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
+int compare_edges_by_zmin __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
+int compare_polys_by_zmax __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
 static void sort_edges_by_z __PROTO((void));
 static void sort_polys_by_z __PROTO((void));
 static TBOOLEAN get_plane __PROTO((p_polygon p, t_plane plane));
@@ -524,7 +528,7 @@ store_vertex (
 
 #ifdef HIDDEN3D_VAR_PTSIZE
     /* Store pointer back to original point */
-    /* Needed to support variable pointsize or pointtype */
+    /* Needed to support variable pointsize */
     thisvert->original = point;
 #endif
 	
@@ -553,12 +557,10 @@ make_edge(
 	thisedge->v1 = vnum1;
 	thisedge->v2 = vnum2;
 	if (lp->p_type == PT_ARROWHEAD) thisedge->style = PT_ARROWHEAD;
-	if (lp->p_type == PT_BACKARROW) thisedge->style = PT_BACKARROW;
     } else {
 	thisedge->v1 = vnum2;
 	thisedge->v2 = vnum1;
 	if (lp->p_type == PT_ARROWHEAD) thisedge->style = PT_BACKARROW;
-	if (lp->p_type == PT_BACKARROW) thisedge->style = PT_ARROWHEAD;
     }
 
     return thisedge - elist;
@@ -1049,7 +1051,7 @@ build_networks(struct surface_points *plots, int pcount)
 	long int crvlen;
 	
 	/* Quietly skip empty plots */
-	if (this_plot->plot_type == NODATA || this_plot->plot_type == KEYENTRY)
+	if (this_plot->plot_type == NODATA)
 	    continue;
 
 	crvlen = this_plot->iso_crvs->p_count;
@@ -1066,14 +1068,14 @@ build_networks(struct surface_points *plots, int pcount)
 
 	/* count 'curves' (i.e. isolines) and vertices in this plot */
 	nverts = 0;
-	if (this_plot->plot_type == FUNC3D) {
+	if(this_plot->plot_type == FUNC3D) {
 	    ncrvs = 0;
-	    for (icrvs = this_plot->iso_crvs;
+	    for(icrvs = this_plot->iso_crvs;
 		icrvs; icrvs = icrvs->next) {
 		ncrvs++;
 	    }
 	    nverts += ncrvs * crvlen;
-	} else if (this_plot->plot_type == DATA3D) {
+	} else if(this_plot->plot_type == DATA3D) {
 	    ncrvs = this_plot->num_iso_read;
 	    if (this_plot->has_grid_topology)
 		nverts += ncrvs * crvlen;
@@ -1085,7 +1087,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    nverts += icrvs->p_count;
 	    }
 	} else {
-	    /* Cannot happen */
+	    graph_error("Plot type is neither function nor data");
 	    return;
 	}
 
@@ -1114,15 +1116,6 @@ build_networks(struct surface_points *plots, int pcount)
 	    nv += 2 * nverts;
 	    ne += nverts;
 	    break;
-	case IMAGE:
-	case RGBIMAGE:
-	case RGBA_IMAGE:
-	    /* Ignore these */
-	    break;
-	case DOTS:
-	    this_plot->lp_properties.flags |= LP_SHOW_POINTS;
-	    this_plot->lp_properties.p_type = -1;
-	    /* fall through */
 	case POINTSTYLE:
 	default:
 	    /* treat all remaining ones like 'points' */
@@ -1161,30 +1154,25 @@ build_networks(struct surface_points *plots, int pcount)
 	lp = &(this_plot->lp_properties);
 
 	/* Quietly skip empty plots */
-	if (this_plot->plot_type == NODATA || this_plot->plot_type == KEYENTRY)
+	if (this_plot->plot_type == NODATA)
 	    continue;
 
+	crvlen = this_plot->iso_crvs->p_count;
+
 	/* Allow individual plots to opt out of hidden3d calculations */
 	if (this_plot->opt_out_of_hidden3d)
 	    continue;
 
-	crvlen = this_plot->iso_crvs->p_count;
-
 	/* We can't use the linetype passed to us, because it has been through */
 	/* load_linetype(), which replaced the nominal linetype with the one   */
 	/* assigned by "set linetype ..."                                      */
 	above = this_plot->hidden3d_top_linetype;
 	below = above + hiddenBacksideLinetypeOffset;
 
-	/* The "nosurface" flag is interpreted by hidden3d mode to mean */
-	/* "don't draw this surface".  I.e. draw only the contours.	*/
-	if (this_plot->opt_out_of_surface)
-	    above = below = LT_NODRAW;
-
 	/* This is a special flag indicating that the user specified an	*/
 	/* explicit surface color in the splot command.			*/
-	if ((lp->flags & LP_EXPLICIT_COLOR))
-	    below = above;
+	if (above == LT_SINGLECOLOR-1)
+	    above = below = LT_SINGLECOLOR;
 
 	/* We will not actually draw PM3D surfaces here, but their 	*/
 	/* edges can be used to calculate occlusion of lines, including */
@@ -1199,14 +1187,11 @@ build_networks(struct surface_points *plots, int pcount)
 
 	if (this_plot->plot_style == VECTOR) {
 	    lp->p_type = PT_ARROWHEAD;
-	    if (this_plot->arrow_properties.head == BACKHEAD)
-		lp->p_type = PT_BACKARROW;
 	    if (this_plot->arrow_properties.head == NOHEAD) {
 		this_plot->arrow_properties.head_length= 1;
 		this_plot->arrow_properties.head_angle = 0;
 	    }
-	    /* NB: It would not work to apply arrowhead properties now */
-	    /* because hidden3d code mixes arrows from multiple plots. */
+	    apply_3dhead_properties(&(this_plot->arrow_properties));
 	}
 
 	/* HBB 20000715: new initialization code block for non-grid
@@ -1226,7 +1211,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    long int thisvertex;
 		    struct coordinate labelpoint;
 
-		    lp->flags |= LP_SHOW_POINTS; /* Labels can use the code for hidden points */
+		    lp->pointflag = 1; /* Labels can use the code for hidden points */
 		    labelpoint.type = INRANGE;
 		    for (label = this_plot->labels->next; label != NULL; label = label->next) {
 			labelpoint.x = label->place.x;
@@ -1237,7 +1222,8 @@ build_networks(struct surface_points *plots, int pcount)
 			else
 			    labelpoint.CRD_COLOR = label->textcolor.lt;
 			
-			thisvertex = store_vertex(&labelpoint, lp, color_from_column);
+			thisvertex = store_vertex(&labelpoint, 
+				&(this_plot->lp_properties), color_from_column);
 			if (thisvertex < 0)
 			    continue;
 			(vlist+thisvertex)->label = label;
@@ -1246,13 +1232,9 @@ build_networks(struct surface_points *plots, int pcount)
 
 		} else for (i = 0; i < icrvs->p_count; i++) {
 		    long int thisvertex, basevertex;
-		    int interval = this_plot->lp_properties.p_interval;
 
-		    /* NULL lp means don't draw a point at this vertex */
-		    if (this_plot->plot_style == LINESPOINTS && interval && (i % interval))
-			thisvertex = store_vertex(points + i, NULL, color_from_column);
-		    else
-			thisvertex = store_vertex(points + i, lp, color_from_column);
+		    thisvertex = store_vertex(points + i, lp,
+					      color_from_column);
 
 		    if (this_plot->plot_style == VECTOR) {
 			store_vertex(icrvs->next->points+i, 0, 0);
@@ -1306,12 +1288,6 @@ build_networks(struct surface_points *plots, int pcount)
 			    store_edge(basevertex, edir_impulse, 0, lp, above);
 			break;
 
-		    case IMAGE:
-		    case RGBIMAGE:
-		    case RGBA_IMAGE:
-			/* Ignore these */
-			break;
-
 		    case POINTSTYLE:
 		    default:	/* treat all the others like 'points' */
 			store_edge(thisvertex, edir_point, crvlen, lp, above);
@@ -1462,11 +1438,9 @@ build_networks(struct surface_points *plots, int pcount)
 				     * all edges are new, so there is
 				     * no other polygon orientation to
 				     * consider */
-				    if (pnum > -2) {
-					if (!plist[pnum].frontfacing)
-					    elist[e1].style = elist[e2].style
-						= elist[e3].style = below;
-				    }
+				    if (!plist[pnum].frontfacing)
+					elist[e1].style = elist[e2].style = elist[e3].style
+					    = below;
 				}
 			    }
 			}
@@ -1533,7 +1507,8 @@ build_networks(struct surface_points *plots, int pcount)
 /* Sort the elist in order of growing zmax. Uses qsort on an array of
  * plist indices, and then fills in the 'next' fields in struct
  * polygon to store the resulting order inside the plist */
-static int
+/* HBB 20010720: removed 'static' to avoid HP-sUX gcc bug */
+int
 compare_edges_by_zmin(SORTFUNC_ARGS p1, SORTFUNC_ARGS p2)
 {
     return SIGN(vlist[elist[*(const long *) p1].v2].z
@@ -1571,7 +1546,8 @@ sort_edges_by_z()
     free(sortarray);
 }
 
-static int
+/* HBB 20010720: removed 'static' to avoid HP-sUX gcc bug */
+int
 compare_polys_by_zmax(SORTFUNC_ARGS p1, SORTFUNC_ARGS p2)
 {
     return (SIGN(plist[*(const long *) p1].zmax
@@ -1664,7 +1640,7 @@ draw_vertex(p_vertex v)
     p_type = v->lp_style->p_type;
 
     TERMCOORD(v, x, y);
-    if ((p_type >= -1 || p_type == PT_CHARACTER || p_type == PT_VARIABLE) && !clip_point(x,y)) {
+    if ((p_type >= -1 || p_type == PT_CHARACTER) && !clip_point(x,y)) {
 	struct t_colorspec *tc = &(v->lp_style->pm3d_color);
 
 	if (v->label)  {
@@ -1677,7 +1653,7 @@ draw_vertex(p_vertex v)
 	    struct lp_style_type style = *(v->lp_style);
 	    load_linetype(&style, (int)v->real_z);
 	    tc = &style.pm3d_color;
-	    apply_pm3dcolor(tc);
+	    apply_pm3dcolor(tc, term);
 	}
 	else if (tc->type == TC_RGB && tc->lt == LT_COLORFROMCOLUMN)
 	    set_rgbcolor_var((unsigned int)v->real_z);
@@ -1694,13 +1670,9 @@ draw_vertex(p_vertex v)
 #endif
 
 	if (p_type == PT_CHARACTER)
-	    (term->put_text)(x, y, v->lp_style->p_char);
-#ifdef HIDDEN3D_VAR_PTSIZE
-	else if (p_type == PT_VARIABLE)
-	    (term->point)(x, y, (int)(v->original->CRD_PTTYPE) - 1);
-#endif
+	    (term->put_text)(x, y, (char *)(&(v->lp_style->p_char)));
 	else
-	    (term->point)(x, y, p_type);
+	    (term->point)(x,y, p_type);
 
 	/* vertex has been drawn --> flag it as done */
 	v->lp_style = NULL;
@@ -1734,7 +1706,7 @@ draw_edge(p_edge e, p_vertex v1, p_vertex v2)
     } else
 
     /* This handles explicit 'lc rgb' in the plot command */
-    if (color.type == TC_RGB && (lptemp.flags & LP_EXPLICIT_COLOR)) {
+    if (color.type == TC_RGB && e->style == LT_SINGLECOLOR) {
 	recolor = TRUE;
     } else
 
@@ -1774,38 +1746,8 @@ draw_edge(p_edge e, p_vertex v1, p_vertex v2)
 	    lptemp.p_type = e->style;
     }
 
-    /* Only the original tip of an arrow should show an arrowhead */
-    /* FIXME:  Arrowhead lines are not themselves subject to hidden line removal */
-    if (arrow) {
-	/* FIXME: e->lp points to this_plot->lp_properties but what we need is */
-	/* a pointer to the immediately following field e->arrow_properties.   */
-	lp_style_type *lp = e->lp;
-	arrow_style_type *as = (arrow_style_type *)(&lp[1]);
-	apply_head_properties(as);
-	if (as->head == BOTH_HEADS)
-	    lptemp.p_type = PT_BOTHHEADS;
-
-	if (e->v2 != v2-vlist && e->v1 != v1-vlist)
-		lptemp.p_type = 0;
-
-	if (lptemp.p_type == PT_BACKARROW) {
-	    if (e->v2 == v2-vlist && e->v1 != v1-vlist)
-		lptemp.p_type = 0;
-	}
-	if (lptemp.p_type == PT_ARROWHEAD) {
-	    if (e->v1 == v1-vlist && e->v2 != v2-vlist)
-		lptemp.p_type = 0;
-	}
-	if (lptemp.p_type == PT_BOTHHEADS) {
-	    if (e->v1 == v1-vlist && e->v2 != v2-vlist)
-		lptemp.p_type = PT_BACKARROW;
-	    if (e->v2 == v2-vlist && e->v1 != v1-vlist)
-		lptemp.p_type = PT_ARROWHEAD;
-	}
-    }
-
     draw3d_line_unconditional(v1, v2, &lptemp, color);
-    if ((e->lp->flags & LP_SHOW_POINTS)) {
+    if (e->lp->pointflag) {
 	draw_vertex(v1);
 	draw_vertex(v2);
     }
@@ -1978,8 +1920,8 @@ in_front(
     grid_y_low = coord_to_treecell(ymin);
     grid_y_high = coord_to_treecell(ymax);
 
-    for (grid_x = grid_x_low; grid_x <= grid_x_high; grid_x++)
-	for (grid_y = grid_y_low; grid_y <= grid_y_high; grid_y++)
+    for (grid_x = grid_x_low; grid_x <= grid_x_high; grid_x ++)
+	for (grid_y = grid_y_low; grid_y <= grid_y_high; grid_y ++)
 	    for (listhead = quadtree[grid_x][grid_y];
 		 listhead >= 0;
 		 listhead = qlist[listhead].next)
@@ -2278,7 +2220,7 @@ draw_label_hidden(p_vertex v, struct lp_style_type *lp, int x, int y)
     vlist[thisvertex] = *v;
     vlist[thisvertex].lp_style = lp; /* Not sure this is necessary */
 
-    lp->flags |= LP_SHOW_POINTS; /* Labels can use the code for hidden points */
+    lp->pointflag = 1; /* Labels can use the code for hidden points */
 
     edgenum = make_edge(thisvertex, thisvertex, lp, lp->l_type, -1);
 
@@ -2305,7 +2247,7 @@ plot3d_hidden(struct surface_points *plots, int pcount)
     if (! edges.end) {
 	/* No drawable edges found. Free all storage and bail out. */
 	term_hidden_line_removal();
-	int_error(NO_CARET, "*All* edges undefined or out of range, thus no plot.");
+	graph_error("*All* edges undefined or out of range, thus no plot.");
     }
 
     if (! polygons.end) {
