@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: interpol.c,v 1.58 2017/03/16 18:16:12 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: interpol.c,v 1.46 2014/05/09 22:14:11 broeker Exp $"); }
 #endif
 
 /* GNUPLOT - interpol.c */
@@ -179,8 +179,9 @@ typedef double five_diag[5];
 
 static int next_curve __PROTO((struct curve_points * plot, int *curve_start));
 static int num_curves __PROTO((struct curve_points * plot));
-static double eval_kdensity __PROTO((struct curve_points *cp, 
-				   int first_point, int num_points, double x));
+static void eval_kdensity __PROTO((struct curve_points *cp, 
+				   int first_point, int num_points, double sr,
+				   coordval *px, coordval *py));
 static void do_kdensity __PROTO((struct curve_points *cp, int first_point,
 				 int num_points, struct coordinate *dest));
 static double *cp_binomial __PROTO((int points));
@@ -193,7 +194,7 @@ static spline_coeff *cp_approx_spline __PROTO((struct curve_points * plot, int f
 static spline_coeff *cp_tridiag __PROTO((struct curve_points * plot, int first_point, int num_points));
 static void do_cubic __PROTO((struct curve_points * plot, spline_coeff * sc, int first_point, int num_points, struct coordinate * dest));
 static void do_freq __PROTO((struct curve_points *plot,	int first_point, int num_points));
-static int compare_points __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
+int compare_points __PROTO((SORTFUNC_ARGS p1, SORTFUNC_ARGS p2));
 
 
 /*
@@ -253,34 +254,37 @@ num_curves(struct curve_points *plot)
 
    The implementation is based closely on the implementation for Bezier
    curves, except for the way the actual interpolation is generated.
-
-   EAM Feb 2015 - Revise to handle logscaled y axis and to 
-   pass in an actual x coordinate rather than a fraction of the min/max range.
-   NB: This code does not deal with logscaled x axis.
-   FIXME: It's silly to recalculate the mean/stddev/bandwidth every time.
 */
 
 /* eval_kdensity is a modification of eval_bezier */
-static double
+static void 
 eval_kdensity ( 
     struct curve_points *cp,
     int first_point,	/* where to start in plot->points (to find x-range) */
     int num_points,	/* to determine end in plot->points */
-    double x		/* x value at which to calculate y */
-    ) {
+    double sr,		/* position inside curve, range [0:1] */
+    coordval *px,	/* OUTPUT: x and y */
+    coordval *py ) {
 
     unsigned int i;
+    unsigned int n = num_points - 1;
     struct coordinate GPHUGE *this_points = (cp->points) + first_point;
   
-    double y, Z, ytmp;
+    double x, y, tmp;
     double avg, sigma;
+    double min =  DBL_MAX;
+    double max = -DBL_MAX;
     double bandwidth, default_bandwidth;
 
     avg = 0.0;
     sigma = 0.0;
-    for (i = 0; i < num_points; i++) {
-	avg   += this_points[i].x;
-	sigma += this_points[i].x * this_points[i].x;
+    for (i = 0; i <= n; i++) {
+      avg   += this_points[i].x;
+      sigma += this_points[i].x * this_points[i].x;
+
+      /* Find min and max of x-range. Necessary since points not sorted! */
+      min = this_points[i].x < min ? this_points[i].x : min;
+      max = this_points[i].x > max ? this_points[i].x : max;
     }
     avg /= (double)num_points;
     sigma = sqrt( sigma/(double)num_points - avg*avg ); /* Standard Deviation */
@@ -296,19 +300,20 @@ eval_kdensity (
     } else
 	bandwidth = cp->smooth_parameter;
 
+    x = min + sr*(max-min); /* The current x-value */
+
     y = 0;
-    for (i = 0; i < num_points; i++) {
-	Z = ( x - this_points[i].x )/bandwidth;
-	ytmp = this_points[i].y;
-	y += AXIS_DE_LOG_VALUE(cp->y_axis,ytmp) * exp( -0.5*Z*Z ) / bandwidth;
+    for (i = 0; i <= n; i++) {
+      tmp = ( x - this_points[i].x )/bandwidth;
+      y += this_points[i].y * exp( - 0.5*tmp*tmp ) / bandwidth;
     }
     y /= sqrt(2.0*M_PI);
 
-    return y;
+    *px = x;
+    *py = y;
 }
 
 /* do_kdensity is based on do_bezier, except for the call to eval_bezier */
-/* EAM Feb 2015: Don't touch xrange, but recalculate y limits  */
 static void 
 do_kdensity( 
     struct curve_points *cp,
@@ -317,34 +322,57 @@ do_kdensity(
     struct coordinate *dest)	/* where to put the interpolated data */
 {
     int i;
-    double x, y;
-    double sxmin, sxmax, step;
+    coordval x, y;
+
+    /* min and max in internal (eg logged) co-ordinates. We update
+     * these, then update the external extrema in user co-ordinates
+     * at the end.
+     */
+
+    double ixmin, ixmax, iymin, iymax;
+    double sxmin, sxmax, symin, symax;	/* starting values of above */
 
     x_axis = cp->x_axis;
     y_axis = cp->y_axis;
 
-    if (X_AXIS.log)
-	int_error(NO_CARET, "kdensity cannot handle logscale x axis");
-    sxmin = X_AXIS.min;
-    sxmax = X_AXIS.max;
-
-    step = (sxmax - sxmin) / (samples_1 - 1);
+    ixmin = sxmin = AXIS_LOG_VALUE(x_axis, X_AXIS.min);
+    ixmax = sxmax = AXIS_LOG_VALUE(x_axis, X_AXIS.max);
+    iymin = symin = AXIS_LOG_VALUE(y_axis, Y_AXIS.min);
+    iymax = symax = AXIS_LOG_VALUE(y_axis, Y_AXIS.max);
 
     for (i = 0; i < samples_1; i++) {
-	x = sxmin + i * step;
-        y = eval_kdensity( cp, first_point, num_points, x );
+        eval_kdensity( cp, first_point, num_points,
+		       (double) i / (double) (samples_1 - 1),
+		       &x, &y );
 
 	/* now we have to store the points and adjust the ranges */
 	dest[i].type = INRANGE;
-	dest[i].x = x;
-	STORE_WITH_LOG_AND_UPDATE_RANGE( dest[i].y, y, dest[i].type, y_axis,
-				cp->noautoscale, NOOP, NOOP);
+	STORE_AND_FIXUP_RANGE( dest[i].x, x, dest[i].type, ixmin, ixmax, 
+			       X_AXIS.autoscale, NOOP, continue);
+	STORE_AND_FIXUP_RANGE( dest[i].y, y, dest[i].type, iymin, iymax, 
+			       Y_AXIS.autoscale, NOOP, NOOP);
+
 	dest[i].xlow = dest[i].xhigh = dest[i].x;
 	dest[i].ylow = dest[i].yhigh = dest[i].y;
+
 	dest[i].z = -1;
     }
+
+    UPDATE_RANGE(ixmax > sxmax, X_AXIS.max, ixmax, x_axis);
+    UPDATE_RANGE(ixmin < sxmin, X_AXIS.min, ixmin, x_axis);
+    UPDATE_RANGE(iymax > symax, Y_AXIS.max, iymax, y_axis);
+    UPDATE_RANGE(iymin < symin, Y_AXIS.min, iymin, y_axis);
 }
 
+
+/*
+ * build up a cntr_struct list from curve_points
+ * this funtion is only used for the alternate entry point to
+ * Gershon's code and thus commented out
+ ***deleted***
+ */
+
+
 /* HBB 990205: rewrote the 'bezier' interpolation routine,
  * to prevent numerical overflow and other undesirable things happening
  * for large data files (num_data about 1000 or so), where binomial
@@ -874,12 +902,9 @@ do_cubic(
     xstart = GPMAX(this_points[0].x, sxmin);
     xend = GPMIN(this_points[num_points - 1].x, sxmax);
 
-    if (xstart >= xend) {
-	/* This entire segment lies outside the current x range. */
-	for (i = 0; i < samples_1; i++)
-	    dest[i].type = OUTRANGE;
-	return;
-    }
+    if (xstart >= xend)
+	int_error(plot->token,
+		  "Cannot smooth: no data within fixed xrange!");
 #endif
     xdiff = (xend - xstart) / (samples_1 - 1);
 
@@ -998,8 +1023,7 @@ gen_interp_frequency(struct curve_points *plot)
 
     curves = num_curves(plot);
 
-    if (plot->plot_smooth == SMOOTH_FREQUENCY_NORMALISED
-    ||  plot->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED) {
+    if (plot->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED) {
 	first_point = 0;
 
 	for (i = 0; i < curves; i++) {
@@ -1023,7 +1047,7 @@ gen_interp_frequency(struct curve_points *plot)
         /* If cumulative, replace the current y-value with the
            sum of all previous y-values. This assumes that the
            data has already been sorted by x-values. */
-        if (plot->plot_smooth == SMOOTH_CUMULATIVE) {
+        if( plot->plot_smooth == SMOOTH_CUMULATIVE ) {
             y = 0;
             for (j = first_point; j < first_point + num_points; j++) {
                 if (plot->points[j].type == UNDEFINED) 
@@ -1053,16 +1077,6 @@ gen_interp_frequency(struct curve_points *plot)
 	    }
 	}
 
-        /* Finally, normalized frequency smoothing means that we take our
-           existing histogram and divide each value by the total */
-        if (plot->plot_smooth == SMOOTH_FREQUENCY_NORMALISED) {
-	    for (j = first_point; j < first_point + num_points; j++) {
-		if (plot->points[j].type == UNDEFINED)
-		    continue;
-		plot->points[j].y /= y_total;
-	    }
-        }
-
 
         do_freq(plot, first_point, num_points);
         first_point += num_points + 1;
@@ -1114,9 +1128,9 @@ gen_interp(struct curve_points *plot)
 	    free((char *) bc);
 	    break;
 	case SMOOTH_KDENSITY:
-	    do_kdensity( plot, first_point, num_points, 
+	  do_kdensity( plot, first_point, num_points, 
 		       new_points + i * (samples_1 + 1));
-	    break;
+	  break;
 	default:		/* keep gcc -Wall quiet */
 	    ;
 	}
@@ -1145,7 +1159,8 @@ gen_interp(struct curve_points *plot)
 /* HBB 20010720: To avoid undefined behaviour that would be caused by
  * casting functions pointers around, changed arguments to what
  * qsort() *really* wants */
-static int
+/* HBB 20010720: removed 'static' to avoid HP-sUX gcc bug */
+int
 compare_points(SORTFUNC_ARGS arg1, SORTFUNC_ARGS arg2)
 {
     struct coordinate const *p1 = arg1;
@@ -1225,7 +1240,6 @@ cp_implode(struct curve_points *cp)
 	    } else {
 		cp->points[j].x = x;
  		if ( cp->plot_smooth == SMOOTH_FREQUENCY ||
- 		     cp->plot_smooth == SMOOTH_FREQUENCY_NORMALISED ||
  		     cp->plot_smooth == SMOOTH_CUMULATIVE ||
 		     cp->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED )
 		    k = 1;
@@ -1268,7 +1282,7 @@ cp_implode(struct curve_points *cp)
 			cp->points[j].type = OUTRANGE;
 		is_outrange:
 		    ;
-		} /* if (! all inrange) */
+		} /* if(! all inrange) */
 
 		j++;		/* next valid entry */
 		k = 0;		/* to read */
@@ -1279,9 +1293,8 @@ cp_implode(struct curve_points *cp)
 	if (k) {
 	    cp->points[j].x = x;
 	    if ( cp->plot_smooth == SMOOTH_FREQUENCY ||
-		 cp->plot_smooth == SMOOTH_FREQUENCY_NORMALISED ||
 		 cp->plot_smooth == SMOOTH_CUMULATIVE ||
-		 cp->plot_smooth == SMOOTH_CUMULATIVE_NORMALISED)
+		 cp->plot_smooth == SMOOTH_CUMULATIVE)
 		k = 1;
 	    cp->points[j].y = y /= (double) k;
 	    cp->points[j].xhigh = sux / (double) k;
@@ -1318,12 +1331,6 @@ cp_implode(struct curve_points *cp)
 	    }
 	    j++;		/* next valid entry */
 	}
-
-	/* FIXME: Monotonic cubic splines support only a single curve per data set */
-	if (j < cp->p_count && cp->plot_smooth == SMOOTH_MONOTONE_CSPLINE) {
-	    break;
-	}
-
 	/* insert invalid point to separate curves */
 	if (j < cp->p_count) {
 	    cp->points[j].type = UNDEFINED;
@@ -1351,16 +1358,14 @@ mcs_interp(struct curve_points *plot)
     int i;
 
     /* These will track the resulting smoothed curve */
-    /* V5: Try to ensure that the sampling is fine enough to pass through the original points */
-    int Nsamp = (samples_1 > 2*N) ? samples_1 : 2*N;
-    struct coordinate *new_points = gp_alloc((Nsamp+1) * sizeof(coordinate), "mcs");
+    struct coordinate *new_points = gp_alloc((samples_1+1) * sizeof(coordinate), "mcs");
     double sxmin = AXIS_LOG_VALUE(plot->x_axis, X_AXIS.min);
     double sxmax = AXIS_LOG_VALUE(plot->x_axis, X_AXIS.max);
     double xstart, xend, xstep;
 
     xstart = GPMAX(p[0].x, sxmin);
     xend = GPMIN(p[N-1].x, sxmax);
-    xstep = (xend - xstart) / (Nsamp - 1);
+    xstep = (xend - xstart) / (samples_1 - 1);
 
     /* Calculate spline coefficients */
 #define DX	xlow
@@ -1369,10 +1374,6 @@ mcs_interp(struct curve_points *plot)
 #define C2	yhigh
 #define C3	z
 
-    /* Work with the un-logged y values */
-    for (i = 0; i < N-1; i++)
-	p[i].y = AXIS_DE_LOG_VALUE(plot->y_axis, p[i].y);
-
     for (i = 0; i < N-1; i++) {
 	p[i].DX = p[i+1].x - p[i].x;
 	p[i].SLOPE = (p[i+1].y - p[i].y) / p[i].DX;
@@ -1398,7 +1399,7 @@ mcs_interp(struct curve_points *plot)
     }
 
     /* Use the coefficients C1, C2, C3 to interpolate over the requested range */
-    for (i = 0; i < Nsamp; i++) {
+    for (i = 0; i < samples_1; i++) {
 	double x = xstart + i * xstep;
 	double y;
 	TBOOLEAN exact = FALSE;
@@ -1439,8 +1440,8 @@ mcs_interp(struct curve_points *plot)
     /* Replace original data with the interpolated curve */
     free(p);
     plot->points = new_points;
-    plot->p_count = Nsamp;
-    plot->p_max = Nsamp + 1;
+    plot->p_count = samples_1;
+    plot->p_max = samples_1 + 1;
 
 #undef DX
 #undef SLOPE
@@ -1448,133 +1449,3 @@ mcs_interp(struct curve_points *plot)
 #undef C2
 #undef C3
 }
-
-/*
- * Binned histogram of input values.
- *
- *   plot FOO using N:(1) bins{=<nbins>} {binrange=[binlow:binhigh]}
- *                        {binwidth=<width>} with boxes
- *
- * This option is EXPERIMENTAL, details may change before inclusion in a stable
- * gnuplot release.
- *
- * If no binrange is given, binlow and binhigh are taken from the x range of the data.
- * In either of these cases binlow is the midpoint x-coordinate of the first bin
- * and binhigh is the midpoint x-coordinate of the last bin.
- * Points that lie exactly on a bin boundary are assigned to the upper bin.
- * Bin assignments are not affected by "set xrange".
- * Notes:
- *    binwidth = (binhigh-binlow) / (nbins-1)
- *        xmin = binlow - binwidth/2
- *        xmax = binhigh + binwidth/2
- *    first bin holds points with (xmin =< x < xmin + binwidth)
- *    last bin holds points with (xmax-binwidth =< x < binhigh + binwidth)
- */
-void
-make_bins(struct curve_points *plot, int nbins,
-          double binlow, double binhigh, double binwidth)
-{
-    int i, binno;
-    double *bin;
-    double bottom, top, range;
-    struct axis *xaxis = &axis_array[plot->x_axis];
-    struct axis *yaxis = &axis_array[plot->y_axis];
-    double ymax = 0;
-    int N = plot->p_count;
-
-    /* Find the range of points to be binned */
-    if (binlow != binhigh) {
-	/* Explicit binrange [min:max] in the plot command */
-	bottom = binlow;
-	top = binhigh;
-    } else {
-	/* Take binrange from the data itself */
-	bottom = VERYLARGE; top = -VERYLARGE;
-	for (i=0; i<N; i++) {
-	    if (bottom > plot->points[i].x)
-		bottom = plot->points[i].x;
-	    if (top < plot->points[i].x)
-		top = plot->points[i].x;
-	}
-	if (top <= bottom)
-	    int_warn(NO_CARET, "invalid bin range [%g:%g]", bottom, top);
-    }
-    bottom = axis_log_value(xaxis, bottom);
-    top = axis_log_value(xaxis, top);
-
-    /* If a fixed binwidth was provided, find total number of bins */
-    if (binwidth > 0) {
-	double temp;
-	nbins = 1 + (top - bottom) / binwidth;
-	temp = nbins * binwidth - (top - bottom);
-	bottom -= temp/2.;
-	top += temp/2.;
-    }
-    /* otherwise we use (N-1) intervals between midpoints of bin 1 and bin N */
-    else {
-	binwidth = (top - bottom) / (nbins - 1);
-	bottom -= binwidth/2.;
-	top += binwidth/2.;
-    }
-    range = top - bottom;
-
-    FPRINTF((stderr,"make_bins: %d bins from %g to %g, binwidth %g\n",
-	    nbins, bottom, top, binwidth));
-
-    bin = gp_alloc(nbins*sizeof(double), "bins");
-    for (i=0; i<nbins; i++)
- 	bin[i] = 0;
-    for (i=0; i<N; i++) {
-	if (plot->points[i].type == UNDEFINED)
-	    continue;
-	binno = floor(nbins * (plot->points[i].x - bottom) / range);
-	if (0 <= binno && binno < nbins)
-	    bin[binno] += axis_de_log_value(yaxis, plot->points[i].y);
-    }
-
-    if (xaxis->autoscale & AUTOSCALE_MIN) {
-	if (xaxis->min > bottom)
-	    xaxis->min = bottom;
-    }
-    if (xaxis->autoscale & AUTOSCALE_MAX) {
-	if (xaxis->max < top)
-	    xaxis->max = top;
-    }
-
-    /* Replace the original data with one entry per bin.
-     * new x = midpoint of bin
-     * new y = number of points in the bin
-     */
-    plot->p_count = nbins;
-    plot->points = gp_realloc( plot->points, nbins * sizeof(struct coordinate), "curve_points");
-    for (i=0; i<nbins; i++) {
-	double bincent = bottom + (0.5 + (double)i) * binwidth;
-	plot->points[i].type = INRANGE;
-	plot->points[i].x     = bincent;
-	plot->points[i].xlow  = bincent - binwidth/2.;
-	plot->points[i].xhigh = bincent + binwidth/2.;
-	plot->points[i].y     = axis_log_value(yaxis, bin[i]);
-	plot->points[i].ylow  = plot->points[i].y;
-	plot->points[i].yhigh = plot->points[i].y;
-	plot->points[i].z = 0;	/* FIXME: leave it alone? */
-	if (inrange(axis_de_log_value(xaxis, bincent), xaxis->min, xaxis->max)) {
-	    if (ymax < bin[i])
-		ymax = bin[i];
-	} else {
-	    plot->points[i].type = OUTRANGE;
-	}
-	FPRINTF((stderr, "bin[%d] %g %g\n", i, plot->points[i].x, plot->points[i].y));
-    }
-
-    if (yaxis->autoscale & AUTOSCALE_MIN) {
-	if (yaxis->min > 0)
-	    yaxis->min = 0;
-    }
-    if (yaxis->autoscale & AUTOSCALE_MAX) {
-	if (yaxis->max < ymax)
-	    yaxis->max = ymax;
-    }
-
-    /* Clean up */
-    free(bin);
-}
