@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.199 2017/05/16 03:22:26 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.168 2014/08/05 17:13:13 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -251,7 +251,7 @@ static void bind_remove __PROTO((bind_t * b));
 static void bind_append __PROTO((char *lhs, char *rhs, char *(*builtin) (struct gp_event_t * ge)));
 static void recalc_ruler_pos __PROTO((void));
 static void turn_ruler_off __PROTO((void));
-static int nearest_label_tag __PROTO((int x, int y));
+static int nearest_label_tag __PROTO((int x, int y, struct termentry * t));
 static void remove_label __PROTO((int x, int y));
 static void put_label __PROTO((char *label, double x, double y));
 
@@ -373,47 +373,16 @@ MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double
 	*y2 = AXIS_DE_LOG_VALUE(SECOND_Y_AXIS, *y2);
     }
 
-    /* If x2 or y2 is linked to a primary axis via mapping function, apply it now */
+    /* If x2 is linked to x via a mapping function, apply it now */
+    /* Similarly for y/y2 */
     if (!is_3d_plot) {
-	AXIS *secondary = &axis_array[SECOND_X_AXIS];
-	if (secondary->linked_to_primary && secondary->link_udf->at)
-	    *x2 = eval_link_function(secondary, *x);
-	secondary = &axis_array[SECOND_Y_AXIS];
-	if (secondary->linked_to_primary && secondary->link_udf->at)
-	    *y2 = eval_link_function(secondary, *y);
+	if (axis_array[SECOND_X_AXIS].linked_to_primary
+	&&  axis_array[SECOND_X_AXIS].link_udf->at)
+	    *x2 = eval_link_function(SECOND_X_AXIS, *x);
+	if (axis_array[SECOND_Y_AXIS].linked_to_primary
+	&&  axis_array[SECOND_Y_AXIS].link_udf->at)
+	    *y2 = eval_link_function(SECOND_Y_AXIS, *y);
     }
-
-#ifdef NONLINEAR_AXES
-    /* If x or y is linked to a (hidden) primary axis, it's a bit more complicated */
-    if (!is_3d_plot) {
-	AXIS *secondary;
-	secondary = &axis_array[FIRST_X_AXIS];
-	if (secondary->linked_to_primary
-	&&  secondary->linked_to_primary->index == -FIRST_X_AXIS) {
-	    *x = axis_mapback(secondary->linked_to_primary, xx);
-	    *x = eval_link_function(secondary, *x);
-	}
-	secondary = &axis_array[FIRST_Y_AXIS];
-	if (secondary->linked_to_primary
-	&&  secondary->linked_to_primary->index == -FIRST_Y_AXIS) {
-	    *y = axis_mapback(secondary->linked_to_primary, yy);
-	    *y = eval_link_function(secondary, *y);
-	}
-	secondary = &axis_array[SECOND_X_AXIS];
-	if (secondary->linked_to_primary
-	&&  secondary->linked_to_primary->index == -SECOND_X_AXIS) {
-	    *x2 = axis_mapback(secondary->linked_to_primary, xx);
-	    *x2 = eval_link_function(secondary, *x2);
-	}
-	secondary = &axis_array[SECOND_Y_AXIS];
-	if (secondary->linked_to_primary
-	&&  secondary->linked_to_primary->index == -SECOND_Y_AXIS) {
-	    *y2 = axis_mapback(secondary->linked_to_primary, yy);
-	    *y2 = eval_link_function(secondary, *y2);
-	}
-    }
-#endif
-
 }
 
 static char *
@@ -489,29 +458,19 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 	sprintf(s, xy_format(), x, y);	/* w/o brackets */
     } else if (mode == MOUSE_COORDINATES_ALT && (fmt || polar)) {
 	if (polar) {
-	    double r;
-	    double phi = atan2(y,x);
+	    double phi, r;
 	    double rmin = (R_AXIS.autoscale & AUTOSCALE_MIN) ? 0.0 : R_AXIS.set_min;
-	    double theta = phi / DEG2RAD;
-
-	    /* Undo "set theta" */
-	    theta = (theta - theta_origin) * theta_direction;
-	    if (theta > 180.)
-		theta = theta - 360.;
-
-	    if (nonlinear(&R_AXIS))
-		r = eval_link_function(&R_AXIS, x/cos(phi) + R_AXIS.linked_to_primary->min);
-	    else if (R_AXIS.log)
+	    phi = atan2(y,x);
+	    if (R_AXIS.log)
 		r = AXIS_UNDO_LOG(POLAR_AXIS, x/cos(phi) + AXIS_DO_LOG(POLAR_AXIS, rmin));
-	    else if (inverted_raxis)
-		r = rmin - x/cos(phi);
 	    else
-		r = rmin + x/cos(phi);
-
+		r = x/cos(phi) + rmin;
 	    if (fmt)
-		sprintf(s, fmt, theta, r);
+		sprintf(s, fmt, phi/ang2rad, r);
 	    else {
-		sprintf(s, "theta: %.1f%s  r: %g", theta, degree_sign, r);
+		sprintf(s, "polar: ");
+		s += strlen(s);
+		sprintf(s, xy_format(), phi/ang2rad, r);
 	    }
 	} else {
 	    sprintf(s, fmt, x, y);	/* user defined format */
@@ -529,26 +488,41 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 static char *
 xDateTimeFormat(double x, char *b, int mode)
 {
-    struct tm tm;
+    time_t xtime_position = SEC_OFFS_SYS + x;
+    struct tm *pxtime_position = gmtime(&xtime_position);
 
+    if (pxtime_position == NULL) {
+	b[0] = NUL;
+	return b;
+    }
     switch (mode) {
     case MOUSE_COORDINATES_XDATE:
-	ggmtime(&tm, x);
-	sprintf(b, "%d. %d. %04d", tm.tm_mday, tm.tm_mon + 1, tm.tm_year);
+	sprintf(b, "%d. %d. %04d", pxtime_position->tm_mday, (pxtime_position->tm_mon) + 1,
+# if 1
+		(pxtime_position->tm_year) + ((pxtime_position->tm_year <= 68) ? 2000 : 1900)
+# else
+		((pxtime_position->tm_year) < 100) ? (pxtime_position->tm_year) : (pxtime_position->tm_year) - 100
+		/*              (pxtime_position->tm_year)+1900 */
+# endif
+	    );
 	break;
     case MOUSE_COORDINATES_XTIME:
-	ggmtime(&tm, x);
-	sprintf(b, "%d:%02d", tm.tm_hour, tm.tm_min);
+	sprintf(b, "%d:%02d", pxtime_position->tm_hour, pxtime_position->tm_min);
 	break;
     case MOUSE_COORDINATES_XDATETIME:
-	ggmtime(&tm, x);
-	sprintf(b, "%d. %d. %04d %d:%02d", tm.tm_mday, tm.tm_mon + 1, tm.tm_year,
-		tm.tm_hour, tm.tm_min);
+	sprintf(b, "%d. %d. %04d %d:%02d", pxtime_position->tm_mday, (pxtime_position->tm_mon) + 1,
+# if 1
+		(pxtime_position->tm_year) + ((pxtime_position->tm_year <= 68) ? 2000 : 1900),
+# else
+		((pxtime_position->tm_year) < 100) ? (pxtime_position->tm_year) : (pxtime_position->tm_year) - 100,
+		/*              (pxtime_position->tm_year)+1900, */
+# endif
+		pxtime_position->tm_hour, pxtime_position->tm_min);
 	break;
     case MOUSE_COORDINATES_TIMEFMT:
 	/* FIXME HBB 20000507: timefmt is for *reading* timedata, not
 	 * for writing them! */
-	gstrftime(b, 0xff, timefmt, x);
+	gstrftime(b, 0xff, axis_array[FIRST_X_AXIS].timefmt, x);
 	break;
     default:
 	sprintf(b, mouse_setting.fmt, x);
@@ -565,7 +539,7 @@ xDateTimeFormat(double x, char *b, int mode)
 do {								\
     if (x >= VERYLARGE)	break;					\
     if (axis_array[axis].datatype == DT_TIMEDATE) {		\
-	char *format = copy_or_invent_formatstring(&axis_array[axis]);	\
+	char *format = copy_or_invent_formatstring(axis);	\
 	while (strchr(format,'\n'))				\
 	     *(strchr(format,'\n')) = ' ';			\
 	gstrftime(sp, 40, format, x);				\
@@ -652,6 +626,7 @@ static AXIS *axis_array_copy = NULL;
 static void
 apply_zoom(struct t_zoom *z)
 {
+    char s[1024];		/* HBB 20011005: made larger */
     int is_splot_map = (is_3d_plot && (splot_map == TRUE));
 
     if (zoom_now != NULL) {	/* remember the current zoom */
@@ -692,25 +667,26 @@ apply_zoom(struct t_zoom *z)
     /* Now we're committed. Notify the terminal the the next replot is a zoom */
     (*term->layer)(TERM_LAYER_BEFORE_ZOOM);
 
-    /* New range on primary axes */
-    set_explicit_range(&axis_array[FIRST_X_AXIS], zoom_now->xmin, zoom_now->xmax);
-    set_explicit_range(&axis_array[FIRST_Y_AXIS], zoom_now->ymin, zoom_now->ymax);
+    sprintf(s, "set xr[%.12g:%.12g]; set yr[%.12g:%.12g]",
+	       zoom_now->xmin, zoom_now->xmax, 
+	       zoom_now->ymin, zoom_now->ymax);
 
     /* EAM Apr 2013 - The tests on VERYLARGE protect against trying to   */
     /* interpret the autoscaling initial state as an actual limit value. */
     if (!is_3d_plot
     && (zoom_now->x2min < VERYLARGE && zoom_now->x2max > -VERYLARGE)) {
-	set_explicit_range(&axis_array[SECOND_X_AXIS], zoom_now->x2min, zoom_now->x2max);
+	sprintf(s + strlen(s), "; set x2r[% #g:% #g]",
+		zoom_now->x2min, zoom_now->x2max);
     }
     if (!is_3d_plot
     && (zoom_now->y2min < VERYLARGE && zoom_now->y2max > -VERYLARGE)) {
-	set_explicit_range(&axis_array[SECOND_Y_AXIS], zoom_now->y2min, zoom_now->y2max);
+	sprintf(s + strlen(s), "; set y2r[% #g:% #g]",
+		zoom_now->y2min, zoom_now->y2max);
     }
 
     /* EAM Jun 2007 - The autoscale save/restore was too complicated, and broke
      * refresh. Just save/restore the complete axis state and have done with it.
      * Well, not _quite_ the complete state.  The labels are maintained dynamically.
-     * Apr 2015 - The same is now true (dynamic storage) for ticfmt, formatstring.
      */
     if (zoom_now == zoom_head) {
 	int i;
@@ -718,18 +694,9 @@ apply_zoom(struct t_zoom *z)
 	    axis_array_copy[i].label = axis_array[i].label;
 	    axis_array_copy[i].ticdef.def.user = axis_array[i].ticdef.def.user;
 	    axis_array_copy[i].ticdef.font = axis_array[i].ticdef.font;
-	    axis_array_copy[i].ticfmt = axis_array[i].ticfmt;
-	    axis_array_copy[i].formatstring = axis_array[i].formatstring;
 	}
 	memcpy(axis_array, axis_array_copy, sizeof(axis_array));
-
-	/* The shadowed primary axis, if any, is not restored by the memcpy.	*/
-	/* We choose to recalculate the limits, but alternatively we could find	*/
-	/* some place to save/restore the unzoomed limits.			*/
-	if (nonlinear(&axis_array[FIRST_X_AXIS]))
-	    clone_linked_axes(&axis_array[FIRST_X_AXIS], axis_array[FIRST_X_AXIS].linked_to_primary);
-	if (nonlinear(&axis_array[FIRST_Y_AXIS]))
-	    clone_linked_axes(&axis_array[FIRST_Y_AXIS], axis_array[FIRST_Y_AXIS].linked_to_primary);
+	s[0] = '\0';	/* FIXME:  Is this better than calling replotrequest()? */
 
 	/* Falling through to do_string_replot() does not work! */
 	if (volatile_data) {
@@ -747,7 +714,7 @@ apply_zoom(struct t_zoom *z)
 	inside_zoom = TRUE;
     }
 
-    do_string_replot("");
+    do_string_replot(s);
     inside_zoom = FALSE;
 }
 
@@ -1033,7 +1000,7 @@ builtin_set_plots_visible(struct gp_event_t *ge)
 	return "`builtin-set-plots-visible`";
     }
     if (term->modify_plots)
-	term->modify_plots(MODPLOTS_SET_VISIBLE, -1);
+	term->modify_plots(MODPLOTS_SET_VISIBLE);
     return (char *) 0;
 }
 
@@ -1044,7 +1011,7 @@ builtin_set_plots_invisible(struct gp_event_t *ge)
 	return "`builtin-set-plots-invisible`";
     }
     if (term->modify_plots)
-	term->modify_plots(MODPLOTS_SET_INVISIBLE, -1);
+	term->modify_plots(MODPLOTS_SET_INVISIBLE);
     return (char *) 0;
 }
 
@@ -1055,7 +1022,7 @@ builtin_invert_plot_visibilities(struct gp_event_t *ge)
 	return "`builtin-invert-plot-visibilities`";
     }
     if (term->modify_plots)
-	term->modify_plots(MODPLOTS_INVERT_VISIBILITIES, -1);
+	term->modify_plots(MODPLOTS_INVERT_VISIBILITIES);
     return (char *) 0;
 }
 
@@ -1147,8 +1114,6 @@ builtin_toggle_mouse(struct gp_event_t *ge)
 	    fprintf(stderr, "turning mouse off.\n");
 	}
     }
-    if (term->set_cursor)
-	term->set_cursor(0, 0, 0);
 # ifdef OS2
     PM_update_menu_items();
 # endif
@@ -1178,9 +1143,11 @@ builtin_toggle_ruler(struct gp_event_t *ge)
 	MousePosToGraphPosReal(ruler.px, ruler.py, &ruler.x, &ruler.y, &ruler.x2, &ruler.y2);
 	(*term->set_ruler) (ruler.px, ruler.py);
 	if ((u = add_udv_by_name("MOUSE_RULER_X"))) {
+	    u->udv_undef = FALSE;
 	    Gcomplex(&u->udv_value,ruler.x,0);
 	}
 	if ((u = add_udv_by_name("MOUSE_RULER_Y"))) {
+	    u->udv_undef = FALSE;
 	    Gcomplex(&u->udv_value,ruler.y,0);
 	}
 	if (display_ipc_commands()) {
@@ -1406,6 +1373,11 @@ event_keypress(struct gp_event_t *ge, TBOOLEAN current)
      * which is a bad thing if you are in the  middle of a mousing operation.
      */
 
+#ifdef _Windows
+    if (paused_for_mouse & PAUSE_KEYSTROKE)
+	kill_pending_Pause_dialog();
+#endif
+    
     if ((paused_for_mouse & PAUSE_KEYSTROKE) && (c > '\0') && current) {
 	load_mouse_variables(x, y, FALSE, c);
 	return;
@@ -1415,6 +1387,7 @@ event_keypress(struct gp_event_t *ge, TBOOLEAN current)
 	if (bind_matches(&keypress, ptr)) {
 	    struct udvt_entry *keywin;
 	    if ((keywin = add_udv_by_name("MOUSE_KEY_WINDOW"))) {
+		keywin->udv_undef = FALSE;
 		Ginteger(&keywin->udv_value, ge->winid);
 	    }
 	    /* Always honor keys set with "bind all" */
@@ -1510,32 +1483,18 @@ int is_mouse_outside_plot(void)
 #undef CHECK_AXIS_OUTSIDE
 }
 
-/* Return a new upper or lower axis limit that is a linear
- * combination of the current limits.
- */
+/* Return a new (upper or lower) axis limit that is a linear
+   combination of the current limits.  */
 static double
 rescale(int AXIS, double w1, double w2) 
 {
-    double newlimit;
-    struct axis *axis = &axis_array[AXIS];
-    double axmin = axis->min;
-    double axmax = axis->max;
-
-    if (nonlinear(axis)) {
-	axmin = eval_link_function(axis->linked_to_primary, axmin);
-	axmax = eval_link_function(axis->linked_to_primary, axmax);
-    }
-
-    newlimit = w1*axmin + w2*axmax;
-
-    if (nonlinear(axis))
-	newlimit = eval_link_function(axis->linked_to_primary->linked_to_secondary, newlimit);
-    else
-	newlimit = AXIS_DE_LOG_VALUE(AXIS,newlimit);
-
-    return newlimit;
+  double logval, val;
+  logval = w1*axis_array[AXIS].min+w2*axis_array[AXIS].max;
+  val = AXIS_DE_LOG_VALUE(AXIS,logval);
+  return val;
 }
 
+
 /* Rescale axes and do zoom. */
 static void
 zoom_rescale_xyx2y2(double a0,double a1,double a2,double a3,double a4,double a5,double a6,
@@ -1620,35 +1579,18 @@ do_zoom_scroll_down()
 }
 
 
-/* Return new lower and upper axis limits from expanding current limits
- * relative to current mouse position.
- */
+/* Return new lower and upper axis limits as current limits resized
+   around current mouse position. */
 static void
 rescale_around_mouse(double *newmin, double *newmax, int AXIS, double mouse_pos, double scale)
 {
-    struct axis *axis = &axis_array[AXIS];
-    struct axis *primary = axis->linked_to_primary;
-    double axmin = axis->min;
-    double axmax = axis->max;
+  double unlog_pos = AXIS_LOG_VALUE(AXIS, mouse_pos);
 
-    if (nonlinear(axis)) {
-	axmin = eval_link_function(primary, axmin);
-	axmax = eval_link_function(primary, axmax);
-	mouse_pos = eval_link_function(primary, mouse_pos);
-    } else {
-	mouse_pos = AXIS_LOG_VALUE(AXIS, mouse_pos);
-    }
+  *newmin = unlog_pos + (axis_array[AXIS].min - unlog_pos) * scale;
+  *newmin = AXIS_DE_LOG_VALUE(AXIS,*newmin);
 
-  *newmin = mouse_pos + (axmin - mouse_pos) * scale;
-  *newmax = mouse_pos + (axmax - mouse_pos) * scale;
-
-    if (nonlinear(axis)) {
-	*newmin = eval_link_function(primary->linked_to_secondary, *newmin);
-	*newmax = eval_link_function(primary->linked_to_secondary, *newmax);
-    } else {
-      *newmin = AXIS_DE_LOG_VALUE(AXIS,*newmin);
-      *newmax = AXIS_DE_LOG_VALUE(AXIS,*newmax);
-    }
+  *newmax = unlog_pos + (axis_array[AXIS].max - unlog_pos) * scale;
+  *newmax = AXIS_DE_LOG_VALUE(AXIS,*newmax);
 }
 
 
@@ -1874,12 +1816,13 @@ event_buttonpress(struct gp_event_t *ge)
     } else if (ALMOST2D) {
         if (!setting_zoom_region) {
 	    if (1 == b) {
-		/* "pause button1" or "pause any" takes precedence over key bindings */
+	      /* "pause button1" or "pause any" takes precedence over key bindings */
 		if (paused_for_mouse & PAUSE_BUTTON1) {
 		    load_mouse_variables(mouse_x, mouse_y, TRUE, b);
 		    trap_release = TRUE;	/* Don't trigger on release also */
 		    return;
 		}
+
 	    } else if (2 == b) {
 		/* not bound in 2d graphs */
 	    } else if (3 == b && 
@@ -1983,7 +1926,7 @@ event_buttonpress(struct gp_event_t *ge)
 	}
     } else {
 	if (term->set_cursor) {
-	    if (button & (1 << 1) || button & (1 << 3))
+	    if (button & (1 << 1))
 		term->set_cursor(1, 0, 0);
 	    else if (button & (1 << 2))
 		term->set_cursor(2, 0, 0);
@@ -2071,15 +2014,14 @@ event_buttonrelease(struct gp_event_t *ge)
 	    }
 	}
     }
-    if (is_3d_plot && (b == 1 || b == 2 || b == 3)) {
+    if (is_3d_plot && (b == 1 || b == 2)) {
 	if (!!(modifier_mask & Mod_Ctrl) && !needreplot) {
 	    /* redraw the 3d plot if its last redraw was 'quick'
 	     * (only axes) because modifier key was pressed */
 	    do_save_3dplot(first_3dplot, plot3d_num, 0);
 	}
 	if (term->set_cursor)
-	    term->set_cursor((button & (1 << 1)) ? 1 : (button & (1 << 2)) ? 2 : 0,
-				0, 0);
+	    term->set_cursor(0, 0, 0);
     }
 
     /* Export current mouse coords to user-accessible variables also */
@@ -2094,8 +2036,17 @@ event_buttonrelease(struct gp_event_t *ge)
 	event_keypress(ge, TRUE);
 	ge->par1 = save;	/* needed for "pause mouse" */
     }
-}
 
+#ifdef _Windows
+    if (paused_for_mouse & PAUSE_CLICK) {
+	/* remove pause message box after 'pause mouse' */
+#ifndef WGP_CONSOLE
+	paused_for_mouse = 0;
+#endif
+	kill_pending_Pause_dialog();
+    }
+#endif
+}
 
 static void
 event_motion(struct gp_event_t *ge)
@@ -2128,8 +2079,8 @@ event_motion(struct gp_event_t *ge)
 	     * we compare the movement in x and y direction, and
 	     * change either scale or zscale */
 	    double relx, rely;
-	    relx = (double)abs(mouse_x - start_x) / (double)term->h_tic;
-	    rely = (double)abs(mouse_y - start_y) / (double)term->v_tic;
+	    relx = fabs(mouse_x - start_x) / term->h_tic;
+	    rely = fabs(mouse_y - start_y) / term->v_tic;
 
 	    if (modifier_mask & Mod_Shift) {
 		xyplane.z += (1 + fabs(xyplane.z))
@@ -2156,12 +2107,7 @@ event_motion(struct gp_event_t *ge)
 	    start_x = mouse_x;
 	    start_y = mouse_y;
 	    redraw = TRUE;
-	} else if (button & (1 << 3)) {
-	    /* dragging with button 3 -> change azimuth */
-	    azimuth += (mouse_x - start_x) * 90.0 / term->xmax;
-	    start_x = mouse_x;
-	    redraw = TRUE;
-	}
+	} /* if (mousebutton 2 is down) */
 
 	if (!ALMOST2D) {
 	    turn_ruler_off();
@@ -2178,7 +2124,6 @@ event_motion(struct gp_event_t *ge)
 		fill_gpval_float("GPVAL_VIEW_ROT_Z", surface_rot_z);
 		fill_gpval_float("GPVAL_VIEW_SCALE", surface_scale);
 		fill_gpval_float("GPVAL_VIEW_ZSCALE", surface_zscale);
-		fill_gpval_float("GPVAL_VIEW_AZIMUTH", azimuth);
 	    } else {
 		/* postpone the replotting */
 		needreplot = TRUE;
@@ -2246,8 +2191,8 @@ event_reset(struct gp_event_t *ge)
 
     if (paused_for_mouse) {
 	paused_for_mouse = 0;
-#ifdef WIN32
-	/* close pause message box */
+#ifdef _Windows
+	/* remove pause message box after 'pause mouse' */
 	kill_pending_Pause_dialog();
 #endif
 	/* This hack is necessary on some systems in order to prevent one  */
@@ -2317,15 +2262,8 @@ do_event(struct gp_event_t *ge)
 	event_buttonrelease(ge);
 	break;
     case GE_replot:
-	/* auto-generated replot (e.g. from replot-on-resize) */
-	/* FIXME: more terminals should use this! */
-	if (replot_line == NULL || replot_line[0] == '\0')
-	    break;
-	if (!strncmp(replot_line,"test",4))
-	    break;
-	if (multiplot)
-	    break;
-	do_string_replot("");
+	/* used only by ggi.trm */
+	do_string("replot");
 	break;
     case GE_reset:
 	event_reset(ge);
@@ -2374,12 +2312,8 @@ do_event(struct gp_event_t *ge)
     case GE_buttonrelease_old:
 	/* ignore */
 	break;
-    case GE_raise:
-	/* FIXME: No generic routine implemented! */
-	/* Individual terminal types must handle it themselves if they care */
-	break;
     default:
-	fprintf(stderr, "%s:%d unrecognized event type %d\n", __FILE__, __LINE__, ge->type);
+	fprintf(stderr, "%s:%d protocol error\n", __FILE__, __LINE__);
 	break;
     }
 
@@ -2389,11 +2323,9 @@ do_event(struct gp_event_t *ge)
 static void
 do_save_3dplot(struct surface_points *plots, int pcount, int quick)
 {
-#if (0)
 #define M_TEST_AXIS(A) \
      (A.log && ((!(A.set_autoscale & AUTOSCALE_MIN) && A.set_min <= 0) || \
 		(!(A.set_autoscale & AUTOSCALE_MAX) && A.set_max <= 0)))
-#endif
 
     if (!plots || (E_REFRESH_NOT_OK == refresh_ok)) {
 	/* !plots might happen after the `reset' command for example
@@ -2403,14 +2335,12 @@ do_save_3dplot(struct surface_points *plots, int pcount, int quick)
 	 */
 	replotrequest();
     } else {
-#if (0)	/* Dead code.  This error is now trapped elsewhere */
 	if (M_TEST_AXIS(X_AXIS) || M_TEST_AXIS(Y_AXIS) || M_TEST_AXIS(Z_AXIS)
 	    || M_TEST_AXIS(CB_AXIS)
 	    ) {
-		int_error(NO_CARET, "axis ranges must be above 0 for log scale!");
+		graph_error("axis ranges must be above 0 for log scale!");
 		return;
 	}
-#endif
 	do_3dplot(plots, pcount, quick);
     }
 
@@ -2509,9 +2439,6 @@ bind_scan_lhs(bind_t * out, const char *in)
 	} else if (!strncasecmp(ptr, "ctrl-", 5)) {
 	    out->modifier |= Mod_Ctrl;
 	    ptr += 5;
-	} else if (!strncasecmp(ptr, "shift-", 6)) {
-	    out->modifier |= Mod_Shift;
-	    ptr += 6;
 	} else if (NO_KEY != (itmp = lookup_key(ptr, &len))) {
 	    out->key = itmp;
 	    ptr += len;
@@ -2541,10 +2468,7 @@ bind_fmt_lhs(const bind_t * in)
 	sprintf(out, "Ctrl-");
     }
     if (in->modifier & Mod_Alt) {
-	strcat(out, "Alt-");
-    }
-    if (in->modifier & Mod_Shift) {
-	strcat(out, "Shift-");
+	strcat(out, "%sAlt-");
     }
     if (in->key > GP_FIRST_KEY && in->key < GP_LAST_KEY) {
 	strcat(out,special_keys[in->key - GP_FIRST_KEY]);
@@ -2569,14 +2493,9 @@ bind_fmt_lhs(const bind_t * in)
 static int
 bind_matches(const bind_t * a, const bind_t * b)
 {
-    int a_mod = a->modifier;
-    int b_mod = b->modifier;
-
-    /* discard Shift modifier (except for mouse button) */
-    if (a->key != GP_Button1) {
-	a_mod &= (Mod_Ctrl | Mod_Alt);
-	b_mod &= (Mod_Ctrl | Mod_Alt);
-    }
+    /* discard Shift modifier */
+    int a_mod = a->modifier & (Mod_Ctrl | Mod_Alt);
+    int b_mod = b->modifier & (Mod_Ctrl | Mod_Alt);
 
     if (a->key == b->key && a_mod == b_mod)
 	return 1;
@@ -2587,10 +2506,10 @@ bind_matches(const bind_t * a, const bind_t * b)
 static void
 bind_display_one(bind_t * ptr)
 {
-    fprintf(stderr, " %-13s ", bind_fmt_lhs(ptr));
+    fprintf(stderr, " %-12s ", bind_fmt_lhs(ptr));
     fprintf(stderr, "%c ", ptr->allwindows ? '*' : ' ');
     if (ptr->command) {
-	fprintf(stderr, "`%s`\n", ptr->command);
+	fprintf(stderr, "`%s'\n", ptr->command);
     } else if (ptr->builtin) {
 	fprintf(stderr, "%s\n", ptr->builtin(0));
     } else {
@@ -2613,7 +2532,8 @@ bind_display(char *lhs)
 	char fmt[] = " %-17s  %s\n";
 	fprintf(stderr, "\n");
 	/* mouse buttons */
-	fprintf(stderr, fmt, "<B1> doubleclick", "send mouse coordinates to clipboard (pm win wxt x11)");
+	fprintf(stderr, fmt, "2x<B1>",
+		"print coordinates to clipboard using `clipboardformat`\n                    (see keys '3', '4')");
 	fprintf(stderr, fmt, "<B2>", "annotate the graph using `mouseformat` (see keys '1', '2')");
 	fprintf(stderr, fmt, "", "or draw labels if `set mouse labels is on`");
 	fprintf(stderr, fmt, "<Ctrl-B2>", "remove label close to pointer if `set mouse labels` is on");
@@ -2621,7 +2541,6 @@ bind_display(char *lhs)
 	fprintf(stderr, fmt, "<B1-Motion>", "change view (rotation); use <Ctrl> to rotate the axes only");
 	fprintf(stderr, fmt, "<B2-Motion>", "change view (scaling); use <Ctrl> to scale the axes only");
 	fprintf(stderr, fmt, "<Shift-B2-Motion>", "vertical motion -- change xyplane");
-	fprintf(stderr, fmt, "<B3-Motion>", "change view (azimuth)");
 
 	/* mouse wheel */
 	fprintf(stderr, fmt, "<wheel-up>", "  scroll up (in +Y direction)");
@@ -2635,10 +2554,8 @@ bind_display(char *lhs)
 
 	fprintf(stderr, "\n");
 	/* keystrokes */
-#if (0)	/* Not implemented in the core code! */
 #ifndef DISABLE_SPACE_RAISES_CONSOLE
 	fprintf(stderr, " %-12s   %s\n", "Space", "raise gnuplot console window");
-#endif
 #endif
 	fprintf(stderr, " %-12s * %s\n", "q", "close this plot window");
 	fprintf(stderr, "\n");
@@ -2887,9 +2804,9 @@ turn_ruler_off()
 	    (*term->set_ruler) (-1, -1);
 	}
 	if ((u = add_udv_by_name("MOUSE_RULER_X")))
-	    u->udv_value.type = NOTDEFINED;
+	    u->udv_undef = TRUE;
 	if ((u = add_udv_by_name("MOUSE_RULER_Y")))
-	    u->udv_value.type = NOTDEFINED;
+	    u->udv_undef = TRUE;
 	if (display_ipc_commands()) {
 	    fprintf(stderr, "turning ruler off.\n");
 	}
@@ -2897,7 +2814,7 @@ turn_ruler_off()
 }
 
 static int
-nearest_label_tag(int xref, int yref)
+nearest_label_tag(int xref, int yref, struct termentry *t)
 {
     double min = -1;
     int min_tag = -1;
@@ -2923,7 +2840,7 @@ nearest_label_tag(int xref, int yref)
 	     * threshold around the label */
 	    double tic_diff_squared;
 	    int htic, vtic;
-	    get_offsets(this_label, &htic, &vtic);
+	    get_offsets(this_label, t, &htic, &vtic);
 	    tic_diff_squared = htic * htic + vtic * vtic;
 	    if (diff_squared < tic_diff_squared) {
 		min = diff_squared;
@@ -2938,7 +2855,7 @@ nearest_label_tag(int xref, int yref)
 static void
 remove_label(int x, int y)
 {
-    int tag = nearest_label_tag(x, y);
+    int tag = nearest_label_tag(x, y, term);
     if (-1 != tag) {
 	char cmd[0x40];
 	sprintf(cmd, "unset label %d", tag);
@@ -2988,39 +2905,48 @@ load_mouse_variables(double x, double y, TBOOLEAN button, int c)
     MousePosToGraphPosReal(x, y, &real_x, &real_y, &real_x2, &real_y2);
 
     if ((current = add_udv_by_name("MOUSE_BUTTON"))) {
+	current->udv_undef = !button;
 	Ginteger(&current->udv_value, button?c:-1);
-	if (!button)
-	    current->udv_value.type = NOTDEFINED;
     }
     if ((current = add_udv_by_name("MOUSE_KEY"))) {
+	current->udv_undef = FALSE;
 	Ginteger(&current->udv_value,c);
     }
     if ((current = add_udv_by_name("MOUSE_CHAR"))) {
 	char *keychar = gp_alloc(2,"key_char");
 	keychar[0] = c;
 	keychar[1] = '\0';
-	gpfree_string(&current->udv_value);
+	if (!current->udv_undef)
+	    gpfree_string(&current->udv_value);
+	current->udv_undef = FALSE;
 	Gstring(&current->udv_value,keychar);
     }
     if ((current = add_udv_by_name("MOUSE_X"))) {
+	current->udv_undef = FALSE;
 	Gcomplex(&current->udv_value,real_x,0);
     }
     if ((current = add_udv_by_name("MOUSE_Y"))) {
+	current->udv_undef = FALSE;
 	Gcomplex(&current->udv_value,real_y,0);
     }
     if ((current = add_udv_by_name("MOUSE_X2"))) {
+	current->udv_undef = FALSE;
 	Gcomplex(&current->udv_value,real_x2,0);
     }
     if ((current = add_udv_by_name("MOUSE_Y2"))) {
+	current->udv_undef = FALSE;
 	Gcomplex(&current->udv_value,real_y2,0);
     }
     if ((current = add_udv_by_name("MOUSE_SHIFT"))) {
+	current->udv_undef = FALSE;
 	Ginteger(&current->udv_value, modifier_mask & Mod_Shift);
     }
     if ((current = add_udv_by_name("MOUSE_ALT"))) {
+	current->udv_undef = FALSE;
 	Ginteger(&current->udv_value, modifier_mask & Mod_Alt);
     }
     if ((current = add_udv_by_name("MOUSE_CTRL"))) {
+	current->udv_undef = FALSE;
 	Ginteger(&current->udv_value, modifier_mask & Mod_Ctrl);
     }
     return;
