@@ -62,8 +62,7 @@ GEZoomStart, GEZoomStop, GERuler, GECopyClipboard, GEMove, GEVector, GELineTo,
 GESetFont, GEEnhancedFlush, GEEnhancedFinish, GEImage, GESetSceneSize, GERaise,
 GEWrapCursor, GEScale, GEActivate, GEDesactivate, GELayer, GEPlotNumber, GEHypertext,
 GETextBox, GEModPlots, GEAfterPlot, GEFontMetricRequest, GEDashPattern,
-GEDone,
-GEPID
+GEDone
 };
 
 enum QtGnuplotModPlots {
