@@ -1,3 +1,7 @@
+/*
+ * $Id: setshow.h,v 1.50 2014/03/23 12:17:59 markisch Exp $
+ */
+
 /* GNUPLOT - setshow.h */
 
 /*[
@@ -40,33 +44,47 @@
 #include "gadgets.h"
 #include "term_api.h"
 
+#define PROGRAM "G N U P L O T"  /* FIXME: move to show.c! */
+
+#define SAVE_NUM_OR_TIME(fp, x, axis)				\
+do{								\
+    if (axis_array[axis].datatype == DT_TIMEDATE) {		\
+	char s[80];						\
+								\
+	putc('"', fp);						\
+	gstrftime(s,80,axis_array[axis].timefmt,(double)(x));	\
+	fputs(conv_text(s), fp);				\
+	putc('"', fp);						\
+    } else {							\
+	fprintf(fp,"%#g",x);					\
+    }								\
+} while(0)
+
+
 /* The set and show commands, in setshow.c */
-void set_command(void);
-void unset_command(void);
-void reset_command(void);
-void show_command(void);
+void set_command __PROTO((void));
+void unset_command __PROTO((void));
+void reset_command __PROTO((void));
+void show_command __PROTO((void));
 /* and some accessible support functions */
-void show_version(FILE *fp);
-void set_format(void);
-void set_colorsequence(int option);
-char *conv_text(const char *s);
-void delete_linestyle(struct linestyle_def **, struct linestyle_def *, struct linestyle_def *);
-void delete_dashtype(struct custom_dashtype_def *, struct custom_dashtype_def *);
-/* void delete_arrowstyle(struct arrowstyle_def *, struct arrowstyle_def *); */
-void reset_key(void);
-void free_marklist(struct ticmark * list);
+void show_version __PROTO((FILE *fp));
+void set_format __PROTO((void));
+void set_colorsequence __PROTO((int option));
+char *conv_text __PROTO((const char *s));
+void delete_linestyle __PROTO((struct linestyle_def **, struct linestyle_def *, struct linestyle_def *));
+void delete_dashtype __PROTO((struct custom_dashtype_def *, struct custom_dashtype_def *));
+/* void delete_arrowstyle __PROTO((struct arrowstyle_def *, struct arrowstyle_def *)); */
+void reset_key __PROTO((void));
+void free_marklist __PROTO((struct ticmark * list));
 extern int enable_reset_palette;
-void reset_palette(void);
-void reset_bars(void);
-void rrange_to_xy(void);
-void unset_monochrome(void);
-void unset_all_tics(void);
+void reset_palette __PROTO((void));
+void rrange_to_xy __PROTO((void));
 
 /* Called from set_label(), plot2d.c and plot3d.c */
-extern void parse_label_options(struct text_label *, int ndim);
-extern struct text_label * new_text_label(int tag);
-extern void disp_value(FILE *, struct value *, TBOOLEAN);
-extern struct ticmark * prune_dataticks(struct ticmark *list);
+extern void parse_label_options __PROTO((struct text_label *this_label, TBOOLEAN));
+extern struct text_label * new_text_label __PROTO((int tag));
+extern void disp_value __PROTO((FILE *, struct value *, TBOOLEAN));
+extern struct ticmark * prune_dataticks __PROTO((struct ticmark *list));
 
 
 #endif /* GNUPLOT_SETSHOW_H */
