@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: stats.c,v 1.14.2.3 2014/10/16 16:46:19 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - stats.c */
 
 /*
@@ -45,16 +49,13 @@
 #define INITIAL_DATA_SIZE (4096)   /* initial size of data arrays */
 
 static int comparator __PROTO(( const void *a, const void *b ));
-static struct file_stats analyze_file __PROTO(( long n, int outofrange, int invalid, int blank, int dblblank, int headers ));
+static struct file_stats analyze_file __PROTO(( long n, int outofrange, int invalid, int blank, int dblblank ));
 static struct sgl_column_stats analyze_sgl_column __PROTO(( double *data, long n, long nr ));
 static struct two_column_stats analyze_two_columns __PROTO(( double *x, double *y,
 							     struct sgl_column_stats res_x,
 							     struct sgl_column_stats res_y,
 							     long n ));
 
-static void clear_one_var __PROTO(( char *prefix, char *base ));
-static void clear_stats_variables __PROTO(( char *prefix ));
-
 static void ensure_output __PROTO((void));
 static char* fmt __PROTO(( char *buf, double val ));
 static void sgl_column_output_nonformat __PROTO(( struct sgl_column_stats s, char *x ));
@@ -66,10 +67,6 @@ static void two_column_output __PROTO(( struct sgl_column_stats x,
 
 static void create_and_set_var __PROTO(( double val, char *prefix,
 					 char *base, char *suffix ));
-static void create_and_set_int_var __PROTO(( int ival, char *prefix,
-					 char *base, char *suffix ));
-static void create_and_store_var __PROTO(( t_value *data, char *prefix,
-					 char *base, char *suffix ));
 
 static void sgl_column_variables __PROTO(( struct sgl_column_stats res,
 					   char *prefix, char *postfix ));
@@ -93,8 +90,6 @@ struct file_stats {
     long invalid;
     long outofrange;
     long blocks;  /* blocks are separated by double blank lines */
-    long columnheaders;
-    int  columns;
 };
 
 struct sgl_column_stats {
@@ -165,7 +160,7 @@ comparator( const void *a, const void *b )
 }
 
 static struct file_stats
-analyze_file( long n, int outofrange, int invalid, int blank, int dblblank, int headers )
+analyze_file( long n, int outofrange, int invalid, int blank, int dblblank )
 {
     struct file_stats res;
 
@@ -174,8 +169,6 @@ analyze_file( long n, int outofrange, int invalid, int blank, int dblblank, int
     res.blanks  = blank;
     res.blocks  = dblblank + 1;  /* blocks are separated by dbl blank lines */
     res.outofrange = outofrange;
-    res.columnheaders = headers;
-    res.columns = df_last_col;
 
     return res;
 }
@@ -210,7 +203,7 @@ analyze_sgl_column( double *data, long n, long nc )
     }
 
     /* Mean and centre of gravity */
-    for (i=0; i<n; i++) {
+    for( i=0; i<n; i++ ) {
 	s  += data[i];
 	s2 += data[i]*data[i];
 	if ( nc > 0 ) {
@@ -224,9 +217,9 @@ analyze_sgl_column( double *data, long n, long nc )
     res.sum_sq = s2;
 
     /* Standard deviation, mean absolute deviation, skewness, and kurtosis */
-    for (i=0; i<n; i++) {
+    for( i=0; i<n; i++ ) {
 	double t = data[i] - res.mean;
-	ad += fabs(t);
+	ad += abs(t);
 	d  += t;
 	d2 += t*t;
 	d3 += t*t*t;
@@ -252,7 +245,7 @@ analyze_sgl_column( double *data, long n, long nc )
     /* sample standard deviation */
     res.ssd = res.stddev * sqrt((double)(n) / (double)(n-1));
 
-    for (i=0; i<n; i++) {
+    for( i=0; i<n; i++ ) {
 	tmp[i].val = data[i];
 	tmp[i].index = i;
     }
@@ -303,7 +296,7 @@ analyze_two_columns( double *x, double *y,
     double s = 0;
     double ssyy, ssxx, ssxy;
 
-    for (i=0; i<n; i++) {
+    for( i=0; i<n; i++ ) {
 	s += x[i] * y[i];
     }
     res.sum_xy = s;
@@ -393,7 +386,6 @@ file_output( struct file_stats s )
 	fprintf( print_out, "%s\t%ld\n", "invalid", s.invalid );
 	fprintf( print_out, "%s\t%ld\n", "blanks", s.blanks );
 	fprintf( print_out, "%s\t%ld\n", "blocks", s.blocks );
-	fprintf( print_out, "%s\t%ld\n", "columnheaders", s.columnheaders );
 	fprintf( print_out, "%s\t%ld\n", "outofrange", s.outofrange );
 	return;
     }
@@ -404,7 +396,6 @@ file_output( struct file_stats s )
     fprintf( print_out, "  Records:           %*ld\n", width, s.records );
     fprintf( print_out, "  Out of range:      %*ld\n", width, s.outofrange );
     fprintf( print_out, "  Invalid:           %*ld\n", width, s.invalid );
-    fprintf( print_out, "  Column headers:    %*ld\n", width, s.columnheaders );
     fprintf( print_out, "  Blank:             %*ld\n", width, s.blanks );
     fprintf( print_out, "  Data Blocks:       %*ld\n", width, s.blocks );
 }
@@ -595,97 +586,16 @@ two_column_output( struct sgl_column_stats x,
    Variable Handling
    ================================================================= */
 
-static void
-clear_one_var( char *prefix, char *base )
-{
-    int len;
-    char *varname;
-
-    len = strlen(prefix) + strlen(base) + 2;
-    varname = (char *)gp_alloc( len, "create_and_set_var" );
-    sprintf( varname, "%s_%s", prefix, base );
-
-    del_udv_by_name(varname, TRUE);
-
-    free( varname );
-}
-
-static void
-clear_stats_variables( char *prefix )
-{
-    /* file variables */
-    clear_one_var( prefix, "records" );
-    clear_one_var( prefix, "invalid" );
-    clear_one_var( prefix, "headers" );
-    clear_one_var( prefix, "blank" );
-    clear_one_var( prefix, "blocks" );
-    clear_one_var( prefix, "outofrange" );
-    clear_one_var( prefix, "columns" );
-
-    /* one column variables */
-    clear_one_var( prefix, "mean" );
-    clear_one_var( prefix, "stddev" );
-    clear_one_var( prefix, "ssd" );
-    clear_one_var( prefix, "skewness" );
-    clear_one_var( prefix, "kurtosis" );
-    clear_one_var( prefix, "adev" );
-    clear_one_var( prefix, "mean_err" );
-    clear_one_var( prefix, "stddev_err" );
-    clear_one_var( prefix, "skewness_err" );
-    clear_one_var( prefix, "kurtosis_err" );
-    clear_one_var( prefix, "sum" );
-    clear_one_var( prefix, "sumsq" );
-    clear_one_var( prefix, "min" );
-    clear_one_var( prefix, "max" );
-    clear_one_var( prefix, "median" );
-    clear_one_var( prefix, "lo_quartile" );
-    clear_one_var( prefix, "up_quartile" );
-    clear_one_var( prefix, "index_min" );
-    clear_one_var( prefix, "index_max" );
-
-    /* matrix variables */
-    clear_one_var( prefix, "index_min_x" );
-    clear_one_var( prefix, "index_min_y" );
-    clear_one_var( prefix, "index_max_x" );
-    clear_one_var( prefix, "index_max_y" );
-    clear_one_var( prefix, "size_x" );
-    clear_one_var( prefix, "size_y" );
-
-    /* two column variables */
-    clear_one_var( prefix, "slope" );
-    clear_one_var( prefix, "intercept" );
-    clear_one_var( prefix, "slope_err" );
-    clear_one_var( prefix, "intercept_err" );
-    clear_one_var( prefix, "correlation" );
-    clear_one_var( prefix, "sumxy" );
-    clear_one_var( prefix, "pos_min_y" );
-    clear_one_var( prefix, "pos_max_y" );
-
-}
-
 static void
 create_and_set_var( double val, char *prefix, char *base, char *suffix )
-{
-    t_value data;
-    Gcomplex( &data, val, 0.0 ); /* data is complex, real=val, imag=0.0 */
-    create_and_store_var( &data, prefix, base, suffix );
-}
-
-static void
-create_and_set_int_var( int ival, char *prefix, char *base, char *suffix )
-{
-    t_value data;
-    Ginteger( &data, ival );
-    create_and_store_var( &data, prefix, base, suffix );
-}
-
-static void
-create_and_store_var( t_value *data, char *prefix, char *base, char *suffix )
 {
     int len;
     char *varname;
     struct udvt_entry *udv_ptr;
 
+    t_value data;
+    Gcomplex( &data, val, 0.0 ); /* data is complex, real=val, imag=0.0 */
+
     /* In case prefix (or suffix) is NULL - make them empty strings */
     prefix = prefix ? prefix : "";
     suffix = suffix ? suffix : "";
@@ -699,7 +609,8 @@ create_and_store_var( t_value *data, char *prefix, char *base, char *suffix )
      * its own copy of the varname.
      */
     udv_ptr = add_udv_by_name(varname);
-    udv_ptr->udv_value = *data;
+    udv_ptr->udv_value = data;
+    udv_ptr->udv_undef = FALSE;
 
     free( varname );
 }
@@ -708,13 +619,12 @@ static void
 file_variables( struct file_stats s, char *prefix )
 {
     /* Suffix does not make sense here! */
-    create_and_set_int_var( s.records, prefix, "records", "" );
-    create_and_set_int_var( s.invalid, prefix, "invalid", "" );
-    create_and_set_int_var( s.columnheaders, prefix, "headers", "" );
-    create_and_set_int_var( s.blanks,  prefix, "blank",   "" );
-    create_and_set_int_var( s.blocks,  prefix, "blocks",  "" );
-    create_and_set_int_var( s.outofrange, prefix, "outofrange", "" );
-    create_and_set_int_var( s.columns, prefix, "columns", "" );
+    create_and_set_var( s.records, prefix, "records", "" );
+    create_and_set_var( s.invalid, prefix, "invalid", "" );
+    create_and_set_var( s.blanks,  prefix, "blank",   "" );
+    create_and_set_var( s.blocks,  prefix, "blocks",  "" );
+    create_and_set_var( s.outofrange, prefix, "outofrange", "" );
+    create_and_set_var( df_last_col, prefix, "columns", "" );
 }
 
 static void
@@ -740,18 +650,18 @@ sgl_column_variables( struct sgl_column_stats s, char *prefix, char *suffix )
 
     /* If data set is matrix */
     if ( s.sx > 0 ) {
-	create_and_set_int_var( (s.min.index) % s.sx, prefix, "index_min_x", suffix );
-	create_and_set_int_var( (s.min.index) / s.sx, prefix, "index_min_y", suffix );
-	create_and_set_int_var( (s.max.index) % s.sx, prefix, "index_max_x", suffix );
-	create_and_set_int_var( (s.max.index) / s.sx, prefix, "index_max_y", suffix );
-	create_and_set_int_var( s.sx, prefix, "size_x", suffix );
-	create_and_set_int_var( s.sy, prefix, "size_y", suffix );
+	create_and_set_var( (s.min.index) % s.sx, prefix, "index_min_x", suffix );
+	create_and_set_var( (s.min.index) / s.sx, prefix, "index_min_y", suffix );
+	create_and_set_var( (s.max.index) % s.sx, prefix, "index_max_x", suffix );
+	create_and_set_var( (s.max.index) / s.sx, prefix, "index_max_y", suffix );
+	create_and_set_var( s.sx, prefix, "size_x", suffix );
+	create_and_set_var( s.sy, prefix, "size_y", suffix );
     } else {
 	create_and_set_var( s.median,         prefix, "median",      suffix );
 	create_and_set_var( s.lower_quartile, prefix, "lo_quartile", suffix );
 	create_and_set_var( s.upper_quartile, prefix, "up_quartile", suffix );
-	create_and_set_int_var( s.min.index, prefix, "index_min", suffix );
-	create_and_set_int_var( s.max.index, prefix, "index_max", suffix );
+	create_and_set_var( s.min.index, prefix, "index_min", suffix );
+	create_and_set_var( s.max.index, prefix, "index_max", suffix );
     }
 }
 
@@ -808,7 +718,6 @@ statsrequest(void)
     int columns;
     double v[2];
     static char *file_name = NULL;
-    char *temp_name;
 
     /* Vars to hold data and results */
     long n;                /* number of records retained */
@@ -819,87 +728,112 @@ statsrequest(void)
     long invalid;          /* number of missing/invalid records */
     long blanks;           /* number of blank lines */
     long doubleblanks;     /* number of repeated blank lines */
-    long columnheaders;    /* number of records treated as headers rather than data */
     long out_of_range;     /* number pts rejected, because out of range */
 
     struct file_stats res_file;
     struct sgl_column_stats res_x = {0}, res_y = {0};
     struct two_column_stats res_xy = {0};
 
+    float *matrix;            /* matrix data. This must be float. */
+    int nc, nr;               /* matrix dimensions. */
+    int index;
+
     /* Vars for variable handling */
     static char *prefix = NULL;       /* prefix for user-defined vars names */
-    TBOOLEAN prefix_from_columnhead = FALSE;
 
     /* Vars that control output */
     TBOOLEAN do_output = TRUE;     /* Generate formatted output */
-    TBOOLEAN array_data = FALSE;
 
     c_token++;
 
     /* Parse ranges */
-    axis_init(&axis_array[FIRST_X_AXIS], FALSE);
-    axis_init(&axis_array[FIRST_Y_AXIS], FALSE);
+    AXIS_INIT2D(FIRST_X_AXIS, 0);
+    AXIS_INIT2D(FIRST_Y_AXIS, 0);
     parse_range(FIRST_X_AXIS);
     parse_range(FIRST_Y_AXIS);
 
     /* Initialize */
     invalid = 0;          /* number of missing/invalid records */
     blanks = 0;           /* number of blank lines */
-    columnheaders = 0;    /* number of records treated as headers rather than data */
     doubleblanks = 0;     /* number of repeated blank lines */
     out_of_range = 0;     /* number pts rejected, because out of range */
     n = 0;                /* number of records retained */
+    nr = 0;               /* Matrix dimensions */
+    nc = 0;
     max_n = INITIAL_DATA_SIZE;
 
     free(data_x);
     free(data_y);
     data_x = vec(max_n);       /* start with max. value */
     data_y = vec(max_n);
-    free(prefix);
-    prefix = NULL;
 
     if ( !data_x || !data_y )
       int_error( NO_CARET, "Internal error: out of memory in stats" );
 
-    n = invalid = blanks = columnheaders = doubleblanks = out_of_range = 0;
+    n = invalid = blanks = doubleblanks = out_of_range = nr = 0;
 
     /* Get filename */
+    free ( file_name );
     i = c_token;
-    temp_name = string_or_express(NULL);
-    if (temp_name) {
-	free(file_name);
-	file_name = gp_strdup(temp_name);
-    } else
+    file_name = string_or_express(NULL);
+    if (file_name )
+	file_name = gp_strdup(file_name);
+    else
 	int_error(i, "missing filename or datablock");
 
-    /* Jan 2015: We used to handle ascii matrix data as a special case but
-     * the code did not work correctly.  Since df_read_matrix() dummies up
-     * ascii matrix data to look as if had been presented as a binary blob,
-     * we should be able to proceed with no special attention other than
-     * to set the effective number of columns to 1.
-     */
-    if (TRUE) {
-	df_set_plot_mode(MODE_PLOT);		/* Used for matrix datafiles */
-	columns = df_open(file_name, 2, NULL);	/* up to 2 using specs allowed */
+    /* ===========================================================
+    v923z: insertion for treating matrices
+      EAM: only handles ascii matrix with uniform grid,
+           and fails to apply any input data transforms
+      =========================================================== */
+    if ( almost_equals(c_token, "mat$rix") ) {
+	df_open(file_name, 3, NULL);
+	index = df_num_bin_records - 1;
 
-	if (columns < 0) {
-	    int_warn(NO_CARET, "Can't read data file");
-	    while (!END_OF_COMMAND)
-		c_token++;
-	    goto stats_cleanup;
+	/* We take these values as set by df_determine_matrix_info
+	See line 1996 in datafile.c */
+	nc = df_bin_record[index].scan_dim[0];
+	nr = df_bin_record[index].scan_dim[1];
+	n = nc * nr;
+
+	matrix = (float *)df_bin_record[index].memory_data;
+
+	/* Fill up a vector, so that we can use the existing code. */
+	/* FIXME: matrix code does not actually use data_x         */
+	if ( !redim_vec(&data_x, n) || !redim_vec(&data_y, n)) {
+	    int_error( NO_CARET,
+		   "Out of memory in stats: too many datapoints (%d)?", n );
 	}
+	for( i=0; i < n; i++ ) {
+	    data_y[i] = (double)matrix[i];
+	}
+	/* We can close the file here, there is nothing else to do */
+	df_close();
+	/* We will invoke single column statistics for the matrix */
+	columns = 1;
 
-	if (df_array && columns == 0)
-	    array_data = TRUE;
+    } else { /* Not a matrix */
+	columns = df_open(file_name, 2, NULL); /* up to 2 using specs allowed */
+
+	if (columns < 0)
+	    int_error(NO_CARET, "Can't read data file");
+
+	if (columns > 2 )
+	    int_error(c_token, "stats command can handle only 2 columns of data");
+
+	/* If the user has set an explicit locale for numeric input, apply it
+	   here so that it affects data fields read from the input file. */
+	/* v923z: where exactly should this be? here or before the matrix case?
+	 * I think, we should move everything here to before trying to open the file.
+	 * There is no point in trying to read anything, if the axis is logarithmic, e.g.
+	 */
+	set_numeric_locale();
 
 	/* For all these below: we could save the state, switch off, then restore */
-#if !defined(NONLINEAR_AXES) || (NONLINEAR_AXES == 0)
-	if (axis_array[FIRST_X_AXIS].log || axis_array[FIRST_Y_AXIS].log)
+	if ( axis_array[FIRST_X_AXIS].log || axis_array[FIRST_Y_AXIS].log )
 	    int_error( NO_CARET, "Stats command not available with logscale active");
-#endif
 
-	if (axis_array[FIRST_X_AXIS].datatype == DT_TIMEDATE
-	||  axis_array[FIRST_Y_AXIS].datatype == DT_TIMEDATE )
+	if ( axis_array[FIRST_X_AXIS].datatype == DT_TIMEDATE || axis_array[FIRST_Y_AXIS].datatype == DT_TIMEDATE )
 	    int_error( NO_CARET, "Stats command not available in timedata mode");
 
 	if ( polar )
@@ -908,69 +842,31 @@ statsrequest(void)
 	if ( parametric )
 	    int_error( NO_CARET, "Stats command not available in parametric mode" );
 
-	/* Parse the remainder of the command line */
-	while( !(END_OF_COMMAND) ) {
-	    if ( almost_equals( c_token, "out$put" ) ) {
-		    do_output = TRUE;
-		    c_token++;
-
-	    } else if ( almost_equals( c_token, "noout$put" ) ) {
-		    do_output = FALSE;
-		    c_token++;
-
-	    } else if ( almost_equals(c_token, "pre$fix")
-		   ||   equals(c_token, "name")) {
-		c_token++;
-		if (almost_equals(c_token,"col$umnheader")) {
-		    df_set_key_title_columnhead(NULL);
-		    prefix_from_columnhead = TRUE;
-		    continue;
-		}
-		prefix = try_to_get_string();
-		if (!legal_identifier(prefix) || !strcmp ("GPVAL_", prefix))
-		    int_error( --c_token, "illegal prefix" );
-
-	    }  else {
-		int_error( c_token, "Unrecognized stats option");
-	    }
-	}
-
-	/* Clear any previous variables STATS_* so that if we exit early */
-	/* they cannot be mistaken as resulting from the current analysis. */
-	clear_stats_variables(prefix ? prefix : "STATS");
-
-	/* If the user has set an explicit locale for numeric input, apply it */
-	/* here so that it affects data fields read from the input file. */
-	set_numeric_locale();
-
 	/* The way readline and friends work is as follows:
 	 - df_open will return the number of columns requested in the using spec
 	   so that "columns" will be 0, 1, or 2 (no using, using 1, using 1:2)
 	 - readline always returns the same number of columns (for us: 1 or 2)
-	 - using n:m = return two columns, skipping lines w/ bad data
-	 - using n   = return single column (supply zeros (0) for the second col)
-	 - no using  = first two columns if both are present on the first line of data
+	 - using 1:2 = return two columns, skipping lines w/ bad data
+	 - using 1   = return single column (supply zeros (0) for the second col)
+	 - no using (gnuplot version 5)
+		     = first two columns if both are present on the first line of data
 		       else first column only
 	 */
+
 	while( (i = df_readline(v, 2)) != DF_EOF ) {
 
 	    if ( n >= max_n ) {
 		max_n = (max_n * 3) / 2; /* increase max_n by factor of 1.5 */
 
 		/* Some of the reallocations went bad: */
-		if ( !redim_vec(&data_x, max_n) || !redim_vec(&data_y, max_n) ) {
+		if ( 0 || !redim_vec(&data_x, max_n) || !redim_vec(&data_y, max_n) ) {
 		    df_close();
 		    int_error( NO_CARET,
-		       "Out of memory in stats: too many datapoints (%d)?", max_n );
+		       "Out of memory in stats: too many datapoints (%d)?",
+		       max_n );
 		}
 	    } /* if (need to extend storage space) */
 
-	    /* FIXME: ascii "matrix" input from df_readline via df_readbinary does not
-	     * flag NaN values so we must check each returned value here
-	     */
-	    if (df_matrix && (i == 2) && isnan(v[1]))
-		i = DF_UNDEFINED;
-
 	    switch (i) {
 	    case DF_MISSING:
 	    case DF_UNDEFINED:
@@ -986,12 +882,8 @@ statsrequest(void)
 	      doubleblanks += 1;
 	      continue;
 
-	    case DF_COLUMN_HEADERS:
-	      columnheaders += 1;
-	      continue;
-
 	    case 0:
-	      int_warn( NO_CARET, "bad data on line %d of file %s",
+	      int_error( NO_CARET, "bad data on line %d of file %s",
 	  		df_line_number, df_filename ? df_filename : "" );
 	      break;
 
@@ -1016,11 +908,6 @@ statsrequest(void)
 	      }
 	      columns = 2;
 	      break;
-
-	    default: /* Who are these? */
-	      FPRINTF((stderr,"unhandled return code %d from df_readline\n", i));
-	      break;
-
 	    }
 	} /* end-while : done reading file */
 	df_close();
@@ -1028,31 +915,47 @@ statsrequest(void)
 	/* now resize fields to actual length: */
 	redim_vec(&data_x, n);
 	redim_vec(&data_y, n);
-    }
+
+    }  /* end of case when the data file is not a matrix */
 
     /* Now finished reading user input; return to C locale for internal use*/
     reset_numeric_locale();
 
+    /* PKJ - TODO: similar for logscale, polar/parametric, timedata */
+
     /* No data! Try to explain why. */
     if ( n == 0 ) {
 	if ( out_of_range > 0 )
-	    int_warn( NO_CARET, "All points out of range" );
+	    int_error( NO_CARET, "All points out of range" );
 	else
-	    int_warn( NO_CARET, "No valid data points found in file" );
-	goto stats_cleanup;
+	    int_error( NO_CARET, "No valid data points found in file" );
     }
 
-    /* The analysis variables are named STATS_* unless the user either */
-    /* gave a specific name or told us to use a columnheader.          */
-    if (!prefix && prefix_from_columnhead && df_key_title && *df_key_title) {
-	prefix = gp_strdup(df_key_title);
-	squash_spaces(prefix, 0);
-	if (!legal_identifier(prefix)) {
-	    int_warn(NO_CARET, "columnhead %s is not a valid prefix", prefix ? prefix : "");
-	    free(prefix);
-	    prefix = NULL;
+    /* Parse the remainder of the command line: 0 to 2 tokens possible */
+    while( !(END_OF_COMMAND) ) {
+	if ( almost_equals( c_token, "out$put" ) ) {
+		do_output = TRUE;
+		c_token++;
+
+	} else if ( almost_equals( c_token, "noout$put" ) ) {
+		do_output = FALSE;
+		c_token++;
+
+	} else if ( almost_equals(c_token, "pre$fix")
+	       ||   equals(c_token, "name")) {
+	    c_token++;
+	    free ( prefix );
+	    prefix = try_to_get_string();
+	    if (!legal_identifier(prefix) || !strcmp ("GPVAL_", prefix))
+		int_error( --c_token, "illegal prefix" );
+
+	}  else {
+	    int_error( c_token, "Expecting [no]output or prefix");
 	}
+
     }
+
+    /* Set defaults if not explicitly set by user */
     if (!prefix)
 	prefix = gp_strdup("STATS_");
     i = strlen(prefix);
@@ -1062,22 +965,12 @@ statsrequest(void)
     }
 
     /* Do the actual analysis */
-    res_file = analyze_file( n, out_of_range, invalid, blanks, doubleblanks, columnheaders );
-
-    /* Jan 2015: Revised detection and handling of matrix data */
-    if (array_data)
-	res_file.columns = columns = 1;
-
-    if (df_matrix) {
-	int nc = df_bin_record[df_num_bin_records-1].scan_dim[0];
+    res_file = analyze_file( n, out_of_range, invalid, blanks, doubleblanks );
+    if ( columns == 1 ) {
 	res_y = analyze_sgl_column( data_y, n, nc );
-	res_file.columns = nc;
-	columns = 1;
+    }
 
-    } else if (columns == 1) {
-	res_y = analyze_sgl_column( data_y, n, 0 );
-
-    } else {
+    if ( columns == 2 ) {
 	/* If there are two columns, then the data file is not a matrix */
 	res_x = analyze_sgl_column( data_x, n, 0 );
 	res_y = analyze_sgl_column( data_y, n, 0 );
@@ -1110,7 +1003,6 @@ statsrequest(void)
     }
 
     /* Cleanup */
-    stats_cleanup:
 
     free(data_x);
     free(data_y);
