@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: tables.c,v 1.133 2014/06/25 05:45:21 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - tables.c */
 
 /*[
@@ -40,8 +44,7 @@
 #include "util.h"
 #include "alloc.h"	/* for init_colornames() */
 #include "graph3d.h"	/* for DGRID3D_* options */
-#include "getcolor.h"
-#include "voxelgrid.h"
+# include "getcolor.h"
 
 /* gnuplot commands */
 
@@ -53,12 +56,9 @@ const struct gen_ftable command_ftbl[] =
 #ifdef USE_MOUSE
     { "bi$nd", bind_command },
 #endif
-    { "array", array_command },
-    { "break", break_command },
     { "ca$ll", call_command },
     { "cd", changedir_command },
     { "cl$ear", clear_command },
-    { "continue", continue_command },
     { "do", do_command },
     { "eval$uate", eval_command },
     { "ex$it", exit_command },
@@ -73,7 +73,6 @@ const struct gen_ftable command_ftbl[] =
     { "pa$use", pause_command },
     { "p$lot", plot_command },
     { "pr$int", print_command },
-    { "printerr$or", printerr_command },
     { "pwd", pwd_command },
     { "q$uit", exit_command },
     { "ref$resh", refresh_command },
@@ -89,13 +88,9 @@ const struct gen_ftable command_ftbl[] =
     { "st$ats", stats_command },
     { "sy$stem", system_command },
     { "test", test_command },
-    { "tog$gle", toggle_command },
     { "und$efine", undefine_command },
     { "uns$et", unset_command },
     { "up$date", update_command },
-    { "vclear", vclear_command },
-    { "vfill", vfill_command },
-    { "voxel", voxel_command },
     { "while", while_command },
     { "{", begin_clause },
     { "}", end_clause },
@@ -119,7 +114,6 @@ const struct gen_table plot_axes_tbl[] =
 const struct gen_table plot_smooth_tbl[] =
 {
     { "a$csplines", SMOOTH_ACSPLINES },
-    { "bins", SMOOTH_BINS },
     { "b$ezier", SMOOTH_BEZIER },
     { "c$splines", SMOOTH_CSPLINES },
     { "s$bezier", SMOOTH_SBEZIER },
@@ -130,8 +124,6 @@ const struct gen_table plot_smooth_tbl[] =
     { "k$density", SMOOTH_KDENSITY },
     { "cn$ormal", SMOOTH_CUMULATIVE_NORMALISED },
     { "mcs$plines", SMOOTH_MONOTONE_CSPLINE },
-    { "fnor$mal", SMOOTH_FREQUENCY_NORMALISED },
-    { "z$sort", SMOOTH_ZSORT },
     { NULL, SMOOTH_NONE }
 };
 
@@ -151,11 +143,10 @@ const struct gen_table dgrid3d_mode_tbl[] =
 /* 'save' command */
 const struct gen_table save_tbl[] =
 {
-    { "fit", SAVE_FIT },
-    { "fun$ctions", SAVE_FUNCS },
-    { "set", SAVE_SET },
-    { "ter$minal", SAVE_TERMINAL },
-    { "var$iables", SAVE_VARS },
+    { "f$unctions", SAVE_FUNCS },
+    { "s$et", SAVE_SET },
+    { "t$erminal", SAVE_TERMINAL },
+    { "v$ariables", SAVE_VARS },
     { NULL, SAVE_INVALID }
 };
 
@@ -171,7 +162,6 @@ const struct gen_table set_tbl[] =
     { "b$ars", S_BARS },
     { "bind", S_BIND },
     { "bor$der", S_BORDER },
-    { "boxdepth", S_BOXDEPTH },
     { "box$width", S_BOXWIDTH },
     { "cl$abel", S_CLABEL },
     { "c$lip", S_CLIP },
@@ -184,12 +174,10 @@ const struct gen_table set_tbl[] =
 
     { "data$file", S_DATAFILE },
 
-    { "debug", S_DEBUG },
     { "dg$rid3d", S_DGRID3D },
     { "du$mmy", S_DUMMY },
     { "enc$oding", S_ENCODING },
     { "dec$imalsign", S_DECIMALSIGN },
-    { "errorbars", S_BARS },
     { "fit", S_FIT },
     { "font$path", S_FONTPATH },
     { "fo$rmat", S_FORMAT },
@@ -199,10 +187,7 @@ const struct gen_table set_tbl[] =
     { "hid$den3d", S_HIDDEN3D },
     { "historysize", S_HISTORYSIZE },	/* Deprecated */
     { "his$tory", S_HISTORY },
-    { "pixmap$s", S_PIXMAP },
-    { "isosurf$ace", S_ISOSURFACE },
     { "is$osamples", S_ISOSAMPLES },
-    { "jitter", S_JITTER },
     { "k$ey", S_KEY },
     { "keyt$itle", S_KEY },
     { "la$bel", S_LABEL },
@@ -224,10 +209,9 @@ const struct gen_table set_tbl[] =
     { "tmar$gin", S_TMARGIN },
     { "bmar$gin", S_BMARGIN },
 
-    { "micro", S_MICRO },
-    { "minus$sign", S_MINUS_SIGN },
+#ifdef USE_MOUSE
     { "mo$use", S_MOUSE },
-    { "mono$chrome", S_MONOCHROME },
+#endif
     { "multi$plot", S_MULTIPLOT },
 
     { "mxt$ics", S_MXTICS },
@@ -241,18 +225,15 @@ const struct gen_table set_tbl[] =
     { "mzt$ics", S_MZTICS },
     { "nomzt$ics", S_NOMZTICS },
     { "mrt$ics", S_MRTICS },
-    { "mtt$ics", S_MTTICS },
+    { "nomrt$ics", S_NOMRTICS },
     { "mcbt$ics", S_MCBTICS },
     { "nomcbt$ics", S_NOMCBTICS },
-    { "nonlinear", S_NONLINEAR },
     { "of$fsets", S_OFFSETS },
     { "or$igin", S_ORIGIN },
     { "o$utput", SET_OUTPUT },
-    { "overflow", S_OVERFLOW },
     { "pa$rametric", S_PARAMETRIC },
     { "pm$3d", S_PM3D },
     { "pal$ette", S_PALETTE },
-    { "color", S_COLOR },
     { "colorb$ox", S_COLORBOX },
     { "colorn$ames", S_COLORNAMES },
     { "colors$equence", S_COLORSEQUENCE },
@@ -263,22 +244,18 @@ const struct gen_table set_tbl[] =
     { "pr$int", S_PRINT },
     { "psdir", S_PSDIR },
     { "obj$ect", S_OBJECT },
-    { "wall$s", S_WALL },
-    { "rgbmax", S_RGBMAX },
     { "sa$mples", S_SAMPLES },
     { "si$ze", S_SIZE },
     { "st$yle", S_STYLE },
     { "su$rface", S_SURFACE },
-    { "spider$plot", S_SPIDERPLOT },
     { "table", S_TABLE },
     { "t$erminal", S_TERMINAL },
     { "termopt$ions", S_TERMOPTIONS },
-    { "theta$0", S_THETA },
     { "ti$cs", S_TICS },
     { "ticsc$ale", S_TICSCALE },
     { "ticsl$evel", S_TICSLEVEL },
     { "timef$mt", S_TIMEFMT },
-    { "times$tamp", S_TIMESTAMP },
+    { "tim$estamp", S_TIMESTAMP },
     { "tit$le", S_TITLE },
     { "v$ariables", S_VARIABLES },
     { "ve$rsion", S_VERSION },
@@ -298,7 +275,6 @@ const struct gen_table set_tbl[] =
     { "y2l$abel", S_Y2LABEL },
     { "zl$abel", S_ZLABEL },
     { "cbl$abel", S_CBLABEL },
-    { "rlabel", S_RLABEL },
 
     { "xti$cs", S_XTICS },
     { "noxti$cs", S_NOXTICS },
@@ -311,7 +287,7 @@ const struct gen_table set_tbl[] =
     { "zti$cs", S_ZTICS },
     { "nozti$cs", S_NOZTICS },
     { "rti$cs", S_RTICS },
-    { "tti$cs", S_TTICS },
+    { "norti$cs", S_NORTICS },
     { "cbti$cs", S_CBTICS },
     { "nocbti$cs", S_NOCBTICS },
 
@@ -352,11 +328,6 @@ const struct gen_table set_tbl[] =
     { "ur$ange", S_URANGE },
     { "vr$ange", S_VRANGE },
 
-    { "vgrid", S_VGRID },
-    { "vxr$ange", S_VXRANGE },
-    { "vyr$ange", S_VYRANGE },
-    { "vzr$ange", S_VZRANGE },
-
     { "xzeroa$xis", S_XZEROAXIS },
     { "x2zeroa$xis", S_X2ZEROAXIS },
     { "yzeroa$xis", S_YZEROAXIS },
@@ -394,7 +365,6 @@ const struct gen_table set_key_tbl[] =
     { "def$ault", S_KEY_DEFAULT },
     { "on", S_KEY_ON },
     { "off", S_KEY_OFF },
-    { "offset", S_KEY_OFFSET},
     { "t$op", S_KEY_TOP },
     { "b$ottom", S_KEY_BOTTOM },
     { "l$eft", S_KEY_LEFT },
@@ -409,7 +379,6 @@ const struct gen_table set_key_tbl[] =
     { "at", S_KEY_MANUAL },
     { "ins$ide", S_KEY_INSIDE },
     { "o$utside", S_KEY_OUTSIDE },
-    { "fix$ed", S_KEY_FIXED },
     { "tm$argin", S_KEY_TMARGIN },
     { "bm$argin", S_KEY_BMARGIN },
     { "lm$argin", S_KEY_LMARGIN },
@@ -428,7 +397,6 @@ const struct gen_table set_key_tbl[] =
     { "sp$acing", S_KEY_SPACING },
     { "w$idth", S_KEY_WIDTH },
     { "h$eight", S_KEY_HEIGHT },
-    { "keyw$idth", S_KEY_KEYWIDTH },
     { "a$utotitles", S_KEY_AUTOTITLES },
     { "noa$utotitles", S_KEY_NOAUTOTITLES },
     { "ti$tle", S_KEY_TITLE },
@@ -436,8 +404,6 @@ const struct gen_table set_key_tbl[] =
     { "font", S_KEY_FONT },
     { "tc", S_KEY_TEXTCOLOR },
     { "text$color", S_KEY_TEXTCOLOR },
-    { "col$s", S_KEY_COLS},
-    { "colu$mns", S_KEY_COLS},
     { "maxcol$s", S_KEY_MAXCOLS},
     { "maxcolu$mns", S_KEY_MAXCOLS},
     { "maxrow$s", S_KEY_MAXROWS},
@@ -467,8 +433,6 @@ const struct gen_table set_colorbox_tbl[] =
     { "nobo$rder",	S_COLORBOX_NOBORDER },
     { "o$rigin",	S_COLORBOX_ORIGIN },
     { "s$ize",		S_COLORBOX_SIZE },
-    { "inv$ert",	S_COLORBOX_INVERT },
-    { "noinv$ert",	S_COLORBOX_NOINVERT },
     { "fr$ont",		S_COLORBOX_FRONT },
     { "ba$ck",		S_COLORBOX_BACK },
     { NULL, S_COLORBOX_INVALID }
@@ -479,8 +443,8 @@ const struct gen_table set_palette_tbl[] =
 {
     { "pos$itive",	S_PALETTE_POSITIVE },
     { "neg$ative",	S_PALETTE_NEGATIVE },
-    { "gray$scale",	S_PALETTE_GRAY },
-    { "grey$scale",	S_PALETTE_GRAY },
+    { "gray",		S_PALETTE_GRAY },
+    { "grey",		S_PALETTE_GRAY },
     { "col$or",		S_PALETTE_COLOR },
     { "rgb$formulae",	S_PALETTE_RGBFORMULAE },
     { "def$ined",       S_PALETTE_DEFINED },
@@ -501,6 +465,8 @@ const struct gen_table color_model_tbl[] =
     { "RGB", C_MODEL_RGB },
     { "HSV", C_MODEL_HSV },
     { "CMY", C_MODEL_CMY },
+    { "YIQ", C_MODEL_YIQ },
+    { "XYZ", C_MODEL_XYZ },
     { NULL,  -1 }
 };
 
@@ -516,11 +482,8 @@ const struct gen_table set_pm3d_tbl[] =
     { "fl$ush",		S_PM3D_FLUSH },
     { "ftr$iangles",	S_PM3D_FTRIANGLES },
     { "noftr$iangles",	S_PM3D_NOFTRIANGLES },
-    { "clip$z", 	S_PM3D_CLIP_Z },
     { "clip1$in",	S_PM3D_CLIP_1IN },
     { "clip4$in",	S_PM3D_CLIP_4IN },
-    { "clipcb", 	S_PM3D_CLIPCB },
-    { "noclipcb",	S_PM3D_NOCLIPCB },
     { "map", 		S_PM3D_MAP },
     { "bo$rder",	S_PM3D_BORDER },
     { "nobo$rder",	S_PM3D_NOBORDER },
@@ -534,9 +497,7 @@ const struct gen_table set_pm3d_tbl[] =
     { "noe$xplicit",	S_PM3D_NOEXPLICIT },
     { "noi$mplicit",	S_PM3D_NOIMPLICIT },
     { "e$xplicit",	S_PM3D_EXPLICIT },
-    { "corners2c$olor",	S_PM3D_WHICH_CORNER },
-    { "light$ing",	S_PM3D_LIGHTING_MODEL },
-    { "nolight$ing",	S_PM3D_NOLIGHTING_MODEL },
+    { "corners2c$olor",S_PM3D_WHICH_CORNER },
     { NULL, S_PM3D_INVALID }
 };
 
@@ -691,8 +652,9 @@ const struct gen_table show_style_tbl[] =
     { "rect$angle", SHOW_STYLE_RECTANGLE },
     { "boxplot", SHOW_STYLE_BOXPLOT },
     { "parallel$axis", SHOW_STYLE_PARALLEL },
-    { "spider$plot", SHOW_STYLE_SPIDERPLOT },
+#ifdef EAM_BOXED_TEXT
     { "textbox", SHOW_STYLE_TEXTBOX },
+#endif
     { NULL, SHOW_STYLE_INVALID }
 };
 
@@ -714,7 +676,6 @@ const struct gen_table plotstyle_tbl[] =
     { "xye$rrorbars", XYERRORBARS },
     { "boxes", BOXES },
     { "hist$ograms", HISTOGRAMS },
-    { "isosurface", ISOSURFACE },
     { "filledc$urves", FILLEDCURVES },
     { "boxer$rorbars", BOXERROR },
     { "boxx$yerrorbars", BOXXYERROR },
@@ -723,30 +684,28 @@ const struct gen_table plotstyle_tbl[] =
     { "fs$teps", FSTEPS },
     { "his$teps", HISTEPS },
     { "vec$tors", VECTOR },
-    { "arrow$s", ARROWS },
     { "fin$ancebars", FINANCEBARS },
     { "can$dlesticks", CANDLESTICKS },
     { "boxplot", BOXPLOT },
     { "pm$3d", PM3DSURFACE },
-    { "poly$gons", POLYGONS },
     { "labels", LABELPOINTS },
     { "ima$ge", IMAGE },
     { "rgbima$ge", RGBIMAGE },
     { "rgba$lpha", RGBA_IMAGE },
+#ifdef EAM_OBJECTS
     { "cir$cles", CIRCLES },
     { "ell$ipses", ELLIPSES },
+#endif
     { "sur$face", SURFACEGRID },
     { "parallel$axes", PARALLELPLOT },
-    { "spider$plot", SPIDERPLOT },
     { "table", TABLESTYLE },
-    { "zerror$fill", ZERRORFILL },
     { NULL, PLOT_STYLE_NONE }
 };
 
 const struct gen_table filledcurves_opts_tbl[] =
 {
     { "c$losed", FILLEDCURVES_CLOSED },
-    { "x$1", FILLEDCURVES_X1 },
+    { "x1", FILLEDCURVES_X1 },
     { "y1", FILLEDCURVES_Y1 },
     { "x2", FILLEDCURVES_X2 },
     { "y2", FILLEDCURVES_Y2 },
@@ -754,17 +713,6 @@ const struct gen_table filledcurves_opts_tbl[] =
     { "r", FILLEDCURVES_ATR },
     { "above", FILLEDCURVES_ABOVE },
     { "below", FILLEDCURVES_BELOW },
-    { "y",  FILLEDCURVES_Y1 },
-    { NULL, -1 }
-};
-
-const struct gen_table fit_verbosity_level[] =
-{
-    { "brief", BRIEF },
-    { "quiet", QUIET },
-    { "noquiet", BRIEF },
-    { "results", RESULTS },
-    { "verbose", VERBOSE },
     { NULL, -1 }
 };
 
