@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: term.c,v 1.296 2014/08/03 21:52:50 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - term.c */
 
 /*[
@@ -89,7 +93,6 @@
 #include "version.h"
 #include "misc.h"
 #include "multiplot.h"
-#include "readline.h"
 
 #ifdef USE_MOUSE
 #include "mouse.h"
@@ -100,16 +103,16 @@ long mouse_mode = 0;
 char* mouse_alt_string = NULL;
 #endif
 
-#ifdef _WIN32
-# include "win/winmain.h"
-# include "win/wcommon.h"
+#ifdef _Windows
+FILE *open_printer __PROTO((void));     /* in wprinter.c */
+void close_printer __PROTO((FILE * outfile));
 # ifdef __MSC__
 #  include <malloc.h>
 #  include <io.h>
 # else
 #  include <alloc.h>
 # endif                         /* MSC */
-#endif /* _WIN32 */
+#endif /* _Windows */
 
 static int termcomp __PROTO((const generic * a, const generic * b));
 
@@ -137,9 +140,6 @@ TBOOLEAN term_initialised;
 /* Whichever one is used first to plot, this locks out the other. */
 void *term_interlock = NULL;
 
-/* true if "set monochrome" */
-TBOOLEAN monochrome = FALSE;
-
 /* true if in multiplot mode */
 TBOOLEAN multiplot = FALSE;
 
@@ -196,7 +196,6 @@ TBOOLEAN ignore_enhanced_text = FALSE;
 
 /* Recycle count for user-defined linetypes */
 int linetype_recycle_count = 0;
-int mono_recycle_count = 0;
 
 
 /* Internal variables */
@@ -233,7 +232,6 @@ static int null_justify_text __PROTO((enum JUSTIFY just));
 static int null_scale __PROTO((double x, double y));
 static void null_layer __PROTO((t_termlayer layer));
 static int null_set_font __PROTO((const char *font));
-static void null_set_color __PROTO((struct t_colorspec *colorspec));
 static void options_null __PROTO((void));
 static void graphics_null __PROTO((void));
 static void UNKNOWN_null __PROTO((void));
@@ -267,7 +265,7 @@ void fflush_binary();
 # define FOPEN_BINARY(file) fopen(file, "wb")
 #endif /* !VMS */
 
-#if defined(MSDOS) || defined(_WIN32)
+#if defined(MSDOS) || defined(WIN32)
 # if defined(__DJGPP__)
 #  include <io.h>
 # endif
@@ -321,7 +319,7 @@ term_close_output()
 	output_pipe_open = FALSE;
     } else
 #endif /* PIPES */
-#ifdef _WIN32
+#ifdef _Windows
     if (stricmp(outstr, "PRN") == 0)
 	close_printer(gpoutfile);
     else
@@ -338,6 +336,12 @@ term_close_output()
     gppsfile = NULL;
 }
 
+#ifdef OS2
+# define POPEN_MODE ("wb")
+#else
+# define POPEN_MODE ("w")
+#endif
+
 /* assigns dest to outstr, so it must be allocated or NULL
  * and it must not be outstr itself !
  */
@@ -366,22 +370,14 @@ term_set_output(char *dest)
 #if defined(PIPES)
 	if (*dest == '|') {
 	    restrict_popen();
-#ifdef _WIN32
-	    if (term && (term->flags & TERM_BINARY))
-		f = popen(dest + 1, "wb");
-	    else
-		f = popen(dest + 1, "w");
-#else
-	    f = popen(dest + 1, "w");
-#endif
-	    if (f == (FILE *) NULL)
+	    if ((f = popen(dest + 1, POPEN_MODE)) == (FILE *) NULL)
 		os_error(c_token, "cannot create pipe; output not changed");
 	    else
 		output_pipe_open = TRUE;
 	} else {
 #endif /* PIPES */
 
-#ifdef _WIN32
+#ifdef _Windows
 	if (outstr && stricmp(outstr, "PRN") == 0) {
 	    /* we can't call open_printer() while printer is open, so */
 	    close_printer(gpoutfile);   /* close printer immediately if open */
@@ -471,38 +467,24 @@ term_initialise()
 	    fputs("Cannot reopen output file in binary", stderr);
 	/* and carry on, hoping for the best ! */
     }
-#if defined(MSDOS) || defined (_WIN32) || defined(OS2)
-# ifdef _WIN32
+#if defined(MSDOS) || defined (_Windows) || defined(OS2)
+# ifdef _Windows
     else if (!outstr && (term->flags & TERM_BINARY))
 # else
     else if (!outstr && !interactive && (term->flags & TERM_BINARY))
 # endif
 	{
-#if defined(_WIN32) && !defined(WGP_CONSOLE)
-#ifdef PIPES
-	    if (!output_pipe_open)
-#endif
-		if (outstr == NULL && !(term->flags & TERM_NO_OUTPUTFILE))
-		    int_error(c_token, "cannot output binary data to wgnuplot text window");
-#endif
 	    /* binary to stdout in non-interactive session... */
 	    fflush(stdout);
 	    setmode(fileno(stdout), O_BINARY);
 	}
 #endif
 
+
     if (!term_initialised || term_force_init) {
 	FPRINTF((stderr, "- calling term->init()\n"));
 	(*term->init) ();
 	term_initialised = TRUE;
-#ifdef HAVE_LOCALE_H
-	/* This is here only from an abundance of caution (a.k.a. paranoia).
-	 * Some terminals (wxt qt caca) are known to change the locale when
-	 * initialized.  Others have been implicated (gd).  Rather than trying
-	 * to catch all such offenders one by one, cover for all of them here.
-	 */
-	setlocale(LC_NUMERIC, "C");
-#endif
     }
 }
 
@@ -595,9 +577,6 @@ term_reset()
 #ifdef USE_MOUSE
     /* Make sure that ^C will break out of a wait for 'pause mouse' */
     paused_for_mouse = 0;
-#ifdef _WIN32
-    kill_pending_Pause_dialog();
-#endif
 #endif
 
     if (!term_initialised)
@@ -637,7 +616,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
     t_dashtype custom_dash_pattern = lp->custom_dash_pattern;
     t_colorspec colorspec = lp->pm3d_color;
 
-    if ((lp->flags & LP_SHOW_POINTS)) {
+    if (lp->pointflag) {
 	/* change points, too
 	 * Currently, there is no 'pointtype' function.  For points
 	 * there is a special function also dealing with (x,y) co-
@@ -653,12 +632,6 @@ term_apply_lp_properties(struct lp_style_type *lp)
      */
     (*term->linewidth) (lp->l_width);
 
-    /* LT_DEFAULT (used only by "set errorbars"?) means don't change it */
-    /* FIXME: If this causes problems, test also for LP_ERRORBAR_SET    */
-    if (lt == LT_DEFAULT)
-	;
-    else
-
     /* The paradigm for handling linetype and dashtype in version 5 is */
     /* linetype < 0 (e.g. LT_BACKGROUND, LT_NODRAW) means some special */
     /* category that will be handled directly by term->linetype().     */
@@ -666,21 +639,16 @@ term_apply_lp_properties(struct lp_style_type *lp)
     /* and a dash pattern.  Now we have separate mechanisms for those. */ 
     if (LT_COLORFROMCOLUMN < lt && lt < 0)
 	(*term->linetype) (lt);
-    else if (term->set_color == null_set_color) {
-	(*term->linetype) (lt-1);
-	return;
-    } else /* All normal lines will be solid unless a dashtype is given */
+    else /* All normal lines will be solid unless a dashtype is given */
 	(*term->linetype) (LT_SOLID);
 
     /* Apply dashtype or user-specified dash pattern, which may override  */
     /* the terminal-specific dot/dash pattern belonging to this linetype. */
-    if (lt == LT_AXIS)
-	; /* LT_AXIS is a special linetype that may incorporate a dash pattern */
-    else if (dt == DASHTYPE_CUSTOM)
+    if (dt == DASHTYPE_CUSTOM)
 	(*term->dashtype) (dt, &custom_dash_pattern);
     else if (dt == DASHTYPE_SOLID)
 	(*term->dashtype) (dt, NULL);
-    else if (dt >= 0)
+    else if (dt > 0)
 	/* The null_dashtype() routine or a version 5 terminal's private  */
 	/* dashtype routine converts this into a call to term->linetype() */
 	/* yielding the same result as in version 4 except possibly for a */
@@ -688,7 +656,7 @@ term_apply_lp_properties(struct lp_style_type *lp)
 	(*term->dashtype) (dt, NULL);
 
     /* Finally adjust the color of the line */
-    apply_pm3dcolor(&colorspec);
+    apply_pm3dcolor(&colorspec, term);
 }
 
 void
@@ -844,10 +812,6 @@ do_point(unsigned int x, unsigned int y, int number)
     int htic, vtic;
     struct termentry *t = term;
 
-    /* use solid lines for point symbols */
-    if (term->dashtype != null_dashtype)
-	term->dashtype(DASHTYPE_SOLID, NULL);
-
     if (number < 0) {           /* do dot */
 	(*t->move) (x, y);
 	(*t->vector) (x, y);
@@ -1134,7 +1098,7 @@ do_arrow(
 
 void
 do_arc(
-    int cx, int cy, /* Center */
+    unsigned int cx, unsigned int cy, /* Center */
     double radius, /* Radius */
     double arc_start, double arc_end, /* Limits of arc in degress */
     int style, TBOOLEAN wedge)
@@ -1163,6 +1127,10 @@ do_arc(
 
     /* Calculate the vertices */
     aspect = (double)term->v_tic / (double)term->h_tic;
+#ifdef WIN32
+    if (strcmp(term->name, "windows") == 0)
+	aspect = 1.;
+#endif
     for (i=0; i<segments; i++) {
 	vertex[i].x = cx + cos(DEG2RAD * (arc_start + i*INC)) * radius;
 	vertex[i].y = cy + sin(DEG2RAD * (arc_start + i*INC)) * radius * aspect;
@@ -1236,15 +1204,15 @@ null_justify_text(enum JUSTIFY just)
 }
 
 
-/* 
- * Deprecated terminal function (pre-version 3)
+/* Change scale of plot.
+ * Parameters are x,y scaling factors for this plot.
+ * Some terminals (eg latex) need to do scaling themselves.
  */
 static int
 null_scale(double x, double y)
 {
     (void) x;                   /* avoid -Wunused warning */
     (void) y;
-    int_error(NO_CARET, "Attempt to call deprecated terminal function");
     return FALSE;               /* can't be done */
 }
 
@@ -1431,9 +1399,8 @@ set_term()
     if (!END_OF_COMMAND) {
 	input_name = gp_input_line + token[c_token].start_index;
 	t = change_term(input_name, token[c_token].length);
-	if (!t && isstringvalue(c_token) && (input_name = try_to_get_string())) {
-	    if (strchr(input_name, ' '))
-		*strchr(input_name, ' ') = '\0';
+	if (!t && isstringvalue(c_token)) {
+	    input_name = try_to_get_string();  /* Cannot fail if isstringvalue succeeded */
 	    t = change_term(input_name, strlen(input_name));
 	    free(input_name);
 	} else {
@@ -1498,6 +1465,9 @@ change_term(const char *origname, int length)
     term = t;
     term_initialised = FALSE;
 
+    if (term->scale != null_scale)
+	fputs("Warning: scale interface is not null_scale - may not work with multiplot\n", stderr);
+
     /* check that optional fields are initialised to something */
     if (term->text_angle == 0)
 	term->text_angle = null_text_angle;
@@ -1525,7 +1495,7 @@ change_term(const char *origname, int length)
 	term->dashtype = null_dashtype;
 
     if (interactive)
-	fprintf(stderr, "\nTerminal type is now '%s'\n", term->name);
+	fprintf(stderr, "Terminal type set to '%s'\n", term->name);
 
     /* Invalidate any terminal-specific structures that may be active */
     invalidate_palette();
@@ -1534,7 +1504,9 @@ change_term(const char *origname, int length)
 }
 
 /*
- * Find an appropriate initial terminal type.
+ * Routine to detect what terminal is being used (or do anything else
+ * that would be nice).  One anticipated (or allowed for) side effect
+ * is that the global ``term'' may be set.
  * The environment variable GNUTERM is checked first; if that does
  * not exist, then the terminal hardware is checked, if possible,
  * and finally, we can check $TERM for some kinds of terminals.
@@ -1546,7 +1518,7 @@ void
 init_terminal()
 {
     char *term_name = DEFAULTTERM;
-#if (defined(MSDOS) && !defined(_WIN32)) || defined(SUN) || defined(X11)
+#if (defined(MSDOS) && !defined(_Windows)) || defined(NEXT) || defined(SUN) || defined(X11)
     char *env_term = NULL;      /* from TERM environment var */
 #endif
 #ifdef X11
@@ -1557,26 +1529,19 @@ init_terminal()
     /* GNUTERM environment variable is primary */
     gnuterm = getenv("GNUTERM");
     if (gnuterm != (char *) NULL) {
-	/* April 2017 - allow GNUTERM to include terminal options */
-	char *set_term = "set term ";
-	char *set_term_command = gp_alloc(strlen(set_term) + strlen(gnuterm) + 4, NULL);
-	strcpy(set_term_command, set_term);
-	strcat(set_term_command, gnuterm);
-	do_string(set_term_command);
-	free(set_term_command);
-	/* replicate environmental variable GNUTERM for internal use */
-	Gstring(&(add_udv_by_name("GNUTERM")->udv_value), gp_strdup(gnuterm));
-	return;
-
+	term_name = gnuterm;
     } else {
 
 #ifdef VMS
 	term_name = vms_init();
 #endif /* VMS */
 
+#ifdef NEXT
+	env_term = getenv("TERM");
 	if (term_name == (char *) NULL
-            && getenv ("DOMTERM") != NULL)
-          term_name = "domterm";
+	    && env_term != (char *) NULL && strcmp(env_term, "next") == 0)
+	    term_name = "next";
+#endif /* NeXT */
 
 #ifdef __BEOS__
 	env_term = getenv("TERM");
@@ -1585,25 +1550,28 @@ init_terminal()
 	    term_name = "be";
 #endif /* BeOS */
 
-#if defined(WXWIDGETS) && defined(_WIN32)
-	if (term_name == (char *) NULL)
-	    term_name = "wxt";
-#endif
+#ifdef SUN
+	env_term = getenv("TERM");      /* try $TERM */
+	if (term_name == (char *) NULL
+	    && env_term != (char *) NULL && strcmp(env_term, "sun") == 0)
+	    term_name = "sun";
+#endif /* SUN */
 
 #ifdef QTTERM
 	if (term_name == (char *) NULL)
-	    term_name = "qt";
+		term_name = "qt";
 #endif
 
 #ifdef WXWIDGETS
 	if (term_name == (char *) NULL)
-	    term_name = "wxt";
+		term_name = "wxt";
 #endif
 
-#ifdef _WIN32
+#ifdef _Windows
+	/* let the wxWidgets terminal be the default when available */
 	if (term_name == (char *) NULL)
-	    term_name = "win";
-#endif /* _WIN32 */
+		term_name = "win";
+#endif /* _Windows */
 
 #if defined(__APPLE__) && defined(__MACH__) && defined(HAVE_FRAMEWORK_AQUATERM)
 	/* Mac OS X with AquaTerm installed */
@@ -1663,6 +1631,7 @@ init_terminal()
 	struct udvt_entry *name = add_udv_by_name("GNUTERM");
 
 	Gstring(&name->udv_value, gp_strdup(term_name));
+	name->udv_undef = FALSE;
 
 	if (strchr(term_name,' '))
 	    namelength = strchr(term_name,' ') - term_name;
@@ -1692,12 +1661,11 @@ test_term()
     struct termentry *t = term;
     const char *str;
     int x, y, xl, yl, i;
-    int xmax_t, ymax_t, x0, y0;
+    unsigned int xmax_t, ymax_t, x0, y0;
     char label[MAX_ID_LEN];
     int key_entry_height;
     int p_width;
     TBOOLEAN already_in_enhanced_text_mode;
-    static t_colorspec black = BLACK_COLORSPEC;
 
     already_in_enhanced_text_mode = t->flags & TERM_ENHANCED_TEXT;
     if (!already_in_enhanced_text_mode)
@@ -1705,10 +1673,10 @@ test_term()
 
     term_start_plot();
     screen_ok = FALSE;
-    xmax_t = (t->xmax * xsize);
-    ymax_t = (t->ymax * ysize);
-    x0 = (xoffset * t->xmax);
-    y0 = (yoffset * t->ymax);
+    xmax_t = (unsigned int) (t->xmax * xsize);
+    ymax_t = (unsigned int) (t->ymax * ysize);
+    x0 = (unsigned int) (xoffset * t->xmax);
+    y0 = (unsigned int) (yoffset * t->ymax);
 
     p_width = pointsize * t->h_tic;
     key_entry_height = pointsize * t->v_tic * 1.25;
@@ -1734,55 +1702,39 @@ test_term()
 	int_error(NO_CARET, "terminal type is unknown");
     else {
 	char tbuf[64];
+	strcpy(tbuf,term->name);
+	strcat(tbuf,"  terminal test");
 	(void) (*t->justify_text) (LEFT);
-	sprintf(tbuf,"%s  terminal test", term->name);
 	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t - t->v_char, tbuf);
-	sprintf(tbuf, "gnuplot version %s.%s  ", gnuplot_version, gnuplot_patchlevel);
-	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t - t->v_char * 2.25, tbuf);
     }
 
+#ifdef USE_MOUSE
+    if (t->set_ruler)
+	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t - t->v_char * 2.5,
+			"Mouse and hotkeys are supported");
+#endif
+
     (*t->linetype) (LT_AXIS);
     (*t->move) (x0 + xmax_t / 2, y0);
     (*t->vector) (x0 + xmax_t / 2, y0 + ymax_t - 1);
     (*t->move) (x0, y0 + ymax_t / 2);
     (*t->vector) (x0 + xmax_t - 1, y0 + ymax_t / 2);
-
-    /* How well can we estimate width and height of characters?
-     * Textbox fill shows true size, surrounding box shows the generic estimate
-     * used to reserve space during plot layout.
-     */
-#ifdef EAM_BOXED_TEXT
-    if (TRUE) {
-	struct text_label sample = EMPTY_LABELSTRUCT;
-	struct textbox_style save_opts = textbox_opts;
-	sample.text = "12345678901234567890";
-	sample.pos = CENTRE;
-	sample.boxed = 1;
-	textbox_opts.opaque = TRUE;
-	textbox_opts.noborder = TRUE;
-	textbox_opts.fillcolor.type = TC_RGB;
-	textbox_opts.fillcolor.lt = 0xccccee;
-
-	(*t->linetype) (LT_SOLID);
-	write_label(xmax_t/2, ymax_t/2, &sample);
-	textbox_opts = save_opts;
-
-	sample.boxed = 0;
-	sample.text = "true vs. estimated text dimensions";
-	write_label(xmax_t/2, ymax_t/2 + 1.5 * t->v_char, &sample);
-
-	newpath();
-	(*t->move) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
-	(*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
-	(*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
-	(*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
-	(*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
-	closepath();
-    }
-#endif
+    /* test width and height of characters */
+    (*t->linetype) (3);
+    newpath();
+    (*t->move) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 + t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 - t->v_char / 2);
+    (*t->vector) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char / 2);
+    closepath();
+    (*t->put_text) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2,
+		    "12345678901234567890");
+    (*t->put_text) (x0 + xmax_t / 2 - t->h_char * 10, y0 + ymax_t / 2 + t->v_char * 1.4,
+		    "test of character width:");
+    (*t->linetype) (LT_BLACK);
 
     /* Test for enhanced text */
-    (*t->linetype) (LT_BLACK);
     if (t->flags & TERM_ENHANCED_TEXT) {
 	char *tmptext1 =   "Enhanced text:   {x@_{0}^{n+1}}";
 	char *tmptext2 = "&{Enhanced text:  }{/:Bold Bold}{/:Italic  Italic}";  
@@ -1810,6 +1762,30 @@ test_term()
     else
 	(*t->put_text) (x0 + xmax_t / 2 - strlen(str) * t->h_char,
 			y0 + ymax_t / 2 + t->v_char * 4, str);
+    /* test text angle */
+    (*t->linetype)(1);
+    str = "rotated ce+ntred text";
+    if ((*t->text_angle) (TEXT_VERTICAL)) {
+	if ((*t->justify_text) (CENTRE))
+	    (*t->put_text) (x0 + t->v_char,
+			    y0 + ymax_t / 2, str);
+	else
+	    (*t->put_text) (x0 + t->v_char,
+			    y0 + ymax_t / 2 - strlen(str) * t->h_char / 2, str);
+	(*t->justify_text) (LEFT);
+	str = " rotated by +45 deg";
+	(*t->text_angle)(45);
+	(*t->put_text)(x0 + t->v_char * 3, y0 + ymax_t / 2, str);
+	(*t->justify_text) (LEFT);
+	str = " rotated by -45 deg";
+	(*t->text_angle)(-45);
+	(*t->put_text)(x0 + t->v_char * 2, y0 + ymax_t / 2, str);
+    } else {
+	(void) (*t->justify_text) (LEFT);
+	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t / 2 - t->v_char * 2, "can't rotate text");
+    }
+    (void) (*t->justify_text) (LEFT);
+    (void) (*t->text_angle) (0);
 
     /* test tic size */
     (*t->linetype)(2);
@@ -1848,47 +1824,29 @@ test_term()
 	y -= key_entry_height;
     }
 
-    /* test arrows (should line up with rotated text) */
+    /* test some arrows */
     (*t->linewidth) (1.0);
     (*t->linetype) (0);
-    (*t->dashtype) (DASHTYPE_SOLID, NULL);
-    x = x0 + 2. * t->v_char;
-    y = y0 + ymax_t/2;
+    x = x0 + xmax_t * .375;
+    y = y0 + ymax_t * .250;
     xl = t->h_tic * 7;
     yl = t->v_tic * 7;
     i = curr_arrow_headfilled;
-    curr_arrow_headfilled = AS_NOBORDER;
-    (*t->arrow) (x, y-yl, x, y+yl, BOTH_HEADS);
-    curr_arrow_headfilled = AS_EMPTY;
-    (*t->arrow) (x, y, x + xl, y + yl, END_HEAD);
     curr_arrow_headfilled = AS_NOFILL;
-    (*t->arrow) (x, y, x + xl, y - yl, END_HEAD);
+    (*t->arrow) (x, y, x + xl, y, END_HEAD);
+    curr_arrow_headfilled = 1;
+    (*t->arrow) (x, y, x - xl, y, END_HEAD);
+    curr_arrow_headfilled = 2;
+    (*t->arrow) (x, y, x, y + yl, END_HEAD);
+    curr_arrow_headfilled = AS_EMPTY;
+    (*t->arrow) (x, y, x, y - yl, END_HEAD);
     curr_arrow_headfilled = i;
-
-    /* test text angle (should match arrows) */
-    (*t->linetype)(0);
-    str = "rotated ce+ntred text";
-    if ((*t->text_angle) (TEXT_VERTICAL)) {
-	if ((*t->justify_text) (CENTRE))
-	    (*t->put_text) (x0 + t->v_char,
-			    y0 + ymax_t / 2, str);
-	else
-	    (*t->put_text) (x0 + t->v_char,
-			    y0 + ymax_t / 2 - strlen(str) * t->h_char / 2, str);
-	(*t->justify_text) (LEFT);
-	str = "  rotate by +45";
-	(*t->text_angle)(45);
-	(*t->put_text)(x0 + t->v_char * 3, y0 + ymax_t / 2, str);
-	(*t->justify_text) (LEFT);
-	str = "  rotate by -45";
-	(*t->text_angle)(-45);
-	(*t->put_text)(x0 + t->v_char * 3, y0 + ymax_t / 2, str);
-    } else {
-	(void) (*t->justify_text) (LEFT);
-	(*t->put_text) (x0 + t->h_char * 2, y0 + ymax_t / 2, "cannot rotate text");
-    }
-    (void) (*t->justify_text) (LEFT);
-    (void) (*t->text_angle) (0);
+    xl = t->h_tic * 5;
+    yl = t->v_tic * 5;
+    (*t->arrow) (x - xl, y - yl, x + xl, y + yl, END_HEAD | BACKHEAD);
+    (*t->arrow) (x - xl, y + yl, x, y, NOHEAD);
+    curr_arrow_headfilled = AS_EMPTY;
+    (*t->arrow) (x, y, x + xl, y - yl, BACKHEAD);
 
     /* test line widths */
     (void) (*t->justify_text) (LEFT);
@@ -1900,31 +1858,12 @@ test_term()
     for (i=1; i<7; i++) {
 	(*t->linewidth) ((float)(i)); (*t->linetype)(LT_BLACK);
 	(*t->move) (x, y); (*t->vector) (x+xl, y);
-	sprintf(label,"  lw %1d", i);
+	sprintf(label,"  lw %1d%c",i,0);
 	(*t->put_text) (x+xl, y, label);
 	y += yl;
     }
     (*t->put_text) (x, y, "linewidth");
 
-    /* test native dashtypes (_not_ the 'set mono' sequence) */
-    (void) (*t->justify_text) (LEFT);
-    xl = xmax_t / 10;
-    yl = ymax_t / 25;
-    x = x0 + xmax_t * .3;
-    y = y0 + yl;
-    
-    for (i=0; i<5; i++) {
- 	(*t->linewidth) (1.0);
-	(*t->linetype) (LT_SOLID);
-	(*t->dashtype) (i, NULL); 
-	(*t->set_color)(&black);
-	(*t->move) (x, y); (*t->vector) (x+xl, y);
-	sprintf(label,"  dt %1d", i+1);
-	(*t->put_text) (x+xl, y, label);
-	y += yl;
-    }
-    (*t->put_text) (x, y, "dashtype");
-
     /* test fill patterns */
     x = x0 + xmax_t * 0.5;
     y = y0;
@@ -1945,7 +1884,7 @@ test_term()
 	(*t->vector)(x+xl,y);
 	(*t->vector)(x,y);
 	closepath();
-	sprintf(label, "%2d", i);
+	sprintf(label,"%2d",i);
 	(*t->put_text)(x+xl/2, y+yl+t->v_char*0.5, label);
 	x += xl * 1.5;
     }
@@ -1986,7 +1925,7 @@ test_term()
 	    str = "No filled polygons";
 	(*t->linetype)(LT_BLACK);
 	i = ((*t->justify_text) (CENTRE)) ? 0 : t->h_char * strlen(str) / 2;
-	(*t->put_text) (cen_x - i, cen_y + radius + t->v_char * 0.5, str);
+	(*t->put_text) (cen_x + i, cen_y + radius + t->v_char * 0.5, str);
     }
 
     term_end_plot();
@@ -2327,10 +2266,8 @@ enhanced_recursion(
 
 		ENH_DEBUG(("Dealing with {\n"));
 
-		/* 30 Sep 2016:  Remove incorrect whitespace-eating loop going */
-		/* waaay back to 31-May-2000 */        /* while (*++p == ' '); */
-		++p;
 		/* get vertical offset (if present) for overprinted text */
+		while (*++p == ' ');
 		if (overprint == 2) {
 		    char *end;
 		    ovp = (float)strtod(p,&end);
@@ -2633,33 +2570,14 @@ int len;
     return len;
 }
 
-/* 
- * Use estimate.trm to mock up a non-enhanced approximation of the
- * original string.
- */
-char *
-estimate_plaintext(char *enhancedtext)
-{
-    if (enhancedtext == NULL)
-	return NULL;
-    estimate_strlen(enhancedtext);
-    return ENHest_plaintext;
-}
-
 void
 ignore_enhanced(TBOOLEAN flag)
 {
-#if (0)
-    /* Apr 2018: This code was introduced long ago (2005; Bug #266) to address
-     * a glitch in the postscript terminal that left the last-used font in
-     * an enhanced text string active afterwards.
-     * We now deal with this in ENHPS_put_text() instead.
-     */
+    /* Force a return to the default font */
     if (flag && !ignore_enhanced_text) {
 	ignore_enhanced_text = TRUE;
 	term->set_font("");
     }
-#endif
     ignore_enhanced_text = flag;
 }
 
@@ -2816,13 +2734,13 @@ lp_use_properties(struct lp_style_type *lp, int tag)
      */
 
     struct linestyle_def *this;
-    int save_flags = lp->flags;
+    int save_pointflag = lp->pointflag;
 
     this = first_linestyle;
     while (this != NULL) {
 	if (this->tag == tag) {
 	    *lp = this->lp_properties;
-	    lp->flags = save_flags;
+	    lp->pointflag = save_pointflag;
 	    return;
 	} else {
 	    this = this->next;
@@ -2844,26 +2762,6 @@ load_linetype(struct lp_style_type *lp, int tag)
     TBOOLEAN recycled = FALSE;
 
 recycle:
-
-    if ((tag > 0)
-    && (monochrome || (term && (term->flags & TERM_MONOCHROME)))) {
-	for (this = first_mono_linestyle; this; this = this->next) {
-	    if (tag == this->tag) {
-		*lp = this->lp_properties;
-		return;
-	    }
-	}
-
-	/* This linetype wasn't defined explicitly.		*/
-	/* Should we recycle one of the first N linetypes?	*/
-	if (tag > mono_recycle_count && mono_recycle_count > 0) {
-	    tag = (tag-1) % mono_recycle_count + 1;
-	    goto recycle;
-	}
-
-	return;
-    }
-
     this = first_perm_linestyle;
     while (this != NULL) {
 	if (this->tag == tag) {
@@ -2874,9 +2772,8 @@ recycle:
 	    lp->d_type = this->lp_properties.d_type;
 	    lp->custom_dash_pattern = this->lp_properties.custom_dash_pattern;
 
-	    /* Needed in version 5.0 to handle old terminals (pbm hpgl ...) */
-	    /* with no support for user-specified colors */
-	    if (term && term->set_color == null_set_color)
+	    /* FIXME:  Not sure this is correct for version 5 */
+	    if (term->flags & TERM_MONOCHROME)
 		lp->l_type = tag;
 
 	    /* Do not recycle point properties. */
@@ -2884,8 +2781,8 @@ recycle:
 	    if (!recycled) {
 	    	lp->p_type = this->lp_properties.p_type;
 	    	lp->p_interval = this->lp_properties.p_interval;
+	    	lp->p_char = this->lp_properties.p_char;
 	    	lp->p_size = this->lp_properties.p_size;
-	    	memcpy(lp->p_char, this->lp_properties.p_char, sizeof(lp->p_char));
 	    }
 	    return;
 	} else {
@@ -2910,29 +2807,6 @@ recycle:
     lp->p_type = (tag <= 0) ? -1 : tag - 1;
 }
 
-/*
- * Version 5 maintains a parallel set of linetypes for "set monochrome" mode.
- * This routine allocates space and initializes the default set.
- */
-void
-init_monochrome()
-{
-    struct lp_style_type mono_default[] = DEFAULT_MONO_LINETYPES;
-
-    if (first_mono_linestyle == NULL) {
-	int i, n = sizeof(mono_default) / sizeof(struct lp_style_type);
-	struct linestyle_def *new;
-	/* copy default list into active list */
-	for (i=n; i>0; i--) {
-	    new = gp_alloc(sizeof(struct linestyle_def), NULL);
-	    new->next = first_mono_linestyle;
-	    new->lp_properties = mono_default[i-1];
-	    new->tag = i;
-	    first_mono_linestyle = new;
-	}
-    }
-}
-
 /*
  * Totally bogus estimate of TeX string lengths.
  * Basically
@@ -2997,13 +2871,13 @@ check_for_mouse_events()
 	term->waitforinput(TERM_ONLY_CHECK_MOUSING);
     }
 #endif
-#ifdef _WIN32
+#ifdef WIN32
     /* Process windows GUI events (e.g. for text window, or wxt and windows terminals) */
     WinMessageLoop();
     /* On Windows, Ctrl-C only sets this flag. */
     /* The next block duplicates the behaviour of inter(). */
     if (ctrlc_flag) {
-	ctrlc_flag = FALSE;
+    ctrlc_flag = FALSE;
 	term_reset();
 	putc('\n', stderr);
 	fprintf(stderr, "Ctrl-C detected!\n");
