@@ -1,3 +1,7 @@
+/*
+ * $Id: term.h,v 1.66 2014/03/19 23:36:07 sfeam Exp $
+ */
+
 /* GNUPLOT - term.h */
 
 /*[
@@ -35,7 +39,8 @@
  *   Edit this file depending on the set of terminals you wish to support.
  * Comment out the terminal types that you don't want or don't have, and
  * uncomment those that you want included. Be aware that some terminal
- * types will require changes in the makefile LIBS definition.
+ * types (eg, SUN) will require changes in the makefile
+ * LIBS definition.
  */
 
 /*
@@ -49,9 +54,6 @@
  * pslatex and epslatex support is now provided by the combination of
  * post.trm and pslatex.trm.  You cannot build pslatex without post.
  * Both drivers are selected by default, but you can disable them below.
- *
- * Enhanced text support is pretty much required for all terminals now.
- * If you build without GP_ENH_EST text layout will be degraded.
  */
 #define GP_ENH_EST 1		/* estimate string length of enhanced text */
 #define POSTSCRIPT_DRIVER 1	/* include post.trm */
@@ -61,16 +63,6 @@
 #define POSTSCRIPT_DRIVER
 #endif
 
-# ifdef GP_ENH_EST
-#  include "estimate.trm"	/* used for enhanced text processing */
-# endif
-
-/* Unicode escape sequences (\U+hhhh) are handling by the enhanced text code.
- * Various terminals check every string to see whether it needs enhanced text
- * processing. This macro allows them to include a check for the presence of
- * unicode escapes.
- */
-#define contains_unicode(S) strstr(S, "\\U+")
 
 /* Define SHORT_TERMLIST to select a few terminals. It is easier
  * to define the macro and list desired terminals in this section.
@@ -79,6 +71,10 @@
 #ifdef SHORT_TERMLIST
 # include "dumb.trm"		/* dumb terminal */
 
+# ifdef GP_ENH_EST
+#  include "estimate.trm"	/* used for enhanced text processing */
+# endif
+
 # ifdef POSTSCRIPT_DRIVER
 #  ifdef  PSLATEX_DRIVER
 #   undef PSLATEX_DRIVER
@@ -92,7 +88,7 @@
 # ifdef OS2
 #  include "pm.trm"		/* OS/2 Presentation Manager */
 # endif
-# ifdef _WIN32
+# ifdef _Windows
 #  include "win.trm"		/* MS-Windows */
 # endif
 #else /* include all applicable terminals not commented out */
@@ -106,36 +102,54 @@
 # include "be.trm"
 #endif
 
-/* MSDOS with djgpp compiler or _WIN32/Mingw or X11 */
-#if (defined(DJGPP) && (!defined(DJSVGA) || (DJSVGA != 0))) || defined(HAVE_GRX)
-# include "djsvga.trm"
-#endif
 
 /****************************************************************************/
-/* MS-DOS */
-#if defined(MSDOS)
+/* MS-DOS and Windows */
+#if defined(MSDOS) || defined(_Windows)
 
 /* MSDOS with emx-gcc compiler */
-# if defined(__EMX__)
+# if defined(MSDOS) && defined(__EMX__)
    /* Vesa-Cards */
 #  define EMXVESA
 #  include "emxvga.trm"
 # endif				/* MSDOS && EMX */
 
-/* MSDOS with OpenWatcom compiler */
-# if defined(__WATCOMC__)
-#  include "pc.trm"
+/* MSDOS with djgpp compiler */
+# if defined(DJGPP) && (!defined(DJSVGA) || (DJSVGA != 0))
+#  include "djsvga.trm"
 # endif
 
-#endif /* MSDOS */
+/* All other Compilers */
+# ifndef _Windows
+#  ifdef PC
+/* uncomment the next line to include SuperVGA support */
+#   define BGI_NAME "svga256"	/* the name of the SVGA.BGI for Borland C */
+/* this also triggers the inclusion of Super VGA support */
+#   include "pc.trm"		/* all PC types except MS WINDOWS */
+#  endif
+# else				/* _Windows */
+#  include "win.trm"		/* MS-Windows */
+# endif				/* _Windows */
+#endif /* MSDOS || _Windows */
 /****************************************************************************/
 
-/* Windows */
-#ifdef _WIN32
-/* Windows GDI/GDI+/Direct2D */
-# include "win.trm"
+
+/* NeXT */
+#ifdef NEXT
+# include "next.trm"
 #endif
 
+/* Apple Mac OS X Server 1.0 (Openstep Unix) */
+/* Apparently, Openstep code won't work on newer versions of
+ * MacOS X. If someone can fix this, and provide a proper
+ * configure test, let us know.
+ */
+/*
+ * #if defined(__APPLE__) && defined(__MACH__)
+ * # include "openstep.trm"
+ * #endif
+*/
+
 /* Apple Mac OS X */
 #ifdef HAVE_FRAMEWORK_AQUATERM
 /* support for AquaTerm.app */
@@ -157,6 +171,22 @@
 /* Terminals for various Unix platforms                                    */
 /***************************************************************************/
 
+/* Linux VGA */
+#ifdef LINUXVGA
+# include "linux.trm"
+
+/* Linux VGAGL */
+# if defined(VGAGL) && defined (THREEDKIT)
+#  include "vgagl.trm"
+# endif
+#endif /* LINUXVGA */
+
+/* SunView */
+#ifdef SUN
+# include "sun.trm"
+#endif
+
+
 /* VAX Windowing System requires UIS libraries */
 #ifdef UIS
 # include "vws.trm"
@@ -164,11 +194,10 @@
 
 /****************************************************************************/
 /* Terminals not relevant for MSDOS, MS-Windows */
-#if !(defined(MSDOS) || defined(_WIN32))
+#if !(defined(MSDOS) || defined(_Windows))
 
-/* This is not really a terminal, it generates a help section for */
-/* using the linux console. */
-# include "linux-vgagl.trm"
+/* AED 512 and AED 767 graphics terminals */
+/* # include "aed.trm" */
 
 /* gpic for groff */
 #ifdef HAVE_GPIC
@@ -176,20 +205,18 @@
 #endif
 
 /* REGIS graphics language */
-#if defined(HAVE_REGIS)
+#ifdef VMS
 # include "regis.trm"
 #endif
 
-#ifdef WITH_TEKTRONIX
-    /* Tektronix 4106, 4107, 4109 and 420x terminals */
-#   include "t410x.trm"
+/* Tektronix 4106, 4107, 4109 and 420x terminals */
+# include "t410x.trm"
 
-    /* a Tek 4010 and others including VT-style */
-#   include "tek.trm"
-#endif
+/* a Tek 4010 and others including VT-style */
+# include "tek.trm"
 
 
-#endif /* !MSDOS && !_WIN32 */
+#endif /* !MSDOS && !_Windows */
 /****************************************************************************/
 
 
@@ -215,7 +242,7 @@
 #include "cgm.trm"
 
 /* CorelDraw! eps format */
-/* #include "corel.trm"  */
+#include "corel.trm"
 
 /* debugging terminal */
 #ifdef DEBUG
@@ -230,12 +257,7 @@
 # include "caca.trm"
 #endif
 
-/* Legacy terminal for export to AutoCad (Release 10.x)
- * DWGR10 format (1988)
- * Still included by popular demand although basically untouched for 20+ years.
- * Someone please update this terminal to adhere to a newer DXF standard!
- * http://images.autodesk.com/adsk/files/autocad_2012_pdf_dxf-reference_enu.pdf
- */
+/* DXF format for use with AutoCad (Release 10.x) */
 #include "dxf.trm"
 
 /* Enhanced Metafile Format driver */
@@ -284,10 +306,11 @@
 #include "mif.trm"
 #endif
 
-/* DEPRECATED since 5.0.6
- * PDF terminal based on non-free library PDFlib or PDFlib-lite from GmbH.
- */
-/* # include "pdf.trm" */
+/* Adobe Portable Document Format (PDF) */
+/* NOTE THAT PDF REQUIRES A SEPARATE LIBRARY : see term/pdf.trm */
+#ifdef HAVE_LIBPDF
+# include "pdf.trm"
+#endif
 
 #if defined(HAVE_GD_PNG) || defined(HAVE_GD_JPEG) || defined(HAVE_GD_GIF)
 # include "gd.trm"
@@ -299,19 +322,20 @@
 #endif
 
 /* QMS laser printers */
-/* #include "qms.trm" */
+#include "qms.trm"
 
 /* W3C Scalable Vector Graphics file */
 #include "svg.trm"
 
 /* x11 tgif tool */
-#ifdef HAVE_TGIF
 #include "tgif.trm"
-#endif
 
 /* tcl/tk with perl extensions */
 #include "tkcanvas.trm"
 
+/* Vectrix 384 printer, also Tandy colour */
+/* #include "v384.trm" */
+
 #ifndef NO_BITMAP_SUPPORT
 
 /* portable bit map */
@@ -342,17 +366,24 @@
 
 #endif /* NO_BITMAP_SUPPORT */
 
-/* TeX related terminals start here */
+/* TeX related terminals */
+#define EMTEX
+#define EEPIC
 
-
-/* LaTeX2e picture environment */
-#include "pict2e.trm"
+/* latex and emtex */
+#include "latex.trm"
 
 /* latex/tex with picture in postscript */
 #ifdef PSLATEX_DRIVER
 #include "pslatex.trm"
 #endif
 
+/* EEPIC-extended LaTeX driver, for EEPIC users */
+#include "eepic.trm"
+
+/* TPIC specials for TeX */
+#include "tpic.trm"
+
 /* LaTeX picture environment with PSTricks macros */
 #include "pstricks.trm"
 
@@ -368,29 +399,14 @@
 /* ConTeXt */
 #include "context.trm"
 
-/*
- * DEPRECATED latex terminals no longer built by default
- */
-#if 0
-#  define EMTEX
-#  define EEPIC
-#  define OLD_LATEX_TERMINAL
-
-#  include "latex.trm"	/* latex and emtex */
-#  include "eepic.trm"	/* EEPIC-extended LaTeX driver */
-#  include "tpic.trm"	/* TPIC specials for TeX */
-#else
-#  include "latex_old.h" /* deprecation notice for docs */
-#  undef OLD_LATEX_TERMINAL
-#endif
-
-/* End of TeX related terminals */
-
-
 #ifdef USE_GGI_DRIVER
 # include "ggi.trm"
 #endif
 
+#ifdef GP_ENH_EST
+#include "estimate.trm"
+#endif
+
 /* WXWIDGETS */
 #ifdef WXWIDGETS
 # include "wxt.trm"
