@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: time.c,v 1.28 2014/01/27 21:53:16 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - time.c */
 
 /*[
@@ -33,8 +37,8 @@
 
 /* This module either adds a routine gstrptime() to read a formatted time,
  * augmenting the standard suite of time routines provided by ansi,
- * or it completely replaces the whole lot with a new set of routines
- * which count time relative to the EPOCH date. Default is to use the
+ * or it completely replaces the whole lot with a new set of routines,
+ * which count time relative to the year 2000. Default is to use the
  * new routines. Define USE_SYSTEM_TIME to use the system routines, at your
  * own risk. One problem in particular is that not all systems allow
  * the time with integer value 0 to be represented symbolically, which
@@ -74,7 +78,7 @@ static int gdysize __PROTO((int yr));
 
 static int mndday[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-static size_t xstrftime __PROTO((char *buf, int bsz, const char *fmt, struct tm * tm, double usec, double fulltime));
+static size_t xstrftime __PROTO((char *buf, size_t bufsz, const char *fmt, struct tm * tm, double usec));
 
 /* days in year */
 static int
@@ -90,28 +94,16 @@ gdysize(int yr)
 }
 
 
-/* gstrptime() interprets a time_spec format string
- * and fills in a time structure that can be passed to gmtime() to
- * recover number of seconds from the EPOCH date.
- * Return value:
- * DT_TIMEDATE	indicates "normal" format elements corresponding to
- *		a date that is returned in tm, with fractional seconds
- *		returned in usec
- * DT_DMS	indicates relative time format elements were encountered
- *		(tH tM tS).  The relative time in seconds is returned
- *		in reltime.
- * DT_BAD	time format could not be interpreted
- *
- * parameters and return values revised for gnuplot version 5.3
- */
+/* new strptime() and gmtime() to allow time to be read as 24 hour,
+ * and spaces in the format string. time is converted to seconds from
+ * year 2000.... */
 
-td_type
-gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
+char *
+gstrptime(char *s, char *fmt, struct tm *tm, double *usec)
 {
-    int yday = 0;
-    TBOOLEAN sanity_check_date = FALSE;
-    TBOOLEAN reltime_formats = FALSE;
+    int yday, date;
 
+    date = yday = 0;
     tm->tm_mday = 1;
     tm->tm_mon = tm->tm_hour = tm->tm_min = tm->tm_sec = 0;
     /* make relative times work (user-defined tic step) */
@@ -125,34 +117,9 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
     /* we do not yet calculate wday or yday, so make them illegal
      * [but yday will be read by %j]
      */
+
     tm->tm_yday = tm->tm_wday = -1;
 
-    /* If the format requests explicit day, month, or year, then we will
-     * do sanity checking to make sure the input makes sense.
-     * For backward compatibility with gnuplot versions through 4.6.6
-     * hour, minute, seconds default to zero with no error return
-     * if the corresponding field cannot be found or interpreted.
-     */ 
-    if (strstr(fmt,"%d")) {
-	tm->tm_mday = -1;
-	sanity_check_date = TRUE;
-    }
-    if (strstr(fmt,"%y") || strstr(fmt,"%Y")) {
-	tm->tm_year = -1;
-	sanity_check_date = TRUE;
-    }
-    if (strstr(fmt,"%m") || strstr(fmt,"%B") || strstr(fmt,"%b")) {
-	tm->tm_mon = -1;
-	sanity_check_date = TRUE;
-    }
-    /* Relative time formats tH tM tS cannot be mixed with date formats */
-    if (strstr(fmt,"%t")) {
-	reltime_formats = TRUE;
-	*reltime = 0.0;
-	sanity_check_date = FALSE;
-    }
-
-
     while (*fmt) {
 	if (*fmt != '%') {
 	    if (*fmt == ' ') {
@@ -209,10 +176,12 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 
 	case 'd':		/* read a day of month */
 	    s = read_int(s, 2, &tm->tm_mday);
+	    date++;
 	    break;
 
 	case 'm':		/* month number */
 	    s = read_int(s, 2, &tm->tm_mon);
+	    date++;
 	    --tm->tm_mon;
 	    break;
 
@@ -225,17 +194,19 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	     */
 	    if (tm->tm_year <= 68)
 		tm->tm_year += 100;
+	    date++;
 	    tm->tm_year += 1900;
 	    break;
 
 	case 'Y':
 	    s = read_int(s, 4, &tm->tm_year);
+	    date++;
 	    break;
 
 	case 'j':
 	    s = read_int(s, 3, &tm->tm_yday);
 	    tm->tm_yday--;
-	    sanity_check_date = TRUE;
+	    date++;
 	    yday++;
 	    break;
 
@@ -263,64 +234,22 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 		double ufraction = 0;
 		double when = strtod (s, &s) - SEC_OFFS_SYS;
 		ggmtime(tm, when);
-		if (fraction && fraction < s)
+		if (fraction)
 		    ufraction = atof(fraction);
 		if (ufraction < 1.)		/* Filter out e.g. 123.456e7 */
 		    *usec = ufraction;
-		*reltime = when;		/* not used unless return DT_DMS */
 		break;
 	    }
 
-	case 't':
-	    /* Relative time formats tH tM tS */
-	    {
-		double cont = 0;
-		fmt++;
-		if (*fmt == 'H') {
-		    cont = 3600. * strtod(s, &s);
-		} else if (*fmt == 'M') {
-		    cont = 60. * strtod(s, &s);
-		} else if (*fmt == 'S') {
-		    cont = strtod(s, &s);
-		} else {
-		    return DT_BAD;
-		}
-		if (*reltime < 0)
-		    *reltime -= fabs(cont);
-		else
-		    *reltime += cont;
-		/* FIXME:  reltime > 0 && cont < 0 should be disallowed */
-		/* FIXME:  leading precision field should be accepted but ignored */
-		break;
-	    }
-
-	case 'a':		/* weekday name (ignored) */
-	case 'A':		/* weekday name (ignored) */
-	    while (isalpha(*s))
-		s++;
-	    break;
-	case 'w':		/* one or two digit weekday number (ignored) */
-	case 'W':		/* one or two digit week number (ignored) */
-	    if (isdigit(*s))
-		s++;
-	    if (isdigit(*s))
-		s++;
-	    break;
-
 	default:
 	    int_warn(DATAFILE, "Bad time format in string");
 	}
 	fmt++;
     }
 
-    /* Relative times are easy.  Just return the value in reltime */
-    if (reltime_formats) {
-	return DT_DMS;
-    }
-
     FPRINTF((stderr, "read date-time : %02d/%02d/%d:%02d:%02d:%02d\n", tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec));
 
-    /* now sanity check the date/time entered, normalising if necessary
+    /* now check the date/time entered, normalising if necessary
      * read_int cannot read a -ve number, but can read %m=0 then decrement
      * it to -1
      */
@@ -349,12 +278,11 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 
     FPRINTF((stderr, "normalised time : %02d/%02d/%d:%02d:%02d:%02d\n", tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec));
 
-    if (sanity_check_date) {
+    if (date) {
 	if (yday) {
 
-	    if (tm->tm_yday < 0) {
-		return DT_BAD;
-	    }
+	    if (tm->tm_yday < 0)
+		int_error(DATAFILE, "Illegal day of year");
 
 	    /* we just set month to jan, day to yday, and let the
 	     * normalising code do the work.
@@ -365,10 +293,12 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	    tm->tm_mday = tm->tm_yday + 1;
 	}
 	if (tm->tm_mon < 0) {
-	    return DT_BAD;
+	    int_error(DATAFILE, "illegal month");
+	    return (NULL);
 	}
 	if (tm->tm_mday < 1) {
-	    return DT_BAD;
+	    int_error(DATAFILE, "illegal day of month");
+	    return (NULL);
 	}
 	if (tm->tm_mon > 11) {
 	    tm->tm_year += tm->tm_mon / 12;
@@ -384,7 +314,7 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	    }
 	}
     }
-    return DT_TIMEDATE;
+    return (s);
 }
 
 size_t
@@ -397,33 +327,28 @@ gstrftime(char *s, size_t bsz, const char *fmt, double l_clock)
 
     usec = l_clock - (double)floor(l_clock);
 
-    return xstrftime(s, bsz, fmt, &tm, usec, l_clock);
+    return xstrftime(s, bsz, fmt, &tm, usec);
 }
 
 
 static size_t
 xstrftime(
     char *str,			/* output buffer */
-    int bsz,			/* remaining space available in buffer */
+    size_t bsz,			/* space available */
     const char *fmt,
     struct tm *tm,
-    double usec,
-    double fulltime)
+    double usec)
 {
     size_t l = 0;			/* chars written so far */
-    int incr = 0;			/* chars just written */
+
     char *s = str;
-    TBOOLEAN sign_printed = FALSE;
 
-    if (bsz <= 0)
-	return 0;
-
-    memset(str, '\0', bsz);
+    memset(s, '\0', bsz);
 
     while (*fmt != '\0') {
 	if (*fmt != '%') {
-	    if (l >= bsz-1)
-		return 0;
+	    if (l >= bsz)
+		return (0);
 	    *s++ = *fmt++;
 	    l++;
 	} else {
@@ -475,8 +400,8 @@ xstrftime(
 			if (!z)					\
 			    z=(dz);				\
 		    }						\
-		    incr = snprintf(s, bsz-l-1, z ? "%0*d" : "%*d", w, (x));	\
-		    CHECK_SPACE(incr);				\
+		    CHECK_SPACE(w);				\
+		    sprintf(s, z ? "%0*d" : "%*d", w, (x));	\
 		} while(0)
 
 	    case '%':
@@ -501,18 +426,27 @@ xstrftime(
 		COPY_STRING(full_month_names[tm->tm_mon]);
 		break;
 
+
+#if 0
+		/* %x not currently supported, so neither is c */
+	    case 'c':
+		if (!xstrftime(s, bsz - l, "%x %X", tm, 0.))
+		    return (0);
+		break;
+#endif
+
 	    case 'd':
 		FORMAT_STRING(1, 2, tm->tm_mday);	/* %02d */
 		break;
 
 	    case 'D':
-		if (!xstrftime(s, bsz - l, "%m/%d/%y", tm, 0., fulltime))
-		    return 0;
+		if (!xstrftime(s, bsz - l, "%m/%d/%y", tm, 0.))
+		    return (0);
 		break;
 
 	    case 'F':
-		if (!xstrftime(s, bsz - l, "%Y-%m-%d", tm, 0., fulltime))
-		    return 0;
+		if (!xstrftime(s, bsz - l, "%Y-%m-%d", tm, 0.))
+		    return (0);
 		break;
 
 	    case 'H':
@@ -550,13 +484,13 @@ xstrftime(
 		break;
 
 	    case 'r':
-		if (!xstrftime(s, bsz - l, "%I:%M:%S %p", tm, 0., fulltime))
-		    return 0;
+		if (!xstrftime(s, bsz - l, "%I:%M:%S %p", tm, 0.))
+		    return (0);
 		break;
 
 	    case 'R':
-		if (!xstrftime(s, bsz - l, "%H:%M", tm, 0., fulltime))
-		    return 0;
+		if (!xstrftime(s, bsz - l, "%H:%M", tm, 0.))
+		    return (0);
 		break;
 
 	    case 's':
@@ -578,81 +512,10 @@ xstrftime(
 		break;
 
 	    case 'T':
-		if (!xstrftime(s, bsz - l, "%H:%M:%S", tm, 0., fulltime))
-		    return 0;
+		if (!xstrftime(s, bsz - l, "%H:%M:%S", tm, 0.))
+		    return (0);
 		break;
 
-	    case 't':		/* Time (as opposed to Date) formats */
-		{
-		int tminute, tsecond;
-
-		    switch (*fmt++) {
-		    case 'H':
-			/* +/- fractional hours (not wrapped at 24h) */
-			if (p > 0) {
-			    incr = snprintf(s, bsz-l-1, "%*.*f", w, p, fulltime/3600.);
-			    CHECK_SPACE(incr);
-			    break;
-			}
-			/* Set flag in case minutes come next */
-			if (fulltime < 0) {
-			    CHECK_SPACE(1);	/* the minus sign */
-			    *s++ = '-';
-			    l++;
-			}
-			sign_printed = TRUE;
-			/* +/- integral hour truncated toward zero */
-			sprintf(s, "%0*d", w, (int)floor(fabs(fulltime/3600.)));
-
-			/* Subtract the hour component from the total */
-			fulltime -= sgn(fulltime) * 3600. * floor(fabs(fulltime/3600.));
-			break;
-		    case 'M':
-			/* +/- fractional minutes (not wrapped at 60m) */
-			if (p > 0) {
-			    incr = snprintf(s, bsz-l-1, "%*.*f", w, p,
-				    sign_printed ? fabs(fulltime)/60. : fulltime/60.);
-			    CHECK_SPACE(incr);
-			    break;
-			}
-			/* +/- integral minute truncated toward zero */
-			tminute = floor((fabs(fulltime/60.)));
-			if (fulltime < 0 && !sign_printed) {
-				*s++ = '-';
-				l++;
-			}
-			sign_printed = TRUE;
-			FORMAT_STRING(1, 2, tminute);	/* %02d */
-
-			/* Subtract the minute component from the total */
-			fulltime -= sgn(fulltime) * 60. * floor(fabs(fulltime/60.));
-			break;
-		    case 'S':
-			/* +/- fractional seconds */
-			tsecond = floor(fabs(fulltime));
-			if (fulltime < 0) {
-			    if (usec > 0)
-				usec = 1.0 - usec;
-			    if (!sign_printed) {
-				*s++ = '-';
-				l++;
-			    }
-			}
-			FORMAT_STRING(1, 2, tsecond);	/* %02d */
-			if (p > 0) {
-			    double base = pow(10., (double)p);
-			    int msec = floor(0.5 + base * usec);
-			    char *f = &s[strlen(s)];
-			    CHECK_SPACE(p+1);
-			    sprintf(f, ".%0*d", p, msec<(int)base?msec:(int)base-1);
-			}
-			break;
-		    default:
-			break;
-		    }
-		    break;
-		}
-
 	    case 'W':		/* mon 1 day of week */
 		{
 		    int week;
@@ -717,6 +580,11 @@ xstrftime(
 		FORMAT_STRING(1, 4, tm->tm_year);	/* %04d */
 		break;
 
+#if 0
+	    case 'Z':
+		COPY_STRING(tm->tm_zone);
+		break;
+#endif
 	    }			/* switch */
 
 	    while (*s != '\0') {
@@ -823,12 +691,6 @@ ggmtime(struct tm *tm, double l_clock)
     while (days >= (i = mndday[tm->tm_mon] + (tm->tm_mon == 1 && (gdysize(tm->tm_year) > 365)))) {
 	days -= i;
 	tm->tm_mon++;
-	/* This catches round-off error that initially assigned a date to year N-1 */
-	/* but counting out seconds puts it in the first second of Jan year N.     */
-	if (tm->tm_mon > 11) {
-	    tm->tm_mon = 0;
-	    tm->tm_year++;
-	}
     }
     tm->tm_mday = days + 1;
 
