@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: util.c,v 1.128 2014/05/11 17:39:59 broeker Exp $"); }
+#endif
+
 /* GNUPLOT - util.c */
 
 /*[
@@ -34,18 +38,23 @@
 
 #include "alloc.h"
 #include "command.h"
-#include "datablock.h"
 #include "datafile.h"		/* for df_showdata and df_reset_after_error */
 #include "internal.h"		/* for eval_reset_after_error */
 #include "misc.h"
 #include "plot.h"
+#include "term_api.h"		/* for term_end_plot() used by graph_error(), also to detect enhanced mode */
 #include "variable.h"		/* For locale handling */
 #include "setshow.h"		/* for conv_text() */
 #include "tabulate.h"		/* for table_mode */
 
-#if defined(HAVE_DIRENT_H) && !defined(_WIN32)
+#if defined(HAVE_DIRENT_H)
 # include <sys/types.h>
 # include <dirent.h>
+#elif defined(_Windows)
+# include <windows.h>
+#endif
+#if defined(__MSC__) || defined (__WATCOMC__)
+# include <io.h>
 #endif
 
 /* Exported (set-table) variables */
@@ -56,12 +65,6 @@ char *decimalsign = NULL;
 /* degree sign.  Defaults to UTF-8 but will be changed to match encoding */
 char degree_sign[8] = "°";
 
-/* encoding-specific characters used by gprintf() */
-const char *micro = NULL;
-const char *minus_sign = NULL;
-TBOOLEAN use_micro = FALSE;
-TBOOLEAN use_minus_sign = FALSE;
-
 /* Holds the name of the current LC_NUMERIC as set by "set decimal locale" */
 char *numeric_locale = NULL;
 
@@ -70,12 +73,6 @@ char *current_locale = NULL;
 
 const char *current_prompt = NULL; /* to be set by read_line() */
 
-/* TRUE if command just typed; becomes FALSE whenever we
- * send some other output to screen.  If FALSE, the command line
- * will be echoed to the screen before the ^ error message.
- */
-TBOOLEAN screen_ok;
-
 /* internal prototypes */
 
 static void mant_exp __PROTO((double, double, TBOOLEAN, double *, int *, const char *));
@@ -165,7 +162,7 @@ type_udv(int t_num)
 
     while (*udv_ptr) {
 	if (equals(t_num, (*udv_ptr)->udv_name)) {
-	    if ((*udv_ptr)->udv_value.type == NOTDEFINED)
+	    if ((*udv_ptr)->udv_undef)
 		return 0;
 	    else
 		return (*udv_ptr)->udv_value.type;
@@ -190,22 +187,6 @@ isletter(int t_num)
 	    (isalpha(c) || (c == '_') || ALLOWED_8BITVAR(c)));
 }
 
-/* Test whether following bit of command line might be parsable as a number.
- * constant, defined variable, function, ...
- */
-TBOOLEAN
-might_be_numeric(int t_num)
-{
-    if (END_OF_COMMAND)
-	return FALSE;
-    if (isanumber(t_num) || is_function(t_num))
-	return TRUE;
-    if (type_udv(t_num) == INTGR || type_udv(t_num) == CMPLX || type_udv(t_num) == ARRAY)
-	return TRUE;
-    if (equals(t_num, "("))
-	return TRUE;
-    return FALSE;
-}
 
 /*
  * is_definition() returns TRUE if the next tokens are of the form
@@ -274,6 +255,42 @@ token_len(int t_num)
     return (size_t)(token[t_num].length);
 }
 
+#ifdef NEXT
+/*
+ * quote_str() no longer has any callers in the core code.
+ * However, it is called by the next/openstep terminal.
+ */
+
+/*
+ * quote_str() does the same thing as copy_str, except it ignores the
+ *   quotes at both ends.  This seems redundant, but is done for
+ *   efficency.
+ */
+void
+quote_str(char *str, int t_num, int max)
+{
+    int i = 0;
+    int start = token[t_num].start_index + 1;
+    int count;
+
+    if ((count = token[t_num].length - 2) >= max) {
+	count = max - 1;
+	FPRINTF((stderr, "str buffer overflow in quote_str"));
+    }
+    if (count > 0) {
+	do {
+	    str[i++] = gp_input_line[start++];
+	} while (i != count);
+    }
+    str[i] = NUL;
+    /* convert \t and \nnn (octal) to char if in double quotes */
+    if (gp_input_line[token[t_num].start_index] == '"')
+	parse_esc(str);
+    else
+	parse_sq(str);
+}
+#endif
+
 /*
  * capture() copies into str[] the part of gp_input_line[] which lies between
  * the begining of token[start] and end of token[end].
@@ -338,7 +355,9 @@ m_quote_capture(char **str, int start, int end)
 }
 
 /*
- * Wrapper for isstring + m_quote_capture or const_string_express
+ * Wrapper for isstring + m_quote_capture that can be used with
+ * or without GP_STRING_VARS enabled.
+ * EAM Aug 2004
  */
 char *
 try_to_get_string()
@@ -536,23 +555,14 @@ gprintf(
 
     char *dest  = &tempdest[0];
     char *limit = &tempdest[MAX_LINE_LEN];
-    static double log10_of_1024; /* to avoid excess precision comparison in check of connection %b -- %B */
-    
-    log10_of_1024 = log10(1024);
-    
 #define remaining_space (size_t)(limit-dest)
 
     *dest = '\0';
 
     set_numeric_locale();
 
-    /* Should never happen but fuzzer managed to hit it */
-    if (!format)
-	format = DEF_FORMAT;
-
-    /* By default we wrap numbers output to latex terminals in $...$ */
-    if (!strcmp(format, DEF_FORMAT)  && !table_mode
-    &&  ((term->flags & TERM_IS_LATEX)))
+    /* Oct 2013 - default format is now expected to be "%h" */
+    if (((term->flags & TERM_IS_LATEX)) && !strcmp(format, DEF_FORMAT))
 	format = DEF_FORMAT_LATEX;
 
     for (;;) {
@@ -649,25 +659,14 @@ gprintf(
 				strcpy(&tmp2[j], "\\cdot");
 				j+= 5;
 			    }
-			} else switch (encoding) {
-			    case S_ENC_UTF8:
-				strcpy(&tmp2[j], "\xc3\x97"); /* UTF character '×' */
-				j+= 2;
-				break;
-			    case S_ENC_CP1252:
-				tmp2[j++] = (*format=='h') ? 0xd7 : 0xb7;
-				break;
-			    case S_ENC_ISO8859_1:
-			    case S_ENC_ISO8859_2:
-			    case S_ENC_ISO8859_9:
-			    case S_ENC_ISO8859_15:
-				tmp2[j++] = (*format=='h') ? 0xd7 : '*';
-				break;
-			    default:
-				tmp2[j++] = (*format=='h') ? 'x' : '*';
-				break;
+			} else if (encoding == S_ENC_UTF8) {
+			    strcpy(&tmp2[j], "\xc3\x97"); /* UTF character '×' */
+			    j+= 2;
+			} else if (encoding == S_ENC_CP1252) {
+			    tmp2[j++] = (*format=='h') ? 0xd7 : 0xb7;
+			} else {
+			    tmp2[j++] = (*format=='h') ? 'x' : '*';
 			}
-
 			strcpy(&tmp2[j], "10^{");
 			j += 4;
 			bracket_flag = TRUE;
@@ -747,7 +746,7 @@ gprintf(
 
 		t[0] = 'f';
 		t[1] = 0;
-		stored_power_base = log10_of_1024;
+		stored_power_base = log10(1024);
 		mant_exp(stored_power_base, x, FALSE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
@@ -762,16 +761,14 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa) {
-		    if (stored_power_base == log10_base) {
+		if (seen_mantissa)
+		    if (stored_power_base == log10_base)
 			power = stored_power;
-		    } else {
+		    else
 			int_error(NO_CARET, "Format character mismatch: %%L is only valid with %%l");
-		    }
-		} else {
+		else
 		    stored_power_base = log10_base;
 		    mant_exp(log10_base, x, FALSE, NULL, &power, "%.0f");
-		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -783,15 +780,13 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa) {
-		    if (stored_power_base == 1.0) {
+		if (seen_mantissa)
+		    if (stored_power_base == 1.0)
 			power = stored_power;
-		    } else {
+		    else
 			int_error(NO_CARET, "Format character mismatch: %%T is only valid with %%t");
-		    }
-		} else {
+		else
 		    mant_exp(1.0, x, FALSE, NULL, &power, "%.0f");
-		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -803,15 +798,13 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa) {
-		    if (stored_power_base == 1.0) {
+		if (seen_mantissa)
+		    if (stored_power_base == 1.0)
 			power = stored_power;
-		    } else {
+		    else
 			int_error(NO_CARET, "Format character mismatch: %%S is only valid with %%s");
-		    }
-		} else {
+		else
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
-		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -823,17 +816,21 @@ gprintf(
 
 		t[0] = 'c';
 		t[1] = 0;
-		if (seen_mantissa) {
-		    if (stored_power_base == 1.0) {
+		if (seen_mantissa)
+		    if (stored_power_base == 1.0)
 			power = stored_power;
-		    } else {
+		    else
 			int_error(NO_CARET, "Format character mismatch: %%c is only valid with %%s");
-		    }
-		} else {
+		else
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
-		}
 
 		if (power >= -24 && power <= 24) {
+		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
+		    /* HBB 20010121: avoid division of -ve ints! */
+		    power = (power + 24) / 3;
+		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
+		} else {
+		    /* please extend the range ! */
 		    /* name  power   name  power
 		       -------------------------
 		       yocto  -24    yotta  24
@@ -844,17 +841,7 @@ gprintf(
 		       nano    -9    Giga    9
 		       micro   -6    Mega    6
 		       milli   -3    kilo    3   */
-		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
-		    /* HBB 20010121: avoid division of -ve ints! */
-		    power = (power + 24) / 3;
-		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
 
-		    /* Replace u with micro character */
-		    if (use_micro && power == 6)
-			snprintf(dest, remaining_space, "%s%s", micro, &temp[2]);
-
-		} else {
-		    /* please extend the range ! */
 		    /* fall back to simple exponential */
 		    snprintf(dest, remaining_space, "e%+02d", power);
 		}
@@ -868,16 +855,14 @@ gprintf(
 
 		t[0] = 'c';
 		t[1] = 'i';
-		t[2] = '\0';
-		if (seen_mantissa) {
-		    if (stored_power_base == log10_of_1024) {
+		t[2] = 0;
+		if (seen_mantissa)
+		    if (stored_power_base == log10(1024))
 			power = stored_power;
-		    } else {
+		    else
 			int_error(NO_CARET, "Format character mismatch: %%B is only valid with %%b");
-		    }
-		} else {
-			mant_exp(log10_of_1024, x, FALSE, NULL, &power, "%.0f");
-		}
+		else
+			mant_exp(log10(1024), x, FALSE, NULL, &power, "%.0f");
 
 		if (power > 0 && power <= 8) {
 		    /* name  power
@@ -896,9 +881,7 @@ gprintf(
 		    snprintf(dest, remaining_space, "x2^{%d}Yi", power-8);
 		} else if (power < 0) {
 		    snprintf(dest, remaining_space, "x2^{%d}", power*10);
-		} else {
-                    snprintf(dest, remaining_space, "  ");
-                }
+		}
 
 		break;
 	    }
@@ -951,47 +934,6 @@ gprintf(
 	    }
 	}
 
-    /* EXPERIMENTAL
-     * Some people prefer a "real" minus sign to the hyphen that standard
-     * formatted input and output both use.  Unlike decimal signs, there is
-     * no internationalization mechanism to specify this preference.
-     * This code replaces all hyphens with the character string specified by
-     * 'set minus_sign "..."'   typically unicode character U+2212 "−".
-     * Use at your own risk.  Should be OK for graphical output, but text output
-     * will not be readable by standard formatted input routines.
-     */
-	if (use_minus_sign		/* set minussign */
-	    && minus_sign		/* current encoding provides one */
-	    && !table_mode		/* not used inside "set table" */
-	    && !(term->flags & TERM_IS_LATEX)	/* but LaTeX doesn't want it */
-	   ) {
-
-	    char *dotpos1 = dest;
-	    char *dotpos2;
-	    size_t newlength = strlen(minus_sign);
-
-	    /* dot is the default hyphen we will be replacing */
-	    int dot = '-';
-
-	    /* replace every dot by the contents of minus_sign */
-	    while ((dotpos2 = strchr(dotpos1,dot)) != NULL) {
-		if (newlength == 1) {	/* The normal case */
-		    *dotpos2 = *minus_sign;
-		    dotpos1++;
-		} else {		/* Some multi-byte minus marker */
-		    size_t taillength = strlen(dotpos2);
-		    dotpos1 = dotpos2 + newlength;
-		    if (dotpos1 + taillength > limit)
-			int_error(NO_CARET,
-				  "format too long due to minus_sign string");
-		    /* move tail end of string out of the way */
-		    memmove(dotpos1, dotpos2 + 1, taillength);
-		    /* insert minus_sign */
-		    memcpy(dotpos2, minus_sign, newlength);
-		}
-	    }
-	}
-
 	/* this was at the end of every single case, before: */
 	dest += strlen(dest);
 	++format;
@@ -1021,6 +963,13 @@ done:
 /* some macros for the error and warning functions below
  * may turn this into a utility function later
  */
+#define PRINT_MESSAGE_TO_STDERR				\
+do {							\
+    fprintf(stderr, "\n%s%s\n",				\
+	    current_prompt ? current_prompt : "",	\
+	    gp_input_line);				\
+} while (0)
+
 #define PRINT_SPACES_UNDER_PROMPT		\
 do {						\
     const char *p;				\
@@ -1031,71 +980,29 @@ do {						\
 	(void) fputc(' ', stderr);		\
 } while (0)
 
-/*
- * Echo back the command or data line that triggered an error,
- * possibly with a caret indicating the token the was not accepted.
- */
-static void
-print_line_with_error(int t_num)
-{
-    int i;
-    int save_line_num = inline_num;
+#define PRINT_SPACES_UPTO_TOKEN						\
+do {									\
+    int i;								\
+									\
+    for (i = 0; i < token[t_num].start_index; i++)			\
+	(void) fputc((gp_input_line[i] == '\t') ? '\t' : ' ', stderr);	\
+} while(0)
 
-    if (t_num == DATAFILE) {
-	/* Print problem line from data file to the terminal */
-	df_showdata();
+#define PRINT_CARET fputs("^\n",stderr);
 
-    } else {
-
-	/* If the current line was built by concatenation of lines inside */
-	/* a {bracketed clause}, try to reconstruct the true line number  */
-	char *copy_of_input_line = gp_strdup(gp_input_line);
-	char *minimal_input_line = copy_of_input_line;
-	char *trunc;
-	while ((trunc = strrchr(copy_of_input_line, '\n')) != NULL) {
-	    int current = (t_num == NO_CARET) ? c_token : t_num;
-	    if (trunc < &copy_of_input_line[token[current].start_index]) {
-		minimal_input_line = trunc+1;
-		t_num = NO_CARET;
-		break;
-	    }
-	    *trunc = '\0';
-	    inline_num--;
-	}
-
-	if (t_num != NO_CARET) {
-	    /* Refresh current command line */
-	    if (!screen_ok)
-		fprintf(stderr, "\n%s%s\n",
-		    current_prompt ? current_prompt : "",
-		    minimal_input_line);
-
-	    PRINT_SPACES_UNDER_PROMPT;
-
-	    /* Print spaces up to token */
-	    for (i = 0; i < token[t_num].start_index; i++)
-		fputc((minimal_input_line[i] == '\t') ? '\t' : ' ', stderr);
-
-	    /* Print token */
-	    fputs("^\n",stderr);
-	}
-	free(copy_of_input_line);
-    }
-
-    PRINT_SPACES_UNDER_PROMPT;
-
-    if (!interactive) {
-	if (lf_head && lf_head->name)
-	    fprintf(stderr, "\"%s\", ", lf_head->name);
-	fprintf(stderr, "line %d: ", inline_num);
-    }
-
-    inline_num = save_line_num;
+#define PRINT_FILE_AND_LINE						\
+if (!interactive) {							\
+    if (lf_head && lf_head->name)                                       \
+	fprintf(stderr, "\"%s\", line %d: ", lf_head->name, inline_num);\
+    else fprintf(stderr, "line %d: ", inline_num);			\
 }
 
-/*
- * os_error() is just like int_error() except that it calls perror().
+/* TRUE if command just typed; becomes FALSE whenever we
+ * send some other output to screen.  If FALSE, the command line
+ * will be echoed to the screen before the ^ error message.
  */
+TBOOLEAN screen_ok;
+
 #if defined(VA_START) && defined(STDC_HEADERS)
 void
 os_error(int t_num, const char *str,...)
@@ -1112,8 +1019,17 @@ os_error(int t_num, const char *str, va_dcl)
 #endif /* VMS */
 
     /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
 
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) {	/* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
     PRINT_SPACES_UNDER_PROMPT;
 
 #ifdef VA_START
@@ -1129,16 +1045,21 @@ os_error(int t_num, const char *str, va_dcl)
 #endif
     putc('\n', stderr);
 
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
+
 #ifdef VMS
     status[1] = vaxc$errno;
     sys$putmsg(status);
+    (void) putc('\n', stderr);
 #else /* VMS */
-    perror("system error");
+    perror("util.c");
+    putc('\n', stderr);
 #endif /* VMS */
 
-    putc('\n', stderr);
-    fill_gpval_string("GPVAL_ERRMSG", strerror(errno));
-    common_error_exit();
+    scanning_range_in_progress = FALSE;
+
+    bail_to_command_line();
 }
 
 
@@ -1157,7 +1078,19 @@ int_error(int t_num, const char str[], va_dcl)
     char error_message[128] = {'\0'};
 
     /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
+
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) { /* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
 
 #ifdef VA_START
     VA_START(args, str);
@@ -1174,30 +1107,19 @@ int_error(int t_num, const char str[], va_dcl)
 #endif
 
     fputs("\n\n", stderr);
-    fill_gpval_string("GPVAL_ERRMSG", error_message);
-    common_error_exit();
-}
 
-
-void
-common_error_exit()
-{
     /* We are bailing out of nested context without ever reaching */
     /* the normal cleanup code. Reset any flags before bailing.   */
     df_reset_after_error();
     eval_reset_after_error();
     clause_reset_after_error();
     parse_reset_after_error();
-    set_iterator = cleanup_iteration(set_iterator);
-    plot_iterator = cleanup_iteration(plot_iterator);
     scanning_range_in_progress = FALSE;
     inside_zoom = FALSE;
-#ifdef HAVE_LOCALE_H
-    setlocale(LC_NUMERIC, "C");
-#endif
 
     /* Load error state variables */
     update_gpval_variables(2);
+    fill_gpval_string("GPVAL_ERRMSG", error_message);
 
     bail_to_command_line();
 }
@@ -1216,7 +1138,19 @@ int_warn(int t_num, const char str[], va_dcl)
 #endif
 
     /* reprint line if screen has been written to */
-    print_line_with_error(t_num);
+
+    if (t_num == DATAFILE) {
+	df_showdata();
+    } else if (t_num != NO_CARET) { /* put caret under error */
+	if (!screen_ok)
+	    PRINT_MESSAGE_TO_STDERR;
+
+	PRINT_SPACES_UNDER_PROMPT;
+	PRINT_SPACES_UPTO_TOKEN;
+	PRINT_CARET;
+    }
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
 
     fputs("warning: ", stderr);
 #ifdef VA_START
@@ -1233,15 +1167,55 @@ int_warn(int t_num, const char str[], va_dcl)
     putc('\n', stderr);
 }
 
+/*{{{  graph_error() */
+/* handle errors during graph-plot in a consistent way */
+/* HBB 20000430: move here, from graphics.c */
+#if defined(VA_START) && defined(STDC_HEADERS)
+void
+graph_error(const char *fmt, ...)
+#else
+void
+graph_error(const char *fmt, va_dcl)
+#endif
+{
+#ifdef VA_START
+    va_list args;
+#endif
+
+    multiplot = FALSE;
+    term_end_plot();
+
+#ifdef VA_START
+    VA_START(args, fmt);
+    /* HBB 20001120: instead, copy the core code from int_error() to
+     * here: */
+    PRINT_SPACES_UNDER_PROMPT;
+    PRINT_FILE_AND_LINE;
+
+# if defined(HAVE_VFPRINTF) || _LIBC
+    vfprintf(stderr, fmt, args);
+# else
+    _doprnt(fmt, args, stderr);
+# endif
+    va_end(args);
+    fputs("\n\n", stderr);
+
+    bail_to_command_line();
+    va_end(args);
+#else
+    int_error(NO_CARET, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
+#endif
+
+}
+
 /*}}} */
 
 
-/*
- * Reduce all multiple white-space chars to single spaces (if remain == 1)
- * or remove altogether (if remain == 0).  Modifies the original string.
- */
+/* Squash spaces in the given string (DFK) */
+/* That is, reduce all multiple white-space chars to single spaces */
+/* Done in place. Currently used only by help_command() */
 void
-squash_spaces(char *s, int remain)
+squash_spaces(char *s)
 {
     char *r = s;	/* reading point */
     char *w = s;	/* writing point */
@@ -1250,7 +1224,7 @@ squash_spaces(char *s, int remain)
     for (w = r = s; *r != NUL; r++) {
 	if (isspace((unsigned char) *r)) {
 	    /* white space; only copy if we haven't just copied a space */
-	    if (!space && remain > 0) {
+	    if (!space) {
 		space = TRUE;
 		*w++ = ' ';
 	    }			/* else ignore multiple spaces */
@@ -1337,15 +1311,25 @@ parse_esc(char *instr)
 /* FIXME HH 20020915: This function does nothing if dirent.h and windows.h
  * not available. */
 TBOOLEAN
-existdir(const char *name)
+existdir (const char *name)
 {
-#if defined(HAVE_DIRENT_H ) || defined(_WIN32)
+#ifdef HAVE_DIRENT_H
     DIR *dp;
-    if ((dp = opendir(name)) == NULL)
+    if (! (dp = opendir(name) ) )
 	return FALSE;
 
     closedir(dp);
     return TRUE;
+#elif defined(_Windows)
+    HANDLE FileHandle;
+    WIN32_FIND_DATA finddata;
+
+    FileHandle = FindFirstFile(name, &finddata);
+    if (FileHandle != INVALID_HANDLE_VALUE) {
+	if (finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+	    return TRUE;
+    }
+    return FALSE;
 #elif defined(VMS)
     return FALSE;
 #else
@@ -1476,35 +1460,11 @@ strlen_utf8(const char *s)
     return j;
 }
 
-
-TBOOLEAN
-is_sjis_lead_byte(char c)
-{
-    unsigned int ch = (unsigned char) c;
-    return ((ch >= 0x81) && (ch <= 0x9f)) || ((ch >= 0xe1) && (ch <= 0xee));
-}
-
-
-size_t
-strlen_sjis(const char *s)
-{
-    int i = 0, j = 0;
-    while (s[i]) {
-	if (is_sjis_lead_byte(s[i])) i++; /* skip */
-	j++;
-	i++;
-    }
-    return j;
-}
-
-
 size_t
 gp_strlen(const char *s)
 {
     if (encoding == S_ENC_UTF8)
 	return strlen_utf8(s);
-    else if (encoding == S_ENC_SJIS)
-	return strlen_sjis(s);
     else
 	return strlen(s);
 }
@@ -1550,8 +1510,8 @@ streq(const char *a, const char *b)
     while (isspace((unsigned char)*b))
 	b++;
 
-    enda = (*a) ? strlen(a) - 1 : 0;
-    endb = (*b) ? strlen(b) - 1 : 0;
+    enda = strlen(a) - 1;
+    endb = strlen(b) - 1;
 
     while (isspace((unsigned char)a[enda]))
 	enda--;
@@ -1572,8 +1532,7 @@ strappend(char **dest, size_t *size, size_t len, const char *src)
     size_t destlen = (len != 0) ? len : strlen(*dest);
     size_t srclen = strlen(src);
     if (destlen + srclen + 1 > *size) {
-	while (destlen + srclen + 1 > *size)
-	    *size *= 2;
+	*size *= 2;
 	*dest = (char *) gp_realloc(*dest, *size, "strappend");
     }
     memcpy(*dest + destlen, src, srclen + 1);
@@ -1636,17 +1595,13 @@ value_to_str(struct value *val, TBOOLEAN need_quotes)
 	break;
     case DATABLOCK:
 	{
-	sprintf(s[j], "<%d line data block>", datablock_size(val));
-	break;
+	char **dataline = val->v.data_array;
+	int nlines = 0;
+	if (dataline != NULL) {
+	    while (*dataline++ != NULL)
+		nlines++;
 	}
-    case ARRAY:
-	{
-	sprintf(s[j], "<%d element array>", val->v.value_array->v.int_val);
-	break;
-	}
-    case NOTDEFINED:
-	{
-	sprintf(s[j], "<undefined>");
+	sprintf(s[j], "<%d line data block>", nlines);
 	break;
 	}
     default:
