@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: version.c,v 1.111.2.1 2014/08/22 16:07:34 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - version.c */
 
 /*[
@@ -35,15 +39,14 @@
 #include "syscfg.h"		/* for FAQ_LOCATION */
 
 
-const double gnuplot_ver = 5.4;
-const char gnuplot_version[] = "5.4";
-const char gnuplot_patchlevel[] = "10";
+const char gnuplot_version[] = "5.0";
+const char gnuplot_patchlevel[] = "rc2";
 #ifdef DEVELOPMENT_VERSION
 #include "timestamp.h"
 #else
-const char gnuplot_date[] = "2023-10-20";
+const char gnuplot_date[] = "2014-08-22 ";
 #endif
-const char gnuplot_copyright[] = "Copyright (C) 1986-1993, 1998, 2004, 2007-2023";
+const char gnuplot_copyright[] = "Copyright (C) 1986-1993, 1998, 2004, 2007-2014";
 
 const char faq_location[] = FAQ_LOCATION;
 
