@@ -1,3 +1,7 @@
+/*
+ * $Id: gp_cairo.c,v 1.87 2014/08/20 04:42:44 sfeam Exp $
+ */
+
 /* GNUPLOT - gp_cairo.c */
 
 /*[
@@ -118,15 +122,6 @@ static int avg_vchar = 150;
 static void gp_cairo_fill(plot_struct *plot, int fillstyle, int fillpar);
 static void gp_cairo_fill_pattern(plot_struct *plot, int fillstyle, int fillpar);
 
-/* Boxed text support */
-static int bounding_box[4];
-static double bounding_xmargin = 1.0;
-static double bounding_ymargin = 1.0;
-static double box_rotation = 0.0;
-static double box_origin_x;
-static double box_origin_y;
-static TBOOLEAN in_textbox = FALSE;
-
 /* array of colors
  * FIXME could be shared with all gnuplot terminals */
 static rgb_color gp_cairo_colorlist[12] = {
@@ -144,7 +139,7 @@ static rgb_color gp_cairo_colorlist[12] = {
 {0.5,0.5,0.5} /* grey */
 };
 
-/* correspondence between gnuplot linetypes and terminal colors */
+/* correspondance between gnuplot linetypes and terminal colors */
 void gp_cairo_set_background( rgb_color background )
 {
 	gp_cairo_colorlist[0] = background;
@@ -177,9 +172,7 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 
 	plot->opened_path = FALSE;
 
-	plot->current_x = -1; plot->current_y = -1;
-
-	safe_strncpy(plot->fontname, "", sizeof(plot->fontname));
+	strncpy(plot->fontname, "", sizeof(plot->fontname));
 	plot->fontsize = 1.0;
 	plot->encoding = S_ENC_DEFAULT;
 
@@ -190,8 +183,6 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 	plot->oversampling = TRUE;
 	plot->oversampling_scale = GP_CAIRO_SCALE;
 
-	plot->upsampling_rate = 1.0;
-
 	plot->linecap = BUTT;
 
 	plot->hinting = 100;
@@ -203,10 +194,15 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 	plot->polygon_path_last = NULL;
 
 	plot->interrupt = FALSE;
-
-	in_textbox = FALSE;
 }
 
+#ifdef EAM_BOXED_TEXT
+/* Boxed text support */
+static int bounding_box[4];
+static double bounding_xmargin = 1.0;
+static double bounding_ymargin = 1.0;
+#endif
+
 /* set the transformation matrix of the context, and other details */
 /* NOTE : depends on the setting of xscale and yscale */
 void gp_cairo_initialize_context(plot_struct *plot)
@@ -230,11 +226,6 @@ void gp_cairo_initialize_context(plot_struct *plot)
 			0.5, 0.5);
 	cairo_set_matrix(plot->cr, &matrix);
 
-	gp_cairo_set_lineprops(plot);
-}
-
-void gp_cairo_set_lineprops(plot_struct *plot)
-{
 	/* Default is square caps, mitered joins */
 	if (plot->linecap == ROUNDED) {
 	    cairo_set_line_cap  (plot->cr, CAIRO_LINE_CAP_ROUND);
@@ -248,6 +239,7 @@ void gp_cairo_set_lineprops(plot_struct *plot)
 	    cairo_set_line_join (plot->cr, CAIRO_LINE_JOIN_MITER);
 	    cairo_set_miter_limit(plot->cr, 3.8);
 	}
+
 }
 
 
@@ -307,12 +299,12 @@ void gp_cairo_set_justify(plot_struct *plot, JUSTIFY mode)
 }
 
 
-void gp_cairo_set_font(plot_struct *plot, const char *name, float fontsize)
+void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize)
 {
     char *c;
     char *fname;
 
-	FPRINTF((stderr,"set_font \"%s\" %f\n", name,fontsize));
+	FPRINTF((stderr,"set_font \"%s\" %d\n", name,fontsize));
 
 	/* Split out Bold and Italic attributes from font name */
 	fname = strdup(name);
@@ -336,17 +328,11 @@ void gp_cairo_set_font(plot_struct *plot, const char *name, float fontsize)
 	} else
 	    plot->fontstyle = PANGO_STYLE_NORMAL;
 
-	safe_strncpy( plot->fontname, fname, sizeof(plot->fontname) );
+	strncpy( plot->fontname, fname, sizeof(plot->fontname) );
 	plot->fontsize = fontsize;
 	free(fname);
 }
 
-/* work-around for "bold font gets stuck" bug */
-void gp_cairo_clear_bold_font(plot_struct *plot)
-{
-	plot->fontweight = PANGO_WEIGHT_NORMAL;
-	plot->fontstyle = PANGO_STYLE_NORMAL;
-}
 
 void gp_cairo_set_linewidth(plot_struct *plot, double linewidth)
 {
@@ -357,13 +343,13 @@ void gp_cairo_set_linewidth(plot_struct *plot, double linewidth)
 	/* draw any open polygon set */
 	gp_cairo_end_polygon(plot);
 
-	if (!strcmp(term->name,"pdfcairo"))
-		linewidth *= 2;
-
-	if (linewidth < 0.20)	/* Admittedly arbitrary */
-	    linewidth = 0.20;
+	if (linewidth < 0.25)	/* Admittedly arbitrary */
+	    linewidth = 0.25;
 	plot->linewidth = linewidth;
 
+	if (!strcmp(term->name,"pdfcairo"))
+		plot->linewidth *= 2;
+
 }
 
 
@@ -401,9 +387,6 @@ void gp_cairo_draw_polygon(plot_struct *plot, int n, gpiPoint *corners)
 	/* begin by stroking any open path */
 	gp_cairo_stroke(plot);
 
-	if (n <= 0)
-		return;
-
 	if (plot->polygons_saturate) {
 		int i;
 		path_item *path;
@@ -467,15 +450,13 @@ void gp_cairo_end_polygon(plot_struct *plot)
 	/* if there's only one polygon, draw it directly */
 	if (path->previous == NULL) {
 		FPRINTF((stderr,"processing one polygon\n"));
-		if (path->n > 0) {
-		    cairo_move_to(plot->cr, path->corners[0].x, path->corners[0].y);
-		    for (i = 1; i < path->n; i++)
-			    cairo_line_to(plot->cr, path->corners[i].x, path->corners[i].y);
-		    cairo_close_path(plot->cr);
-		    plot->color = path->color;
-		    gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
-		    cairo_fill(plot->cr);
-		}
+		cairo_move_to(plot->cr, path->corners[0].x, path->corners[0].y);
+		for (i=1;i<path->n;++i)
+			cairo_line_to(plot->cr, path->corners[i].x, path->corners[i].y);
+		cairo_close_path(plot->cr);
+		plot->color = path->color;
+		gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
+		cairo_fill(plot->cr);
 		free(path->corners);
 		free(path);
 		plot->polygon_path_last = NULL;
@@ -494,8 +475,8 @@ void gp_cairo_end_polygon(plot_struct *plot)
 	context_sav = plot->cr;
 	surface = cairo_surface_create_similar(cairo_get_target(plot->cr),
                                              CAIRO_CONTENT_COLOR_ALPHA,
-                                             plot->device_xmax*plot->upsampling_rate*SCALE,
-                                             plot->device_ymax*plot->upsampling_rate*SCALE);
+                                             plot->device_xmax*SCALE,
+                                             plot->device_ymax*SCALE);
 	context = cairo_create(surface);
 	cairo_set_operator(context,CAIRO_OPERATOR_SATURATE);
 	if (plot->antialiasing)
@@ -527,7 +508,7 @@ void gp_cairo_end_polygon(plot_struct *plot)
 		plot->color = path->color;
 		gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
 		cairo_fill(plot->cr);
-		/* free the resources, and go to the next point */
+		/* free the ressources, and go to the next point */
 		free(path->corners);
 		path2 = path->previous;
 		free(path);
@@ -566,6 +547,8 @@ void gp_cairo_set_dashtype(plot_struct *plot, int type, t_dashtype *custom_dash_
 	};
 	int lt = (type) % 5;
 
+	FPRINTF((stderr,"set_dashtype %d\n", type));
+
 	if (type == DASHTYPE_CUSTOM && custom_dash_type) {
 		/* Convert to internal representation */
 		int i;
@@ -583,7 +566,13 @@ void gp_cairo_set_dashtype(plot_struct *plot, int type, t_dashtype *custom_dash_
 			plot->current_dashpattern[i] = custom_dash_type->pattern[i]
 				* plot->dashlength
 				* plot->oversampling_scale * empirical_scale;
-		gp_cairo_set_linestyle(plot, GP_CAIRO_DASH);
+		FPRINTF((stderr,"gp_cairo_set_dashtype: custom pattern\n"));
+		FPRINTF((stderr,"	%f %f %f %f %f %f %f %f\n",
+			plot->current_dashpattern[0], plot->current_dashpattern[1],
+			plot->current_dashpattern[2], plot->current_dashpattern[3],
+			plot->current_dashpattern[4], plot->current_dashpattern[5],
+			plot->current_dashpattern[6], plot->current_dashpattern[7]));
+		plot->linestyle = GP_CAIRO_DASH;
 
 	} else if (type > 0 && lt != 0) {
 		/* Use old (version 4) set of linetype patterns */
@@ -597,12 +586,13 @@ void gp_cairo_set_dashtype(plot_struct *plot, int type, t_dashtype *custom_dash_
 				* plot->dashlength
 				* plot->oversampling_scale
 				* empirical_scale;
-		gp_cairo_set_linestyle(plot, GP_CAIRO_DASH);
+		plot->linestyle = GP_CAIRO_DASH;
 
 	} else {
 		/* Every 5th pattern in the old set is solid */
-		gp_cairo_set_linestyle(plot, GP_CAIRO_SOLID);
+		plot->linestyle = GP_CAIRO_SOLID;
 	}
+
 }
 
 void gp_cairo_stroke(plot_struct *plot)
@@ -621,11 +611,10 @@ void gp_cairo_stroke(plot_struct *plot)
 
 	cairo_save(plot->cr);
 
-	if (plot->linetype == LT_NODRAW) {
-		cairo_set_operator(plot->cr, CAIRO_OPERATOR_DEST);
-		lw = 0.0;
+	if (plot->linetype == LT_NODRAW)
+		cairo_set_operator(plot->cr, CAIRO_OPERATOR_XOR);
 
-	} else if (lt == LT_AXIS || plot->linestyle == GP_CAIRO_DOTS) {
+	else if (lt == LT_AXIS || plot->linestyle == GP_CAIRO_DOTS) {
 		/* Grid lines (lt 0) */
 		double dashes[2];
 		double empirical_scale = 1.0;
@@ -654,16 +643,13 @@ void gp_cairo_stroke(plot_struct *plot)
 
 void gp_cairo_move(plot_struct *plot, int x, int y)
 {
-	/* Dec 2014 - Do not let zero-length moves interrupt     */
-	/* the current line/polyline context, e.g. dash pattern. */
-	if (x == plot->current_x && y == plot->current_y)
-		return;
-
 	/* begin by stroking any open path */
 	gp_cairo_stroke(plot);
 	/* also draw any open polygon set */
 	gp_cairo_end_polygon(plot);
 
+	FPRINTF((stderr,"move\n"));
+
 	plot->current_x = x;
 	plot->current_y = y;
 	plot->orig_current_x = x;
@@ -756,7 +742,7 @@ static gchar * gp_cairo_convert(plot_struct *plot, const char* string)
 		if (error->code != G_CONVERT_ERROR_ILLEGAL_SEQUENCE) {
 			fprintf(stderr, "Unable to convert \"%s\": %s\n", string, error->message);
 			g_error_free (error);
-			return strdup("");
+			return "";
 		}
 		/* The sequence is invalid in the chosen charset.
 		 * we will try to fall back to iso_8859_1, and if it doesn't work,
@@ -781,14 +767,14 @@ static gchar * gp_cairo_convert(plot_struct *plot, const char* string)
 }
 
 /*
- * The following #ifdef _WIN32 section is all to work around a bug in
+ * The following #ifdef WIN32 section is all to work around a bug in
  * the cairo/win32 backend for font rendering.  It has the effect of
  * testing for libfreetype support, and using that instead if possible.
  * Suggested by cairo developer Behdad Esfahbod.
  * Allin Cottrell suggests that this not necessary anymore for newer
  * versions of cairo.
  */
-#if defined(_WIN32) && (CAIRO_VERSION_MAJOR < 2) && (CAIRO_VERSION_MINOR < 10)
+#if defined(WIN32) && (CAIRO_VERSION_MAJOR < 2) && (CAIRO_VERSION_MINOR < 10)
 static PangoLayout *
 gp_cairo_create_layout(cairo_t *cr)
 {
@@ -810,7 +796,7 @@ gp_cairo_create_layout(cairo_t *cr)
 #endif
 
     layout = pango_layout_new(context);
-    g_clear_object (&context);
+    g_object_unref(context);
 
     return layout;
 }
@@ -822,18 +808,12 @@ gp_cairo_create_layout(cairo_t *cr)
 }
 #endif
 
-void gp_cairo_set_resolution(int dpi)
-{
-	pango_cairo_font_map_set_resolution(PANGO_CAIRO_FONT_MAP(pango_cairo_font_map_get_default()), dpi);
-}
-
 void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 		    int *width, int *height)
 {
 	double x,y;
-	double arg;
+	double arg = plot->text_angle * M_PI/180;
 	double vert_just, delta, deltax, deltay;
-	double box_x, box_y;
 	PangoRectangle ink_rect;
 	PangoRectangle logical_rect;
 	PangoLayout *layout;
@@ -842,7 +822,6 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 #ifdef MAP_SYMBOL
 	TBOOLEAN symbol_font_parsed = FALSE;
 #endif /*MAP_SYMBOL*/
-	int baseline_offset;
 
 
 	/* begin by stroking any open path */
@@ -857,7 +836,7 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	if (!strcmp(plot->fontname,"Symbol")) {
 		FPRINTF((stderr,"Parsing a Symbol string\n"));
 		string_utf8 = gp_cairo_convert_symbol_to_unicode(plot, string);
-		safe_strncpy(plot->fontname, gp_cairo_default_font(), sizeof(plot->fontname));
+		strncpy(plot->fontname, gp_cairo_default_font(), sizeof(plot->fontname));
 		symbol_font_parsed = TRUE;
 	} else
 #endif /*MAP_SYMBOL*/
@@ -876,7 +855,7 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 #ifdef MAP_SYMBOL
 	/* restore the Symbol font setting */
 	if (symbol_font_parsed)
-		safe_strncpy(plot->fontname, "Symbol", sizeof(plot->fontname));
+		strncpy(plot->fontname, "Symbol", sizeof(plot->fontname));
 #endif /*MAP_SYMBOL*/
 	pango_font_description_set_size (desc, (int) (plot->fontsize*PANGO_SCALE*plot->oversampling_scale) );
 
@@ -898,14 +877,15 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	/* EAM Dec 2012 - The problem is that avg_vchar is not kept in sync with the	*/
 	/* font size.  It is changed when the set_font command is received, not when	*/
 	/* it is executed in the display list. Try basing off plot->fontsize instead. 	*/
+	/* vert_just = ((double)ink_rect.height/2 +(double)ink_rect.y) / PANGO_SCALE;	*/
+	/* vert_just = avg_vchar/2;							*/
+	vert_just = 0.8 * (float)(plot->fontsize * plot->oversampling_scale);
 
-	baseline_offset = pango_layout_get_baseline(layout) / PANGO_SCALE;
-	vert_just = 0.5 * (float)(plot->fontsize * plot->oversampling_scale);
-	vert_just = baseline_offset - vert_just;
+	x = (double) x1;
+	y = (double) y1;
 
-	arg = plot->text_angle * M_PI/180;
-	x = (double)x1 - vert_just * sin(arg);
-	y = (double)y1 - vert_just * cos(arg);
+	x -= vert_just * sin(arg);
+	y -= vert_just * cos(arg);
 
 	delta = ((double)logical_rect.width/2) / PANGO_SCALE;
 
@@ -930,11 +910,10 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 #endif /* helper point */
 
 	cairo_save (plot->cr);
-	cairo_translate(plot->cr, x, y);
-	cairo_rotate(plot->cr, -arg);
-
 	cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g, plot->color.b,
 				1. - plot->color.alpha);
+	cairo_move_to (plot->cr, x-0.5, y-0.5);
+	cairo_rotate(plot->cr, -arg);
 
 	/* Inform Pango to re-layout the text with the new transformation */
 	pango_cairo_update_layout (plot->cr, layout);
@@ -943,44 +922,36 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	 * Do it by ourselves, or we can get spurious lines on future calls. */
 	cairo_new_path(plot->cr);
 
-	if (in_textbox) {
-		box_rotation = -arg;
-		box_origin_x = x1;
-		box_origin_y = y1;
-		box_y = box_origin_y - vert_just;
-		switch (plot->justify_mode) {
-		case LEFT:
-			box_x = box_origin_x;
-			break;
-		default:
-		case CENTRE:
-			box_x = box_origin_x - delta;
-			break;
-		case RIGHT:
-			box_x = box_origin_x - 2*delta;
-			break;
-		}
+#ifdef EAM_BOXED_TEXT
+	cairo_set_line_width(plot->cr, plot->linewidth*plot->oversampling_scale);
+	cairo_rotate(plot->cr, arg);
+	cairo_translate(plot->cr, x, y);
+	cairo_rotate(plot->cr, -arg);
 
-		/* Update bounding box for boxed label text */
-		pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
+	{
+	PangoRectangle ink, logical;
+	pango_layout_get_pixel_extents (layout, &ink, &logical);
 
-		/* Auto-initialization */
-		if (bounding_box[0] < 0 && bounding_box[1] < 0) {
-		    bounding_box[0] = bounding_box[2] = box_x;
-		    bounding_box[1] = bounding_box[3] = box_y;
-		}
-		if (bounding_box[0] > box_x + ink_rect.x)
-		    bounding_box[0] = box_x + ink_rect.x;
-		if (bounding_box[2] < box_x + ink_rect.x + ink_rect.width)
-		    bounding_box[2] = box_x + ink_rect.x + ink_rect.width;
-		if (bounding_box[1] > box_y + ink_rect.y)
-		    bounding_box[1] = box_y + ink_rect.y;
-		if (bounding_box[3] < box_y + ink_rect.y + ink_rect.height)
-		    bounding_box[3] = box_y + ink_rect.y + ink_rect.height;
+	/* Auto-initialization */
+	if (bounding_box[0] < 0 && bounding_box[1] < 0) {
+	    bounding_box[0] = bounding_box[2] = x;
+	    bounding_box[1] = bounding_box[3] = y;
 	}
 
+	/* Would it look better to use logical bounds rather than ink? */
+	if (bounding_box[0] > x + ink.x)
+	    bounding_box[0] = x + ink.x;
+	if (bounding_box[2] < x + ink.x + ink.width)
+	    bounding_box[2] = x + ink.x + ink.width;
+	if (bounding_box[1] > y + ink.y)
+	    bounding_box[1] = y + ink.y;
+	if (bounding_box[3] < y + ink.y + ink.height)
+	    bounding_box[3] = y + ink.y + ink.height;
+	}
+#endif
+
 	/* free the layout object */
-	g_clear_object (&layout);
+	g_object_unref (layout);
 	cairo_restore (plot->cr);
 }
 
@@ -1123,7 +1094,7 @@ void gp_cairo_draw_point(plot_struct *plot, int x1, int y1, int style)
 		if (style == 14)
 			cairo_fill_preserve(plot->cr);
 		cairo_stroke(plot->cr);
-		break;
+		break;				
 	default :
 		break;
 	}
@@ -1177,8 +1148,8 @@ void gp_cairo_draw_image(plot_struct *plot, unsigned int * image, int x1, int y1
 	image_surface = cairo_image_surface_create_for_data((unsigned char*) image,
 				CAIRO_FORMAT_ARGB32, M, N, 4*M);
 
-	scale_x = (double)M/(double)abs( x2 - x1 );
-	scale_y = (double)N/(double)abs( y2 - y1 );
+	scale_x = (double)M/fabs( x2 - x1 );
+	scale_y = (double)N/fabs( y2 - y1 );
 
 	FPRINTF((stderr,"M %d N %d x1 %d y1 %d\n", M, N, x1, y1));
 	cairo_save( plot->cr );
@@ -1300,7 +1271,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		return;
 
 	FPRINTF((stderr, "enhanced flush str=\"%s\" font=%s op=%d sf=%d wf=%d base=%f os=%d wt=%d sl=%d\n",
-		gp_cairo_enhanced_string,
+		enhanced_text,
 		gp_cairo_enhanced_font,
 		gp_cairo_enhanced_overprint,
 		gp_cairo_enhanced_showflag,
@@ -1320,11 +1291,11 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		enhanced_text_utf8 = gp_cairo_convert_symbol_to_unicode(plot, gp_cairo_enhanced_string);
 
 		if (!strcmp(plot->fontname,"Symbol")) {
-			safe_strncpy(gp_cairo_enhanced_font,
+			strncpy(gp_cairo_enhanced_font,
 				plot->fontname,
 				sizeof(gp_cairo_enhanced_font));
 		} else {
-			safe_strncpy(gp_cairo_enhanced_font,
+			strncpy(gp_cairo_enhanced_font,
 				gp_cairo_default_font(), sizeof(gp_cairo_enhanced_font));
 		}
 		symbol_font_parsed = TRUE;
@@ -1349,11 +1320,8 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_text (save_layout, gp_cairo_save_utf8, -1);
 		pango_layout_set_attributes (save_layout, gp_cairo_enhanced_save_AttrList);
 		pango_layout_get_extents(save_layout, NULL, &save_logical_rect);
-		g_clear_object (&save_layout);
-
+		g_object_unref (save_layout);
 		pango_attr_list_unref( gp_cairo_enhanced_save_AttrList );
-		gp_cairo_enhanced_save_AttrList = NULL;
-
 		/* invert the size, so we will go back to the saved state */
 		save_logical_rect.width = -save_logical_rect.width;
 		/* EAM FIXME:  Zero height necessary but I don't understand why */
@@ -1361,7 +1329,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		/* adding a blank character with the corresponding shape */
 		gp_cairo_add_shape(save_logical_rect,start);
 
-		safe_strncpy(gp_cairo_save_utf8, "", sizeof(gp_cairo_save_utf8));
+		strncpy(gp_cairo_save_utf8, "", sizeof(gp_cairo_save_utf8));
 		gp_cairo_enhanced_restore_now = FALSE;
 		start++;
 	}
@@ -1380,7 +1348,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		else
 			pango_layout_set_attributes (underprinted_layout, gp_cairo_enhanced_underprinted_AttrList);
 		pango_layout_get_extents(underprinted_layout, NULL, &underprinted_logical_rect);
-		g_clear_object (&underprinted_layout);
+		g_object_unref (underprinted_layout);
 
 		/* compute the size of the text to overprint*/
 
@@ -1397,7 +1365,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_font_description (current_layout, current_desc);
 		pango_font_description_free (current_desc);
 		pango_layout_get_extents(current_layout, &current_ink_rect, &current_logical_rect);
-		g_clear_object (&current_layout);
+		g_object_unref (current_layout);
 
 		/* calculate the distance to remove to center the overprinted text */
 		underprinted_logical_rect.width = -(underprinted_logical_rect.width + current_logical_rect.width)/2;
@@ -1406,7 +1374,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		/* adding a blank character with the corresponding shape */
 		gp_cairo_add_shape(underprinted_logical_rect, start);
 
-		safe_strncpy(gp_cairo_underprinted_utf8, "", sizeof(gp_cairo_underprinted_utf8));
+		strncpy(gp_cairo_underprinted_utf8, "", sizeof(gp_cairo_underprinted_utf8));
 		/* increment the position as we added a character */
 		start++;
 	}
@@ -1426,7 +1394,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_text (current_layout, gp_cairo_utf8, -1);
 		pango_layout_set_attributes (current_layout, gp_cairo_enhanced_AttrList);
 		pango_layout_get_extents(current_layout, &current_ink_rect, &current_logical_rect);
-		g_clear_object (&current_layout);
+		g_object_unref (current_layout);
 
 		/* we first compute the size of the text */
 		/* Create a PangoLayout, set the font and text */
@@ -1442,7 +1410,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_font_description_free (hide_desc);
 
 		pango_layout_get_extents(hide_layout, &hide_ink_rect, &hide_logical_rect);
-		g_clear_object (&hide_layout);
+		g_object_unref (hide_layout);
 
 		/* rect.y must be reworked to take previous text into account, which may be smaller */
 		/* hide_logical_rect.y is always initialized at zero, but should be : */
@@ -1471,7 +1439,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_font_description (zerowidth_layout, zerowidth_desc);
 		pango_font_description_free (zerowidth_desc);
 		pango_layout_get_extents(zerowidth_layout, NULL, &zerowidth_logical_rect);
-		g_clear_object (&zerowidth_layout);
+		g_object_unref (zerowidth_layout);
 
 		/* invert the size, so we will go back to the start of the string */
 		zerowidth_logical_rect.width = -zerowidth_logical_rect.width;
@@ -1516,7 +1484,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 
 #ifdef MAP_SYMBOL
 	if (symbol_font_parsed)
-		safe_strncpy(gp_cairo_enhanced_font, "Symbol", sizeof(gp_cairo_enhanced_font));
+		strncpy(gp_cairo_enhanced_font, "Symbol", sizeof(gp_cairo_enhanced_font));
 #endif /* MAP_SYMBOL */
 
 	g_free(enhanced_text_utf8);
@@ -1552,12 +1520,11 @@ void gp_cairo_enhanced_open(plot_struct *plot, char* fontname, double fontsize,
 	}
 
 	if (!gp_cairo_enhanced_opened_string) {
-		/* Strip off Bold or Italic and apply immediately
-		 * Is it really necessary to preserve plot->fontname?
-		 */
+		// Strip off Bold or Italic and apply immediately
+		// Is it really necessary to preserve plot->fontname?
 		char *save_plot_font = strdup(plot->fontname);
 		gp_cairo_set_font(plot, fontname, plot->fontsize);
-		safe_strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
+		strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
 		strcpy(plot->fontname, save_plot_font);
 		free(save_plot_font);
 
@@ -1590,7 +1557,7 @@ void gp_cairo_enhanced_init(plot_struct *plot, int len)
 	gp_cairo_enhanced_overprint = FALSE;
 	gp_cairo_enhanced_showflag = TRUE;
 	gp_cairo_enhanced_fontsize = plot->fontsize*plot->oversampling_scale;
-	safe_strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
+	strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
 	gp_cairo_enhanced_AttrList = pango_attr_list_new();
 }
 
@@ -1599,8 +1566,6 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	PangoRectangle ink_rect, logical_rect;
 	PangoLayout *layout;
 	double vert_just, arg, enh_x, enh_y, delta, deltax, deltay;
-	double box_x, box_y;
-	int baseline_offset;
 
 	/* Create a PangoLayout, set the font and text */
 	layout = gp_cairo_create_layout (plot->cr);
@@ -1610,11 +1575,9 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	pango_layout_set_attributes (layout, gp_cairo_enhanced_AttrList);
 
 	pango_layout_get_extents(layout, &ink_rect, &logical_rect);
-
+	
 	/* NB: See explanatory comments in gp_cairo_draw_text() */
-	baseline_offset = pango_layout_get_baseline(layout) / PANGO_SCALE;
-	vert_just = 0.5 * (float)(plot->fontsize * plot->oversampling_scale);
-	vert_just = baseline_offset - vert_just;
+	vert_just = 0.8 * (float)(plot->fontsize * plot->oversampling_scale);
 
 	arg = plot->text_angle * M_PI/180;
 	enh_x = x - vert_just * sin(arg);
@@ -1639,7 +1602,8 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	}
 
 	cairo_save(plot->cr);
-	cairo_translate(plot->cr, enh_x, enh_y);
+	cairo_move_to (plot->cr, enh_x, enh_y);
+	/* angle in radians */
 	cairo_rotate(plot->cr, -arg);
 
 	cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g, plot->color.b,
@@ -1651,48 +1615,32 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	 * Do it by ourselves, or we can get spurious lines on future calls. */
 	cairo_new_path(plot->cr);
 
-	if (in_textbox) {
-		box_rotation = -arg;
-		box_origin_x = x;
-		box_origin_y = y;
-		box_y = box_origin_y - vert_just;
-		switch (plot->justify_mode) {
-		case LEFT:
-			box_x = box_origin_x;
-			break;
-		default:
-		case CENTRE:
-			box_x = box_origin_x - delta;
-			break;
-		case RIGHT:
-			box_x = box_origin_x - 2*delta;
-			break;
-		}
+#ifdef EAM_BOXED_TEXT
+	/* Update bounding box for boxed label text */
+	pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
 
-		/* Update bounding box for boxed label text */
-		pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
-
-		/* Auto-initialization */
-		if (bounding_box[0] < 0 && bounding_box[1] < 0) {
-		    bounding_box[0] = bounding_box[2] = box_x;
-		    bounding_box[1] = bounding_box[3] = box_y;
-		}
-		if (bounding_box[0] > box_x + ink_rect.x)
-		    bounding_box[0] = box_x + ink_rect.x;
-		if (bounding_box[2] < box_x + ink_rect.x + ink_rect.width)
-		    bounding_box[2] = box_x + ink_rect.x + ink_rect.width;
-		if (bounding_box[1] > box_y + ink_rect.y)
-		    bounding_box[1] = box_y + ink_rect.y;
-		if (bounding_box[3] < box_y + ink_rect.y + ink_rect.height)
-		    bounding_box[3] = box_y + ink_rect.y + ink_rect.height;
+	/* Auto-initialization */
+	if (bounding_box[0] < 0 && bounding_box[1] < 0) {
+	    bounding_box[0] = bounding_box[2] = x;
+	    bounding_box[1] = bounding_box[3] = y;
 	}
 
+	/* Would it look better to use logical bounds rather than ink_rect? */
+	if (bounding_box[0] > enh_x + ink_rect.x)
+	    bounding_box[0] = enh_x + ink_rect.x;
+	if (bounding_box[2] < enh_x + ink_rect.x + ink_rect.width)
+	    bounding_box[2] = enh_x + ink_rect.x + ink_rect.width;
+	if (bounding_box[1] > enh_y + ink_rect.y)
+	    bounding_box[1] = enh_y + ink_rect.y;
+	if (bounding_box[3] < enh_y + ink_rect.y + ink_rect.height)
+	    bounding_box[3] = enh_y + ink_rect.y + ink_rect.height;
+#endif
+	
 	/* free the layout object */
 	pango_attr_list_unref( gp_cairo_enhanced_AttrList );
-	gp_cairo_enhanced_AttrList = NULL;
-	g_clear_object (&layout);
+	g_object_unref (layout);
 	cairo_restore(plot->cr);
-	safe_strncpy(gp_cairo_utf8, "", sizeof(gp_cairo_utf8));
+	strncpy(gp_cairo_utf8, "", sizeof(gp_cairo_utf8));
 	free(gp_cairo_enhanced_string);
 }
 
@@ -1750,6 +1698,7 @@ void gp_cairo_fill(plot_struct *plot, int fillstyle, int fillpar)
 	}
 }
 
+#ifdef EAM_BOXED_TEXT
 void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 {
 	int dx, dy;
@@ -1759,7 +1708,6 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		/* Initialize bounding box for this text string */
 		bounding_box[0] = bounding_box[2] = x;
 		bounding_box[1] = bounding_box[3] = y;
-		in_textbox = TRUE;
 		break;
 
 	case TEXTBOX_OUTLINE:
@@ -1772,44 +1720,30 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		gp_cairo_end_polygon(plot);
 
 		cairo_save(plot->cr);
-
-		/* In progress: textbox rotation
-		 * 1) translate to text origin
-		 * 2) subtract translation from bounding box
-		 * 3) rotate about new origin
-		 * 4) stroke/fill
-		 */
-		cairo_translate(plot->cr, box_origin_x, box_origin_y);
-		cairo_rotate(plot->cr, box_rotation);
-		cairo_translate(plot->cr, -box_origin_x, -box_origin_y);
-
-		dx = 0.25 * bounding_xmargin * (float)(plot->fontsize * plot->oversampling_scale);
-		dy = 0.25 * bounding_ymargin * (float)(plot->fontsize * plot->oversampling_scale);
+		dx = 0.5 * bounding_xmargin * (float)(plot->fontsize * plot->oversampling_scale);
+		dy = 0.5 * bounding_ymargin * (float)(plot->fontsize * plot->oversampling_scale);
 		if (option == TEXTBOX_GREY)
 		    dy = 0;
-		gp_cairo_move(plot,   bounding_box[0]-dx, bounding_box[1]-dy);
-		gp_cairo_vector(plot, bounding_box[0]-dx, bounding_box[3]+dy);
-		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[3]+dy);
-		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[1]-dy);
-		gp_cairo_vector(plot, bounding_box[0]+dx, bounding_box[1]-dy);
+		gp_cairo_move(plot,   bounding_box[0]-dx, bounding_box[1]-dy); 
+		gp_cairo_vector(plot, bounding_box[0]-dx, bounding_box[3]+dy); 
+		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[3]+dy); 
+		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[1]-dy); 
+		gp_cairo_vector(plot, bounding_box[0]+dx, bounding_box[1]-dy); 
 		cairo_close_path(plot->cr);
 		if (option == TEXTBOX_BACKGROUNDFILL) {
-		    cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g,
-					  plot->color.b, 1. - plot->color.alpha);
+		    rgb_color *background = &gp_cairo_colorlist[0];
+		    cairo_set_source_rgb(plot->cr, background->r, background->g, background->b);
 		    cairo_fill(plot->cr);
 		} else if (option == TEXTBOX_GREY) {
 		    cairo_set_source_rgba(plot->cr, 0.75, 0.75, 0.75, 0.50);
 		    cairo_fill(plot->cr);
-		} else {  /* option == TEXTBOX_OUTLINE */
-		    cairo_set_line_width(plot->cr,
-					 plot->linewidth*plot->oversampling_scale);
-		    cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g,
-					  plot->color.b, 1. - plot->color.alpha);
+		} else {
+		    cairo_set_line_width(plot->cr, 1.0*plot->oversampling_scale);
+		    cairo_set_source_rgb(plot->cr,
+			plot->color.r, plot->color.g, plot->color.b);
 		    cairo_stroke(plot->cr);
 		}
-
 		cairo_restore(plot->cr);
-		in_textbox = FALSE;
 		break;
 
 	case TEXTBOX_MARGINS: /* Change the margin between text and box */
@@ -1821,6 +1755,7 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		break;
 	}
 }
+#endif
 
 #define PATTERN_SIZE 8
 
@@ -1936,7 +1871,6 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	PangoRectangle ink_rect;
 	PangoRectangle logical_rect;
 	unsigned int tmp_v_char, tmp_h_char;
-	extern int debug;
 
 	/* Create a PangoLayout, set the font and text */
 	layout = gp_cairo_create_layout (plot->cr);
@@ -1950,7 +1884,7 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	pango_layout_set_font_description (layout, desc);
 	pango_font_description_free (desc);
 	pango_layout_get_extents(layout, &ink_rect, &logical_rect);
-	g_clear_object (&layout);
+	g_object_unref (layout);
 
 	/* we don't use gnuplot_x() and gnuplot_y() in the following
 	 * as the scale should have just been updated to 1.
@@ -1959,16 +1893,6 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	tmp_v_char = (int) ceil( (double) logical_rect.height/PANGO_SCALE) - 1;
 	tmp_h_char = (int) ceil( (double) logical_rect.width/(10*PANGO_SCALE));
 
-	/* Desperation fallback case.
-	 * There have been reports of failure to obtain font metrics.
-	 * We don't know why (pango use of harfbuzz is a suspect).
-	 */
-	if (tmp_v_char <= 1 || tmp_h_char <= 1 || debug == 7) {
-		tmp_h_char = 140 + (plot->fontsize - 10.) * 16.;
-		tmp_v_char = 300 * (plot->fontsize / 10.);
-		fprintf(stderr, "warning: problem determining pango font metrics\n");
-	}
-
 	if (v_char)
 		*v_char = tmp_v_char;
 	if (h_char)
@@ -2023,7 +1947,7 @@ const char* gp_cairo_enhanced_get_fontname(plot_struct *plot)
 const char *
 gp_cairo_default_font(void)
 {
-#ifdef _WIN32
+#ifdef WIN32
 	return "Tahoma";
 #else
 	return "Sans";
