@@ -1,5 +1,5 @@
 /*
- * $Id: wxt_gui.h,v 1.48.2.6 2016/03/22 17:08:36 markisch Exp $
+ * $Id: wxt_gui.h,v 1.48 2014/06/20 22:58:46 sfeam Exp $
  */
 
 /* GNUPLOT - wxt_gui.h */
@@ -81,9 +81,6 @@
 #include <wx/dataobj.h>
 #include <wx/clipbrd.h>
 
-/* Save File dialog */
-#include <wx/filedlg.h>
-
 /* wxImage facility */
 #include <wx/image.h>
 
@@ -102,9 +99,6 @@
 /* wxMemoryInputStream, for the embedded PNG icons */
 #include <wx/mstream.h>
 
-/* Debugging support, required to turn off asserts */
-#include <wx/debug.h>
-
 /* c++ vectors and lists, used to store gnuplot commands */
 #include <vector>
 #include <list>
@@ -182,14 +176,6 @@ extern "C" {
 /* for cairo_t */
 # include <cairo.h>
 
-#ifdef CAIRO_HAS_SVG_SURFACE
-#include <cairo/cairo-svg.h>
-#endif
-
-#ifdef CAIRO_HAS_PDF_SURFACE
-#include <cairo/cairo-pdf.h>
-#endif
-
 # ifdef USE_GTK
 #  include <gdk/gdk.h>
 #  include <gtk/gtk.h>
@@ -240,15 +226,15 @@ public:
 static wxtThread * thread;
 #endif /* WXT_MULTITHREADED */
 
-DECLARE_LOCAL_EVENT_TYPE(wxExitLoopEvent, -1)
-DEFINE_LOCAL_EVENT_TYPE(wxExitLoopEvent)
+DECLARE_EVENT_TYPE(wxExitLoopEvent, -1)
+DEFINE_EVENT_TYPE(wxExitLoopEvent)
 
-DECLARE_LOCAL_EVENT_TYPE(wxCreateWindowEvent, -1)
-DEFINE_LOCAL_EVENT_TYPE(wxCreateWindowEvent)
+DECLARE_EVENT_TYPE(wxCreateWindowEvent, -1)
+DEFINE_EVENT_TYPE(wxCreateWindowEvent)
 
 #ifdef USE_MOUSE
-DECLARE_LOCAL_EVENT_TYPE(wxStatusTextEvent, -1)
-DEFINE_LOCAL_EVENT_TYPE(wxStatusTextEvent)
+DECLARE_EVENT_TYPE(wxStatusTextEvent, -1)
+DEFINE_EVENT_TYPE(wxStatusTextEvent)
 #endif /* USE_MOUSE */
 
 /* Define a new application type, each gui should derive a class from wxApp */
@@ -467,7 +453,6 @@ class wxtConfigDialog : public wxDialog
 	bool persist_setting;
 	bool ctrl_setting;
 	bool toggle_setting;
-	bool redraw_setting;
 	/* rendering_setting :
 	 * 0 = no antialiasing, no oversampling
 	 * 1 = antialiasing, no oversampling
@@ -489,7 +474,6 @@ public:
 	void OnClose( wxCloseEvent& event );
 	void OnSize( wxSizeEvent& event );
 	void OnCopy( wxCommandEvent& event );
-	void OnExport( wxCommandEvent& event );
 #ifdef USE_MOUSE
 	void OnReplot( wxCommandEvent& event );
 	void OnToggleGrid( wxCommandEvent& event );
@@ -509,8 +493,6 @@ public:
 
 	wxtPanel * panel;
 	bool config_displayed;
-	wxToolBar * toolbar;
-
 private:
 	wxtConfigDialog * config_dialog;
 
@@ -522,7 +504,6 @@ private:
 enum {
 /* start at wxID_HIGHEST to avoid collisions */
 Toolbar_CopyToClipboard = wxID_HIGHEST,
-Toolbar_ExportToFile,
 Toolbar_Replot,
 Toolbar_ToggleGrid,
 Toolbar_ZoomPrevious,
@@ -549,6 +530,11 @@ static wxCursor wxt_cursor_right;
 static wxCursor wxt_cursor_rotate;
 static wxCursor wxt_cursor_size;
 
+#ifdef DEBUG
+ /* performance watch */
+ static wxStopWatch sw;
+#endif
+
 /* wxt_abort_init is set to true if there is an error when
  * wxWidgets is initialized, for example if the X server is unreachable.
  * If there has been an error, we should not try to initialize again,
