@@ -1,5 +1,5 @@
 /*
- * $Id: wxt_term.h,v 1.16 2011/11/06 04:06:10 sfeam Exp $
+ * $Id: wxt_term.h,v 1.25.2.3 2014/12/31 07:36:56 markisch Exp $
  */
 
 /* GNUPLOT - wxt_term.h */
@@ -76,7 +76,7 @@ void wxt_pointsize(double ptsize);
 void wxt_image(unsigned int M, unsigned int N, coordval * image, gpiPoint * corner, t_imagecolor color_mode);
 
 # ifdef USE_MOUSE
-int wxt_waitforinput(void);
+int wxt_waitforinput(int);
 void wxt_put_tmptext(int, const char str[]);
 void wxt_set_ruler(int x, int y);
 void wxt_set_cursor(int, int, int);
@@ -91,6 +91,15 @@ void wxt_enhanced_writec(int c);
 void wxt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN widthflag, TBOOLEAN showflag, int overprint);
 
 void wxt_layer(t_termlayer layer);
+void wxt_hypertext(int type, const char *text);
+
+#ifdef EAM_BOXED_TEXT
+void wxt_boxed_text(unsigned int x, unsigned int y, int option);
+#endif
+
+void wxt_modify_plots(unsigned int, int);
+
+void wxt_dashtype(int type, t_dashtype *custom_dash_pattern);
 
 void wxt_raise_terminal_window __PROTO((int));
 void wxt_raise_terminal_group __PROTO((void));
@@ -99,25 +108,31 @@ void wxt_lower_terminal_group __PROTO((void));
 void wxt_close_terminal_window __PROTO((int number));
 void wxt_update_title __PROTO((int number));
 void wxt_update_size __PROTO((int number));
+void wxt_update_position __PROTO((int number));
+TBOOLEAN wxt_active_window_opened(void);
+TBOOLEAN wxt_window_opened(void);
 
 /* state variables shared between wxt.trm and wxt_gui.cpp */
 extern int wxt_window_number;
 extern TBOOLEAN wxt_enhanced_enabled;
-extern TBOOLEAN wxt_dashed;
 extern double wxt_dashlength;
+extern double wxt_lw;
 extern int wxt_background;
 extern rgb_color wxt_rgb_background;
 extern int wxt_persist;
 extern int wxt_raise;
 extern int wxt_ctrl;
 extern int wxt_toggle;
+extern int wxt_redraw;
 extern char wxt_set_fontname[MAX_ID_LEN + 1];
 extern int wxt_set_fontsize;
 extern double wxt_set_fontscale;
-extern int wxt_rounded;
+extern t_linecap wxt_linecap;
 extern char wxt_title[MAX_ID_LEN + 1];
 extern int wxt_width;
 extern int wxt_height;
+extern int wxt_posx;
+extern int wxt_posy;
 
 extern int wxt_axis_mask;
 typedef struct wxt_axis_state_t {
