@@ -1,4 +1,7 @@
-    gnuplot.mouse_version = " 04 March 2018";
+/*
+ * $Id: gnuplot_mouse.js,v 1.22 2013/05/07 22:13:46 sfeam Exp $
+ */
+    gnuplot.mouse_version = " 7 May 2013";
 
 // Mousing code for use with gnuplot's 'canvas' terminal driver.
 // The functions defined here assume that the javascript plot produced by
@@ -116,28 +119,15 @@ gnuplot.mouse_update = function(e)
   gnuplot.ploty = -(gnuplot.mousey - gnuplot.plot_ybot);
 
   // Limit tracking to the interior of the plot
-  if (gnuplot.plotx < 0 || gnuplot.ploty < 0){
-      if (gnuplot.hypertext_list != "undefined" && gnuplot.hypertext_list.length > 0) {
-	  gnuplot.check_hypertext();
-      } 
-      return;
-  }
-  if (gnuplot.mousex > gnuplot.plot_xmax || gnuplot.mousey < gnuplot.plot_ytop){
-      if (gnuplot.hypertext_list != "undefined" && gnuplot.hypertext_list.length > 0) {
-	  gnuplot.check_hypertext();
-      } 
-      return;
-  }
+  if (gnuplot.plotx < 0 || gnuplot.ploty < 0) return;
+  if (gnuplot.mousex > gnuplot.plot_xmax || gnuplot.mousey < gnuplot.plot_ytop) return;
 
   var axis_xmin = (gnuplot.zoomed) ? gnuplot.zoom_axis_xmin : gnuplot.plot_axis_xmin;
   var axis_xmax = (gnuplot.zoomed) ? gnuplot.zoom_axis_xmax : gnuplot.plot_axis_xmax;
   var axis_ymin = (gnuplot.zoomed) ? gnuplot.zoom_axis_ymin : gnuplot.plot_axis_ymin;
   var axis_ymax = (gnuplot.zoomed) ? gnuplot.zoom_axis_ymax : gnuplot.plot_axis_ymax;
 
-    if (gnuplot.plot_logaxis_x < 0) {
-	// nonlinear axis or custom mapping function
-	x = NaN;
-    } else if (gnuplot.plot_logaxis_x != 0) {
+    if (gnuplot.plot_logaxis_x != 0) {
 	x = Math.log(axis_xmax) - Math.log(axis_xmin);
 	x = x * (gnuplot.plotx / (gnuplot.plot_xmax-gnuplot.plot_xmin)) + Math.log(axis_xmin);
 	x = Math.exp(x);
@@ -145,9 +135,7 @@ gnuplot.mouse_update = function(e)
 	x =  axis_xmin + (gnuplot.plotx / (gnuplot.plot_xmax-gnuplot.plot_xmin)) * (axis_xmax - axis_xmin);
     }
 
-    if (gnuplot.plot_logaxis_y < 0) {
-	y = NaN;
-    } else if (gnuplot.plot_logaxis_y != 0) {
+    if (gnuplot.plot_logaxis_y != 0) {
 	y = Math.log(axis_ymax) - Math.log(axis_ymin);
 	y = y * (-gnuplot.ploty / (gnuplot.plot_ytop-gnuplot.plot_ybot)) + Math.log(axis_ymin);
 	y = Math.exp(y);
@@ -182,15 +170,11 @@ gnuplot.mouse_update = function(e)
     label_x = "ang= " + polar.ang.toPrecision(4);
     label_y = "R= " + polar.r.toPrecision(4);
   } else {
-    if (isNaN(x))
-      label_x = "";
-    else if (typeof(gnuplot.plot_timeaxis_x) == "string" && gnuplot.plot_timeaxis_x != "")
+    if (typeof(gnuplot.plot_timeaxis_x) == "string" && gnuplot.plot_timeaxis_x != "")
       label_x = gnuplot.datafmt(x);
     else
       label_x = x.toPrecision(4);
-    if (isNaN(y))
-      label_y = "";
-    else if (typeof(gnuplot.plot_timeaxis_y) == "string" && gnuplot.plot_timeaxis_y != "")
+    if (typeof(gnuplot.plot_timeaxis_y) == "string" && gnuplot.plot_timeaxis_y != "")
       label_y = gnuplot.datafmt(y);
     else
       label_y = y.toPrecision(4);
@@ -221,7 +205,7 @@ gnuplot.mouse_update = function(e)
   }
 
   // See if we are over a hypertext anchor point
-  if (gnuplot.hypertext_list != "undefined" && gnuplot.hypertext_list.length > 0) {
+  if (typeof(gnuplot.hypertext_list != "unknown") && gnuplot.hypertext_list.length > 0) {
     gnuplot.check_hypertext();
   }
 }
@@ -254,27 +238,9 @@ gnuplot.check_hypertext = function()
 		if (len < ll) len = ll;
 	    }
 	}
-
-	// text shift (so it is not covered by cursor)
-	var shiftx = 14;
-
-	if (linkx > gnuplot.plot_term_xmax-(len+8) ){
-	    // position text left of cursor if we are to far in the right hemisphere
-	    // else we might print over the border
-	    linkx = linkx - len;
-	    shiftx = -shiftx;
-	}
-	if (linky > 14*lines.length ){
-	    // position text top of cursor if we are low enough in the plot
-	    // else we might print over the border
-	    linky = linky - 14*lines.length;
-	}
-
 	ctx.fillStyle = "rgba(238,238,238,0.8)"
-	ctx.fillRect(linkx+shiftx, linky+4, len+8, 14*lines.length);
-	for (var l=0; l<lines.length; l++) {
-	    ctx.drawText("sans", 10, linkx+shiftx+4, linky+14+l*14, lines[l]);
-	}
+	ctx.fillRect(linkx+10, linky+4, len+8, 14*lines.length);
+	ctx.drawText("sans", 10, linkx+14, linky+14, text);
 	break;
     }
   } 
@@ -337,19 +303,10 @@ gnuplot.convert_to_polar = function (x,y)
     phi = Math.atan2(y,x);
     if (gnuplot.plot_logaxis_r) 
         r = Math.exp( (x/Math.cos(phi) + Math.log(gnuplot.plot_axis_rmin)/Math.LN10) * Math.LN10);
-    else if (gnuplot.plot_axis_rmin > gnuplot.plot_axis_rmax)
-	r = gnuplot.plot_axis_rmin - x/Math.cos(phi);
     else
-        r = gnuplot.plot_axis_rmin + x/Math.cos(phi);
-    phi = phi * (180./Math.PI);
-    if (gnuplot.polar_sense < 0)
-	phi = -phi;
-    if (gnuplot.polar_theta0 != undefined)
-	phi = phi + gnuplot.polar_theta0;
-    if (phi > 180.)
-	phi = phi - 360.;
+        r = x/Math.cos(phi) + gnuplot.plot_axis_rmin;
+    polar.ang = phi * 180./Math.PI;
     polar.r = r;
-    polar.ang = phi;
     return polar;
 }
 
@@ -370,28 +327,16 @@ gnuplot.saveclick = function (event)
   if (button == "LEFT") {
     ctx.strokeStyle="black";
     ctx.strokeRect(gnuplot.mousex, gnuplot.mousey, 1, 1);
-    if (gnuplot.plot_logaxis_x < 0)
-      label_x = "";
-    else if (typeof(gnuplot.plot_timeaxis_x) == "string" && gnuplot.plot_timeaxis_x != "")
+    if (typeof(gnuplot.plot_timeaxis_x) == "string" && gnuplot.plot_timeaxis_x != "")
       label_x = gnuplot.datafmt(x);
     else
       label_x = x.toPrecision(4);
-    if (gnuplot.plot_logaxis_y < 0)
-      label_y = "";
-    else if (typeof(gnuplot.plot_timeaxis_y) == "string" && gnuplot.plot_timeaxis_y != "")
+    if (typeof(gnuplot.plot_timeaxis_y) == "string" && gnuplot.plot_timeaxis_y != "")
       label_y = gnuplot.datafmt(y);
     else
       label_y = y.toPrecision(4);
     click = " " + label_x + ", " + label_y;
-
-    var len = ctx.measureText("sans", 9, click);
-    if (gnuplot.mousex > gnuplot.plot_term_xmax-(len) ){
-	// draw left of cursor when we are near the right side of the plot
-	len += ctx.measureText("sans", 9, " ");
-	ctx.drawText("sans", 9, gnuplot.mousex-len, gnuplot.mousey, click);
-    } else {
-	ctx.drawText("sans", 9, gnuplot.mousex, gnuplot.mousey, click);
-    }
+    ctx.drawText("sans", 9, gnuplot.mousex, gnuplot.mousey, click);
   }
 
   // Save starting corner of zoom box
